# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/chart.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.artifacts.v1 import chart_pb2 as chalk_dot_artifacts_dot_v1_dot_chart__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.chart.v1 import densetimeserieschart_pb2 as chalk_dot_chart_dot_v1_dot_densetimeserieschart__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/server/v1/chart.proto\x12\x0f\x63halk.server.v1\x1a\x1e\x63halk/artifacts/v1/chart.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a)chalk/chart/v1/densetimeserieschart.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"L\n\x06Series\x12\x16\n\x06points\x18\x01 \x03(\x01R\x06points\x12\x14\n\x05label\x18\x02 \x01(\tR\x05label\x12\x14\n\x05units\x18\x03 \x01(\tR\x05units"t\n\x05\x43hart\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12/\n\x06series\x18\x02 \x03(\x0b\x32\x17.chalk.server.v1.SeriesR\x06series\x12$\n\x0ex_timestamp_ms\x18\x04 \x03(\x03R\x0cxTimestampMs"0\n\x05Point\x12\x19\n\x05value\x18\x01 \x01(\x03H\x00R\x05value\x88\x01\x01:\x02\x18\x01\x42\x08\n\x06_value"l\n\nTimeSeries\x12.\n\x06points\x18\x01 \x03(\x0b\x32\x16.chalk.server.v1.PointR\x06points\x12\x14\n\x05label\x18\x02 \x01(\tR\x05label\x12\x14\n\x05units\x18\x03 \x01(\tR\x05units:\x02\x18\x01"\xd7\x01\n\x0fTimeSeriesChart\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12\x33\n\x06series\x18\x02 \x03(\x0b\x32\x1b.chalk.server.v1.TimeSeriesR\x06series\x12\x35\n\x08x_series\x18\x03 \x03(\x0b\x32\x1a.google.protobuf.TimestampR\x07xSeries\x12>\n\rwindow_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod:\x02\x18\x01"\xbe\x01\n\x11ListChartsFilters\x12P\n\x10link_entity_kind\x18\x01 \x01(\x0e\x32!.chalk.artifacts.v1.ChartLinkKindH\x00R\x0elinkEntityKind\x88\x01\x01\x12-\n\x10linked_entity_id\x18\x02 \x01(\tH\x01R\x0elinkedEntityId\x88\x01\x01\x42\x13\n\x11_link_entity_kindB\x13\n\x11_linked_entity_id"m\n\x12ListChartPageToken\x12@\n\x0e\x63reated_at_hwm\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0c\x63reatedAtHwm\x12\x15\n\x06id_hwm\x18\x02 \x01(\tR\x05idHwm"\xba\x01\n\x11ListChartsRequest\x12\x41\n\x07\x66ilters\x18\x01 \x01(\x0b\x32".chalk.server.v1.ListChartsFiltersH\x00R\x07\x66ilters\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x12"\n\npage_token\x18\x03 \x01(\tH\x02R\tpageToken\x88\x01\x01\x42\n\n\x08_filtersB\x08\n\x06_limitB\r\n\x0b_page_token"\xda\x01\n\x12ListChartsResponse\x12<\n\x06\x63harts\x18\x01 \x03(\x0b\x32 .chalk.artifacts.v1.MetricConfigB\x02\x18\x01R\x06\x63harts\x12\x45\n\x11\x63harts_with_links\x18\x02 \x03(\x0b\x32\x19.chalk.artifacts.v1.ChartR\x0f\x63hartsWithLinks\x12+\n\x0fnext_page_token\x18\x03 \x01(\tH\x00R\rnextPageToken\x88\x01\x01\x42\x12\n\x10_next_page_token"\xd2\x01\n\x17GetChartSnapshotRequest\x12\x45\n\rmetric_config\x18\x01 \x01(\x0b\x32 .chalk.artifacts.v1.MetricConfigR\x0cmetricConfig\x12\x39\n\nstart_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime"\xcf\x01\n\x18GetChartSnapshotResponse\x12<\n\x06\x63harts\x18\x01 \x03(\x0b\x32$.chalk.chart.v1.DenseTimeSeriesChartR\x06\x63harts\x12\x35\n\x08x_series\x18\x02 \x03(\x0b\x32\x1a.google.protobuf.TimestampR\x07xSeries\x12>\n\rwindow_period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod2\xd9\x01\n\rChartsService\x12Z\n\nListCharts\x12".chalk.server.v1.ListChartsRequest\x1a#.chalk.server.v1.ListChartsResponse"\x03\x80}\x06\x12l\n\x10GetChartSnapshot\x12(.chalk.server.v1.GetChartSnapshotRequest\x1a).chalk.server.v1.GetChartSnapshotResponse"\x03\x80}\x06\x42\x93\x01\n\x13\x63om.chalk.server.v1B\nChartProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.chart_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\nChartProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_POINT"]._options = None
    _globals["_POINT"]._serialized_options = b"\030\001"
    _globals["_TIMESERIES"]._options = None
    _globals["_TIMESERIES"]._serialized_options = b"\030\001"
    _globals["_TIMESERIESCHART"]._options = None
    _globals["_TIMESERIESCHART"]._serialized_options = b"\030\001"
    _globals["_LISTCHARTSRESPONSE"].fields_by_name["charts"]._options = None
    _globals["_LISTCHARTSRESPONSE"].fields_by_name["charts"]._serialized_options = b"\030\001"
    _globals["_CHARTSSERVICE"].methods_by_name["ListCharts"]._options = None
    _globals["_CHARTSSERVICE"].methods_by_name["ListCharts"]._serialized_options = b"\200}\006"
    _globals["_CHARTSSERVICE"].methods_by_name["GetChartSnapshot"]._options = None
    _globals["_CHARTSSERVICE"].methods_by_name["GetChartSnapshot"]._serialized_options = b"\200}\006"
    _globals["_SERIES"]._serialized_start = 221
    _globals["_SERIES"]._serialized_end = 297
    _globals["_CHART"]._serialized_start = 299
    _globals["_CHART"]._serialized_end = 415
    _globals["_POINT"]._serialized_start = 417
    _globals["_POINT"]._serialized_end = 465
    _globals["_TIMESERIES"]._serialized_start = 467
    _globals["_TIMESERIES"]._serialized_end = 575
    _globals["_TIMESERIESCHART"]._serialized_start = 578
    _globals["_TIMESERIESCHART"]._serialized_end = 793
    _globals["_LISTCHARTSFILTERS"]._serialized_start = 796
    _globals["_LISTCHARTSFILTERS"]._serialized_end = 986
    _globals["_LISTCHARTPAGETOKEN"]._serialized_start = 988
    _globals["_LISTCHARTPAGETOKEN"]._serialized_end = 1097
    _globals["_LISTCHARTSREQUEST"]._serialized_start = 1100
    _globals["_LISTCHARTSREQUEST"]._serialized_end = 1286
    _globals["_LISTCHARTSRESPONSE"]._serialized_start = 1289
    _globals["_LISTCHARTSRESPONSE"]._serialized_end = 1507
    _globals["_GETCHARTSNAPSHOTREQUEST"]._serialized_start = 1510
    _globals["_GETCHARTSNAPSHOTREQUEST"]._serialized_end = 1720
    _globals["_GETCHARTSNAPSHOTRESPONSE"]._serialized_start = 1723
    _globals["_GETCHARTSNAPSHOTRESPONSE"]._serialized_end = 1930
    _globals["_CHARTSSERVICE"]._serialized_start = 1933
    _globals["_CHARTSSERVICE"]._serialized_end = 2150
# @@protoc_insertion_point(module_scope)
