# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/scheduled_query_run.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.server.v1 import offline_queries_pb2 as chalk_dot_server_dot_v1_dot_offline__queries__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n)chalk/server/v1/scheduled_query_run.proto\x12\x0f\x63halk.server.v1\x1a%chalk/server/v1/offline_queries.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xbf\x04\n\x11ScheduledQueryRun\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12(\n\rdeployment_id\x18\x03 \x01(\tH\x00R\x0c\x64\x65ploymentId\x88\x01\x01\x12\x1a\n\x06run_id\x18\x04 \x01(\tH\x01R\x05runId\x88\x01\x01\x12"\n\rcron_query_id\x18\x05 \x01(\x03R\x0b\x63ronQueryId\x12\x33\n\x16\x63ron_query_schedule_id\x18\x06 \x01(\x03R\x13\x63ronQueryScheduleId\x12\x1b\n\tcron_name\x18\x07 \x01(\tR\x08\x63ronName\x12-\n\x10gcr_execution_id\x18\x08 \x01(\tH\x02R\x0egcrExecutionId\x88\x01\x01\x12%\n\x0cgcr_job_name\x18\t \x01(\tH\x03R\ngcrJobName\x88\x01\x01\x12(\n\x10offline_query_id\x18\n \x01(\tR\x0eofflineQueryId\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAtB\x10\n\x0e_deployment_idB\t\n\x07_run_idB\x13\n\x11_gcr_execution_idB\x0f\n\r_gcr_job_name"\xa7\x01\n\x1bGetScheduledQueryRunRequest\x12\x17\n\x06run_id\x18\x01 \x01(\x03H\x00R\x05runId\x12*\n\x10offline_query_id\x18\x02 \x01(\tH\x00R\x0eofflineQueryId\x12\x35\n\x08get_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskR\x07getMaskB\x0c\n\nidentifier"\xd1\x01\n\x1cGetScheduledQueryRunResponse\x12R\n\x13scheduled_query_run\x18\x01 \x01(\x0b\x32".chalk.server.v1.ScheduledQueryRunR\x11scheduledQueryRun\x12K\n\roffline_query\x18\x02 \x01(\x0b\x32!.chalk.server.v1.OfflineQueryMetaH\x00R\x0cofflineQuery\x88\x01\x01\x42\x10\n\x0e_offline_queryB\x9f\x01\n\x13\x63om.chalk.server.v1B\x16ScheduledQueryRunProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.scheduled_query_run_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\026ScheduledQueryRunProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_SCHEDULEDQUERYRUN"]._serialized_start = 169
    _globals["_SCHEDULEDQUERYRUN"]._serialized_end = 744
    _globals["_GETSCHEDULEDQUERYRUNREQUEST"]._serialized_start = 747
    _globals["_GETSCHEDULEDQUERYRUNREQUEST"]._serialized_end = 914
    _globals["_GETSCHEDULEDQUERYRUNRESPONSE"]._serialized_start = 917
    _globals["_GETSCHEDULEDQUERYRUNRESPONSE"]._serialized_end = 1126
# @@protoc_insertion_point(module_scope)
