# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/webhook.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1d\x63halk/server/v1/webhook.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x87\x03\n\x07Webhook\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12\x17\n\x07team_id\x18\x03 \x01(\tR\x06teamId\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12\x10\n\x03url\x18\x05 \x01(\tR\x03url\x12$\n\rsubscriptions\x18\x06 \x03(\tR\rsubscriptions\x12\x1b\n\x06secret\x18\x07 \x01(\tH\x00R\x06secret\x88\x01\x01\x12\x36\n\x07headers\x18\x08 \x01(\x0b\x32\x17.google.protobuf.StructH\x01R\x07headers\x88\x01\x01\x12\x39\n\ncreated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAtB\t\n\x07_secretB\n\n\x08_headers"\xf5\x01\n\x14\x43reateWebhookRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x10\n\x03url\x18\x03 \x01(\tR\x03url\x12$\n\rsubscriptions\x18\x04 \x03(\tR\rsubscriptions\x12\x1b\n\x06secret\x18\x05 \x01(\tH\x00R\x06secret\x88\x01\x01\x12\x36\n\x07headers\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x01R\x07headers\x88\x01\x01\x42\t\n\x07_secretB\n\n\x08_headers"K\n\x15\x43reateWebhookResponse\x12\x32\n\x07webhook\x18\x01 \x01(\x0b\x32\x18.chalk.server.v1.WebhookR\x07webhook"\xf9\x01\n\x14UpdateWebhookRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x15\n\x03url\x18\x03 \x01(\tH\x01R\x03url\x88\x01\x01\x12$\n\rsubscriptions\x18\x04 \x03(\tR\rsubscriptions\x12\x1b\n\x06secret\x18\x05 \x01(\tH\x02R\x06secret\x88\x01\x01\x12\x36\n\x07headers\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x03R\x07headers\x88\x01\x01\x42\x07\n\x05_nameB\x06\n\x04_urlB\t\n\x07_secretB\n\n\x08_headers"K\n\x15UpdateWebhookResponse\x12\x32\n\x07webhook\x18\x01 \x01(\x0b\x32\x18.chalk.server.v1.WebhookR\x07webhook"&\n\x14\x44\x65leteWebhookRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"K\n\x15\x44\x65leteWebhookResponse\x12\x32\n\x07webhook\x18\x01 \x01(\x0b\x32\x18.chalk.server.v1.WebhookR\x07webhook"#\n\x11GetWebhookRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"H\n\x12GetWebhookResponse\x12\x32\n\x07webhook\x18\x01 \x01(\x0b\x32\x18.chalk.server.v1.WebhookR\x07webhook"<\n\x13ListWebhooksRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId"L\n\x14ListWebhooksResponse\x12\x34\n\x08webhooks\x18\x01 \x03(\x0b\x32\x18.chalk.server.v1.WebhookR\x08webhooks2\x83\x04\n\x0eWebhookService\x12\x63\n\rCreateWebhook\x12%.chalk.server.v1.CreateWebhookRequest\x1a&.chalk.server.v1.CreateWebhookResponse"\x03\x80}\x05\x12\x63\n\rUpdateWebhook\x12%.chalk.server.v1.UpdateWebhookRequest\x1a&.chalk.server.v1.UpdateWebhookResponse"\x03\x80}\x05\x12\x63\n\rDeleteWebhook\x12%.chalk.server.v1.DeleteWebhookRequest\x1a&.chalk.server.v1.DeleteWebhookResponse"\x03\x80}\x05\x12]\n\nGetWebhook\x12".chalk.server.v1.GetWebhookRequest\x1a#.chalk.server.v1.GetWebhookResponse"\x06\x90\x02\x01\x80}\x06\x12\x63\n\x0cListWebhooks\x12$.chalk.server.v1.ListWebhooksRequest\x1a%.chalk.server.v1.ListWebhooksResponse"\x06\x90\x02\x01\x80}\x06\x42\x95\x01\n\x13\x63om.chalk.server.v1B\x0cWebhookProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.webhook_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\014WebhookProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_WEBHOOKSERVICE"].methods_by_name["CreateWebhook"]._options = None
    _globals["_WEBHOOKSERVICE"].methods_by_name["CreateWebhook"]._serialized_options = b"\200}\005"
    _globals["_WEBHOOKSERVICE"].methods_by_name["UpdateWebhook"]._options = None
    _globals["_WEBHOOKSERVICE"].methods_by_name["UpdateWebhook"]._serialized_options = b"\200}\005"
    _globals["_WEBHOOKSERVICE"].methods_by_name["DeleteWebhook"]._options = None
    _globals["_WEBHOOKSERVICE"].methods_by_name["DeleteWebhook"]._serialized_options = b"\200}\005"
    _globals["_WEBHOOKSERVICE"].methods_by_name["GetWebhook"]._options = None
    _globals["_WEBHOOKSERVICE"].methods_by_name["GetWebhook"]._serialized_options = b"\220\002\001\200}\006"
    _globals["_WEBHOOKSERVICE"].methods_by_name["ListWebhooks"]._options = None
    _globals["_WEBHOOKSERVICE"].methods_by_name["ListWebhooks"]._serialized_options = b"\220\002\001\200}\006"
    _globals["_WEBHOOK"]._serialized_start = 147
    _globals["_WEBHOOK"]._serialized_end = 538
    _globals["_CREATEWEBHOOKREQUEST"]._serialized_start = 541
    _globals["_CREATEWEBHOOKREQUEST"]._serialized_end = 786
    _globals["_CREATEWEBHOOKRESPONSE"]._serialized_start = 788
    _globals["_CREATEWEBHOOKRESPONSE"]._serialized_end = 863
    _globals["_UPDATEWEBHOOKREQUEST"]._serialized_start = 866
    _globals["_UPDATEWEBHOOKREQUEST"]._serialized_end = 1115
    _globals["_UPDATEWEBHOOKRESPONSE"]._serialized_start = 1117
    _globals["_UPDATEWEBHOOKRESPONSE"]._serialized_end = 1192
    _globals["_DELETEWEBHOOKREQUEST"]._serialized_start = 1194
    _globals["_DELETEWEBHOOKREQUEST"]._serialized_end = 1232
    _globals["_DELETEWEBHOOKRESPONSE"]._serialized_start = 1234
    _globals["_DELETEWEBHOOKRESPONSE"]._serialized_end = 1309
    _globals["_GETWEBHOOKREQUEST"]._serialized_start = 1311
    _globals["_GETWEBHOOKREQUEST"]._serialized_end = 1346
    _globals["_GETWEBHOOKRESPONSE"]._serialized_start = 1348
    _globals["_GETWEBHOOKRESPONSE"]._serialized_end = 1420
    _globals["_LISTWEBHOOKSREQUEST"]._serialized_start = 1422
    _globals["_LISTWEBHOOKSREQUEST"]._serialized_end = 1482
    _globals["_LISTWEBHOOKSRESPONSE"]._serialized_start = 1484
    _globals["_LISTWEBHOOKSRESPONSE"]._serialized_end = 1560
    _globals["_WEBHOOKSERVICE"]._serialized_start = 1563
    _globals["_WEBHOOKSERVICE"]._serialized_end = 2078
# @@protoc_insertion_point(module_scope)
