# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/auth.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import audit_pb2 as chalk_dot_auth_dot_v1_dot_audit__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.server.v1 import link_pb2 as chalk_dot_server_dot_v1_dot_link__pb2
from chalk._gen.chalk.utils.v1 import sensitive_pb2 as chalk_dot_utils_dot_v1_dot_sensitive__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1a\x63halk/server/v1/auth.proto\x12\x0f\x63halk.server.v1\x1a\x19\x63halk/auth/v1/audit.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1a\x63halk/server/v1/link.proto\x1a\x1e\x63halk/utils/v1/sensitive.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x9d\x01\n\x0fGetTokenRequest\x12\x1b\n\tclient_id\x18\x01 \x01(\tR\x08\x63lientId\x12)\n\rclient_secret\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\x0c\x63lientSecret\x12\x1d\n\ngrant_type\x18\x03 \x01(\tR\tgrantType\x12\x19\n\x05scope\x18\x04 \x01(\tH\x00R\x05scope\x88\x01\x01\x42\x08\n\x06_scope"\xf7\x05\n\x10GetTokenResponse\x12\'\n\x0c\x61\x63\x63\x65ss_token\x18\x01 \x01(\tB\x04\xd8\xa1\'\x01R\x0b\x61\x63\x63\x65ssToken\x12\x1d\n\ntoken_type\x18\x02 \x01(\tR\ttokenType\x12\x1d\n\nexpires_in\x18\x03 \x01(\x05R\texpiresIn\x12\x39\n\nexpires_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\texpiresAt\x12\x1d\n\napi_server\x18\x05 \x01(\tR\tapiServer\x12\x34\n\x13primary_environment\x18\x06 \x01(\tH\x00R\x12primaryEnvironment\x88\x01\x01\x12H\n\x07\x65ngines\x18\x07 \x03(\x0b\x32..chalk.server.v1.GetTokenResponse.EnginesEntryR\x07\x65ngines\x12U\n\x0cgrpc_engines\x18\x08 \x03(\x0b\x32\x32.chalk.server.v1.GetTokenResponse.GrpcEnginesEntryR\x0bgrpcEngines\x12o\n\x16\x65nvironment_id_to_name\x18\t \x03(\x0b\x32:.chalk.server.v1.GetTokenResponse.EnvironmentIdToNameEntryR\x13\x65nvironmentIdToName\x1a:\n\x0c\x45nginesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a>\n\x10GrpcEnginesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x46\n\x18\x45nvironmentIdToNameEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x16\n\x14_primary_environment"j\n\x18UpdateLinkSessionRequest\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12\x17\n\x07user_id\x18\x02 \x01(\tR\x06userId\x12\x1d\n\nsession_id\x18\x03 \x01(\tR\tsessionId"+\n\x19UpdateLinkSessionResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id2\xbf\x03\n\x0b\x41uthService\x12Z\n\x08GetToken\x12 .chalk.server.v1.GetTokenRequest\x1a!.chalk.server.v1.GetTokenResponse"\t\x80}\x01\x8a\xd3\x0e\x02\x08\x01\x12u\n\x11\x43reateLinkSession\x12).chalk.server.v1.CreateLinkSessionRequest\x1a*.chalk.server.v1.CreateLinkSessionResponse"\t\x80}\x01\x8a\xd3\x0e\x02\x08\x01\x12\x66\n\x0eGetLinkSession\x12&.chalk.server.v1.GetLinkSessionRequest\x1a\'.chalk.server.v1.GetLinkSessionResponse"\x03\x80}\x01\x12u\n\x11UpdateLinkSession\x12).chalk.server.v1.UpdateLinkSessionRequest\x1a*.chalk.server.v1.UpdateLinkSessionResponse"\t\x80}\x01\x8a\xd3\x0e\x02\x08\x01\x42\x92\x01\n\x13\x63om.chalk.server.v1B\tAuthProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.auth_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\tAuthProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_GETTOKENREQUEST"].fields_by_name["client_secret"]._options = None
    _globals["_GETTOKENREQUEST"].fields_by_name["client_secret"]._serialized_options = b"\330\241'\001"
    _globals["_GETTOKENRESPONSE_ENGINESENTRY"]._options = None
    _globals["_GETTOKENRESPONSE_ENGINESENTRY"]._serialized_options = b"8\001"
    _globals["_GETTOKENRESPONSE_GRPCENGINESENTRY"]._options = None
    _globals["_GETTOKENRESPONSE_GRPCENGINESENTRY"]._serialized_options = b"8\001"
    _globals["_GETTOKENRESPONSE_ENVIRONMENTIDTONAMEENTRY"]._options = None
    _globals["_GETTOKENRESPONSE_ENVIRONMENTIDTONAMEENTRY"]._serialized_options = b"8\001"
    _globals["_GETTOKENRESPONSE"].fields_by_name["access_token"]._options = None
    _globals["_GETTOKENRESPONSE"].fields_by_name["access_token"]._serialized_options = b"\330\241'\001"
    _globals["_AUTHSERVICE"].methods_by_name["GetToken"]._options = None
    _globals["_AUTHSERVICE"].methods_by_name["GetToken"]._serialized_options = b"\200}\001\212\323\016\002\010\001"
    _globals["_AUTHSERVICE"].methods_by_name["CreateLinkSession"]._options = None
    _globals["_AUTHSERVICE"].methods_by_name[
        "CreateLinkSession"
    ]._serialized_options = b"\200}\001\212\323\016\002\010\001"
    _globals["_AUTHSERVICE"].methods_by_name["GetLinkSession"]._options = None
    _globals["_AUTHSERVICE"].methods_by_name["GetLinkSession"]._serialized_options = b"\200}\001"
    _globals["_AUTHSERVICE"].methods_by_name["UpdateLinkSession"]._options = None
    _globals["_AUTHSERVICE"].methods_by_name[
        "UpdateLinkSession"
    ]._serialized_options = b"\200}\001\212\323\016\002\010\001"
    _globals["_GETTOKENREQUEST"]._serialized_start = 235
    _globals["_GETTOKENREQUEST"]._serialized_end = 392
    _globals["_GETTOKENRESPONSE"]._serialized_start = 395
    _globals["_GETTOKENRESPONSE"]._serialized_end = 1154
    _globals["_GETTOKENRESPONSE_ENGINESENTRY"]._serialized_start = 936
    _globals["_GETTOKENRESPONSE_ENGINESENTRY"]._serialized_end = 994
    _globals["_GETTOKENRESPONSE_GRPCENGINESENTRY"]._serialized_start = 996
    _globals["_GETTOKENRESPONSE_GRPCENGINESENTRY"]._serialized_end = 1058
    _globals["_GETTOKENRESPONSE_ENVIRONMENTIDTONAMEENTRY"]._serialized_start = 1060
    _globals["_GETTOKENRESPONSE_ENVIRONMENTIDTONAMEENTRY"]._serialized_end = 1130
    _globals["_UPDATELINKSESSIONREQUEST"]._serialized_start = 1156
    _globals["_UPDATELINKSESSIONREQUEST"]._serialized_end = 1262
    _globals["_UPDATELINKSESSIONRESPONSE"]._serialized_start = 1264
    _globals["_UPDATELINKSESSIONRESPONSE"]._serialized_end = 1307
    _globals["_AUTHSERVICE"]._serialized_start = 1310
    _globals["_AUTHSERVICE"]._serialized_end = 1757
# @@protoc_insertion_point(module_scope)
