# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/monitoring.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import audit_pb2 as chalk_dot_auth_dot_v1_dot_audit__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.utils.v1 import sensitive_pb2 as chalk_dot_utils_dot_v1_dot_sensitive__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n chalk/server/v1/monitoring.proto\x12\x0f\x63halk.server.v1\x1a\x19\x63halk/auth/v1/audit.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1e\x63halk/utils/v1/sensitive.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xd3\x02\n\x17PagerDutyEventV2Payload\x12\x18\n\x07summary\x18\x01 \x01(\tR\x07summary\x12=\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\ttimestamp\x88\x01\x01\x12>\n\x08severity\x18\x03 \x01(\x0e\x32".chalk.server.v1.PagerDutySeverityR\x08severity\x12\x16\n\x06source\x18\x04 \x01(\tR\x06source\x12!\n\tcomponent\x18\x05 \x01(\tH\x01R\tcomponent\x88\x01\x01\x12\x19\n\x05group\x18\x06 \x01(\tH\x02R\x05group\x88\x01\x01\x12\x19\n\x05\x63lass\x18\x07 \x01(\tH\x03R\x05\x63lass\x88\x01\x01\x42\x0c\n\n_timestampB\x0c\n\n_componentB\x08\n\x06_groupB\x08\n\x06_class"L\n\x14PagerDutyEventV2Link\x12\x12\n\x04href\x18\x01 \x01(\tR\x04href\x12\x17\n\x04text\x18\x02 \x01(\tH\x00R\x04text\x88\x01\x01\x42\x07\n\x05_text"j\n\x15PagerDutyEventV2Image\x12\x10\n\x03src\x18\x01 \x01(\tR\x03src\x12\x17\n\x04href\x18\x02 \x01(\tH\x00R\x04href\x88\x01\x01\x12\x15\n\x03\x61lt\x18\x03 \x01(\tH\x01R\x03\x61lt\x88\x01\x01\x42\x07\n\x05_hrefB\x06\n\x04_alt"\xc9\x03\n\x10PagerDutyEventV2\x12\x42\n\x07payload\x18\x01 \x01(\x0b\x32(.chalk.server.v1.PagerDutyEventV2PayloadR\x07payload\x12\x1f\n\x0brouting_key\x18\x02 \x01(\tR\nroutingKey\x12H\n\x0c\x65vent_action\x18\x03 \x01(\x0e\x32%.chalk.server.v1.PagerDutyEventActionR\x0b\x65ventAction\x12 \n\tdedup_key\x18\x04 \x01(\tH\x00R\x08\x64\x65\x64upKey\x88\x01\x01\x12\x1b\n\x06\x63lient\x18\x05 \x01(\tH\x01R\x06\x63lient\x88\x01\x01\x12"\n\nclient_url\x18\x06 \x01(\tH\x02R\tclientUrl\x88\x01\x01\x12;\n\x05links\x18\x07 \x03(\x0b\x32%.chalk.server.v1.PagerDutyEventV2LinkR\x05links\x12>\n\x06images\x18\x08 \x03(\x0b\x32&.chalk.server.v1.PagerDutyEventV2ImageR\x06imagesB\x0c\n\n_dedup_keyB\t\n\x07_clientB\r\n\x0b_client_url"\xa5\x01\n\x14PagerDutyIntegration\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x18\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\x08R\x07\x64\x65\x66\x61ult\x12\x1a\n\x05token\x18\x04 \x01(\tB\x04\xd8\xa1\'\x01R\x05token\x12%\n\x0e\x65nvironment_id\x18\x05 \x01(\tR\renvironmentIdB\x07\n\x05_name"\x9c\x02\n\x15IncidentIoIntegration\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1a\n\x05token\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\x05token\x12%\n\x0e\x65nvironment_id\x18\x03 \x01(\tR\renvironmentId\x12\x17\n\x04name\x18\x04 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1f\n\tsource_id\x18\x05 \x01(\tB\x02\x18\x01R\x08sourceId\x12%\n\x0csource_token\x18\x06 \x01(\tB\x02\x18\x01R\x0bsourceToken\x12\x1f\n\x0bseverity_id\x18\x07 \x01(\tR\nseverityId\x12%\n\x0eseverity_token\x18\x08 \x01(\tR\rseverityTokenB\x07\n\x05_name"\xa1\x02\n\x11IncidentIoEventV2\x12\x19\n\x08route_id\x18\x01 \x01(\tR\x07routeId\x12%\n\x0broute_token\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\nrouteToken\x12 \n\tdedup_key\x18\x03 \x01(\tH\x00R\x08\x64\x65\x64upKey\x88\x01\x01\x12"\n\nsource_url\x18\x04 \x01(\tH\x01R\tsourceUrl\x88\x01\x01\x12%\n\x0b\x64\x65scription\x18\x05 \x01(\tH\x02R\x0b\x64\x65scription\x88\x01\x01\x12\x16\n\x06status\x18\x06 \x01(\tR\x06status\x12\x14\n\x05title\x18\x07 \x01(\tR\x05title:\x02\x18\x01\x42\x0c\n\n_dedup_keyB\r\n\x0b_source_urlB\x0e\n\x0c_description"1\n\x1fTestPagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"<\n\x1eGetPagerDutyIntegrationRequest\x12\x13\n\x02id\x18\x01 \x01(\tH\x00R\x02id\x88\x01\x01\x42\x05\n\x03_id"j\n\x1fGetPagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x01 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"T\n TestPagerDutyIntegrationResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message"X\n\x1e\x41\x64\x64PagerDutyIntegrationRequest\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x14\n\x05token\x18\x03 \x01(\tR\x05tokenB\x07\n\x05_name"j\n\x1f\x41\x64\x64PagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x01 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"3\n!DeletePagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"$\n"DeletePagerDutyIntegrationResponse"\xab\x01\n!UpdatePagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1d\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\x08H\x01R\x07\x64\x65\x66\x61ult\x88\x01\x01\x12\x1f\n\x05token\x18\x04 \x01(\tB\x04\xd8\xa1\'\x01H\x02R\x05token\x88\x01\x01\x42\x07\n\x05_nameB\n\n\x08_defaultB\x08\n\x06_token"m\n"UpdatePagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x02 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"7\n%SetDefaultPagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"8\n&SetDefaultPagerDutyIntegrationResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"$\n"GetAllPagerDutyIntegrationsRequest"p\n#GetAllPagerDutyIntegrationsResponse\x12I\n\x0cintegrations\x18\x01 \x03(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0cintegrations"2\n TestIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"U\n!TestIncidentIoIntegrationResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message"1\n\x1fGetIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"l\n GetIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"\x93\x01\n\x1f\x41\x64\x64IncidentIoIntegrationRequest\x12.\n\x10integration_name\x18\x01 \x01(\tH\x00R\x0fintegrationName\x88\x01\x01\x12+\n\x11integration_token\x18\x03 \x01(\tR\x10integrationTokenB\x13\n\x11_integration_name"l\n AddIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"4\n"DeleteIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"%\n#DeleteIncidentIoIntegrationResponse"\x81\x01\n"UpdateIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1f\n\x05token\x18\x03 \x01(\tB\x04\xd8\xa1\'\x01H\x01R\x05token\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_token"o\n#UpdateIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"%\n#GetAllIncidentIoIntegrationsRequest"r\n$GetAllIncidentIoIntegrationsResponse\x12J\n\x0cintegrations\x18\x01 \x03(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0cintegrations*\xb8\x01\n\x11PagerDutySeverity\x12#\n\x1fPAGER_DUTY_SEVERITY_UNSPECIFIED\x10\x00\x12\x1c\n\x18PAGER_DUTY_SEVERITY_INFO\x10\x01\x12\x1f\n\x1bPAGER_DUTY_SEVERITY_WARNING\x10\x02\x12\x1d\n\x19PAGER_DUTY_SEVERITY_ERROR\x10\x03\x12 \n\x1cPAGER_DUTY_SEVERITY_CRITICAL\x10\x04*\xb2\x01\n\x14PagerDutyEventAction\x12\'\n#PAGER_DUTY_EVENT_ACTION_UNSPECIFIED\x10\x00\x12#\n\x1fPAGER_DUTY_EVENT_ACTION_TRIGGER\x10\x01\x12\'\n#PAGER_DUTY_EVENT_ACTION_ACKNOWLEDGE\x10\x02\x12#\n\x1fPAGER_DUTY_EVENT_ACTION_RESOLVE\x10\x03\x32\xef\x0e\n\x11MonitoringService\x12\x84\x01\n\x18TestPagerDutyIntegration\x12\x30.chalk.server.v1.TestPagerDutyIntegrationRequest\x1a\x31.chalk.server.v1.TestPagerDutyIntegrationResponse"\x03\x80}\x06\x12\x87\x01\n\x17\x41\x64\x64PagerDutyIntegration\x12/.chalk.server.v1.AddPagerDutyIntegrationRequest\x1a\x30.chalk.server.v1.AddPagerDutyIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x90\x01\n\x1a\x44\x65letePagerDutyIntegration\x12\x32.chalk.server.v1.DeletePagerDutyIntegrationRequest\x1a\x33.chalk.server.v1.DeletePagerDutyIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x9f\x01\n\x1eSetDefaultPagerDutyIntegration\x12\x36.chalk.server.v1.SetDefaultPagerDutyIntegrationRequest\x1a\x37.chalk.server.v1.SetDefaultPagerDutyIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1aUpdatePagerDutyIntegration\x12\x32.chalk.server.v1.UpdatePagerDutyIntegrationRequest\x1a\x33.chalk.server.v1.UpdatePagerDutyIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x90\x01\n\x1bGetAllPagerDutyIntegrations\x12\x33.chalk.server.v1.GetAllPagerDutyIntegrationsRequest\x1a\x34.chalk.server.v1.GetAllPagerDutyIntegrationsResponse"\x06\x90\x02\x01\x80}\x06\x12\x84\x01\n\x17GetPagerDutyIntegration\x12/.chalk.server.v1.GetPagerDutyIntegrationRequest\x1a\x30.chalk.server.v1.GetPagerDutyIntegrationResponse"\x06\x90\x02\x01\x80}\x06\x12\x87\x01\n\x19TestIncidentIoIntegration\x12\x31.chalk.server.v1.TestIncidentIoIntegrationRequest\x1a\x32.chalk.server.v1.TestIncidentIoIntegrationResponse"\x03\x80}\x06\x12\x8a\x01\n\x18\x41\x64\x64IncidentIoIntegration\x12\x30.chalk.server.v1.AddIncidentIoIntegrationRequest\x1a\x31.chalk.server.v1.AddIncidentIoIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1b\x44\x65leteIncidentIoIntegration\x12\x33.chalk.server.v1.DeleteIncidentIoIntegrationRequest\x1a\x34.chalk.server.v1.DeleteIncidentIoIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x96\x01\n\x1bUpdateIncidentIoIntegration\x12\x33.chalk.server.v1.UpdateIncidentIoIntegrationRequest\x1a\x34.chalk.server.v1.UpdateIncidentIoIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1cGetAllIncidentIoIntegrations\x12\x34.chalk.server.v1.GetAllIncidentIoIntegrationsRequest\x1a\x35.chalk.server.v1.GetAllIncidentIoIntegrationsResponse"\x06\x90\x02\x01\x80}\x06\x12\x87\x01\n\x18GetIncidentIoIntegration\x12\x30.chalk.server.v1.GetIncidentIoIntegrationRequest\x1a\x31.chalk.server.v1.GetIncidentIoIntegrationResponse"\x06\x90\x02\x01\x80}\x06\x42\x98\x01\n\x13\x63om.chalk.server.v1B\x0fMonitoringProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.monitoring_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\017MonitoringProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_PAGERDUTYINTEGRATION"].fields_by_name["token"]._options = None
    _globals["_PAGERDUTYINTEGRATION"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["token"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_id"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_id"]._serialized_options = b"\030\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_token"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_token"]._serialized_options = b"\030\001"
    _globals["_INCIDENTIOEVENTV2"].fields_by_name["route_token"]._options = None
    _globals["_INCIDENTIOEVENTV2"].fields_by_name["route_token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOEVENTV2"]._options = None
    _globals["_INCIDENTIOEVENTV2"]._serialized_options = b"\030\001"
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"].fields_by_name["token"]._options = None
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"].fields_by_name["token"]._options = None
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_MONITORINGSERVICE"].methods_by_name["TestPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name["TestPagerDutyIntegration"]._serialized_options = b"\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["AddPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "AddPagerDutyIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["DeletePagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "DeletePagerDutyIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["SetDefaultPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "SetDefaultPagerDutyIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["UpdatePagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "UpdatePagerDutyIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetAllPagerDutyIntegrations"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetAllPagerDutyIntegrations"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetPagerDutyIntegration"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["TestIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name["TestIncidentIoIntegration"]._serialized_options = b"\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["AddIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "AddIncidentIoIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["DeleteIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "DeleteIncidentIoIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["UpdateIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "UpdateIncidentIoIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetAllIncidentIoIntegrations"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetAllIncidentIoIntegrations"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetIncidentIoIntegration"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_PAGERDUTYSEVERITY"]._serialized_start = 4149
    _globals["_PAGERDUTYSEVERITY"]._serialized_end = 4333
    _globals["_PAGERDUTYEVENTACTION"]._serialized_start = 4336
    _globals["_PAGERDUTYEVENTACTION"]._serialized_end = 4514
    _globals["_PAGERDUTYEVENTV2PAYLOAD"]._serialized_start = 213
    _globals["_PAGERDUTYEVENTV2PAYLOAD"]._serialized_end = 552
    _globals["_PAGERDUTYEVENTV2LINK"]._serialized_start = 554
    _globals["_PAGERDUTYEVENTV2LINK"]._serialized_end = 630
    _globals["_PAGERDUTYEVENTV2IMAGE"]._serialized_start = 632
    _globals["_PAGERDUTYEVENTV2IMAGE"]._serialized_end = 738
    _globals["_PAGERDUTYEVENTV2"]._serialized_start = 741
    _globals["_PAGERDUTYEVENTV2"]._serialized_end = 1198
    _globals["_PAGERDUTYINTEGRATION"]._serialized_start = 1201
    _globals["_PAGERDUTYINTEGRATION"]._serialized_end = 1366
    _globals["_INCIDENTIOINTEGRATION"]._serialized_start = 1369
    _globals["_INCIDENTIOINTEGRATION"]._serialized_end = 1653
    _globals["_INCIDENTIOEVENTV2"]._serialized_start = 1656
    _globals["_INCIDENTIOEVENTV2"]._serialized_end = 1945
    _globals["_TESTPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 1947
    _globals["_TESTPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 1996
    _globals["_GETPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 1998
    _globals["_GETPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2058
    _globals["_GETPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2060
    _globals["_GETPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2166
    _globals["_TESTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2168
    _globals["_TESTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2252
    _globals["_ADDPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2254
    _globals["_ADDPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2342
    _globals["_ADDPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2344
    _globals["_ADDPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2450
    _globals["_DELETEPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2452
    _globals["_DELETEPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2503
    _globals["_DELETEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2505
    _globals["_DELETEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2541
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2544
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2715
    _globals["_UPDATEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2717
    _globals["_UPDATEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2826
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2828
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2883
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2885
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2941
    _globals["_GETALLPAGERDUTYINTEGRATIONSREQUEST"]._serialized_start = 2943
    _globals["_GETALLPAGERDUTYINTEGRATIONSREQUEST"]._serialized_end = 2979
    _globals["_GETALLPAGERDUTYINTEGRATIONSRESPONSE"]._serialized_start = 2981
    _globals["_GETALLPAGERDUTYINTEGRATIONSRESPONSE"]._serialized_end = 3093
    _globals["_TESTINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3095
    _globals["_TESTINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3145
    _globals["_TESTINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3147
    _globals["_TESTINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3232
    _globals["_GETINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3234
    _globals["_GETINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3283
    _globals["_GETINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3285
    _globals["_GETINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3393
    _globals["_ADDINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3396
    _globals["_ADDINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3543
    _globals["_ADDINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3545
    _globals["_ADDINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3653
    _globals["_DELETEINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3655
    _globals["_DELETEINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3707
    _globals["_DELETEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3709
    _globals["_DELETEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3746
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3749
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3878
    _globals["_UPDATEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3880
    _globals["_UPDATEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3991
    _globals["_GETALLINCIDENTIOINTEGRATIONSREQUEST"]._serialized_start = 3993
    _globals["_GETALLINCIDENTIOINTEGRATIONSREQUEST"]._serialized_end = 4030
    _globals["_GETALLINCIDENTIOINTEGRATIONSRESPONSE"]._serialized_start = 4032
    _globals["_GETALLINCIDENTIOINTEGRATIONSRESPONSE"]._serialized_end = 4146
    _globals["_MONITORINGSERVICE"]._serialized_start = 4517
    _globals["_MONITORINGSERVICE"]._serialized_end = 6420
# @@protoc_insertion_point(module_scope)
