# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/link.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.utils.v1 import sensitive_pb2 as chalk_dot_utils_dot_v1_dot_sensitive__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1a\x63halk/server/v1/link.proto\x12\x0f\x63halk.server.v1\x1a\x1e\x63halk/utils/v1/sensitive.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x8e\x02\n\tLinkToken\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1b\n\tclient_id\x18\x02 \x01(\tR\x08\x63lientId\x12)\n\rclient_secret\x18\x03 \x01(\tB\x04\xd8\xa1\'\x01R\x0c\x63lientSecret\x12\x1d\n\napi_server\x18\x04 \x01(\tR\tapiServer\x12\x32\n\x12\x61\x63tive_environment\x18\x05 \x01(\tH\x00R\x11\x61\x63tiveEnvironment\x88\x01\x01\x12;\n\x0bvalid_until\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\nvalidUntilB\x15\n\x13_active_environment"]\n\x15GetLinkSessionRequest\x12!\n\tlink_code\x18\x01 \x01(\tB\x04\xd8\xa1\'\x01R\x08linkCode\x12!\n\x0cproject_name\x18\x02 \x01(\tR\x0bprojectName"\xce\x01\n\x16GetLinkSessionResponse\x12:\n\x06status\x18\x01 \x01(\x0e\x32".chalk.server.v1.LinkSessionStatusR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x35\n\x05token\x18\x03 \x01(\x0b\x32\x1a.chalk.server.v1.LinkTokenH\x00R\x05token\x88\x01\x01\x12\x1d\n\nsession_id\x18\x04 \x01(\tR\tsessionIdB\x08\n\x06_token"\x1a\n\x18\x43reateLinkSessionRequest"\xbb\x01\n\x19\x43reateLinkSessionResponse\x12!\n\tlink_code\x18\x01 \x01(\tB\x04\xd8\xa1\'\x01R\x08linkCode\x12!\n\tauth_link\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\x08\x61uthLink\x12\x39\n\nexpires_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\texpiresAt\x12\x1d\n\nsession_id\x18\x04 \x01(\tR\tsessionId*\xe0\x01\n\x11LinkSessionStatus\x12#\n\x1fLINK_SESSION_STATUS_UNSPECIFIED\x10\x00\x12\x1f\n\x1bLINK_SESSION_STATUS_PENDING\x10\x01\x12\x1f\n\x1bLINK_SESSION_STATUS_SUCCESS\x10\x02\x12\x1e\n\x1aLINK_SESSION_STATUS_FAILED\x10\x03\x12!\n\x1dLINK_SESSION_STATUS_NOT_FOUND\x10\x04\x12!\n\x1dLINK_SESSION_STATUS_FORBIDDEN\x10\x05\x42\x92\x01\n\x13\x63om.chalk.server.v1B\tLinkProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.link_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\tLinkProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_LINKTOKEN"].fields_by_name["client_secret"]._options = None
    _globals["_LINKTOKEN"].fields_by_name["client_secret"]._serialized_options = b"\330\241'\001"
    _globals["_GETLINKSESSIONREQUEST"].fields_by_name["link_code"]._options = None
    _globals["_GETLINKSESSIONREQUEST"].fields_by_name["link_code"]._serialized_options = b"\330\241'\001"
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["link_code"]._options = None
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["link_code"]._serialized_options = b"\330\241'\001"
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["auth_link"]._options = None
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["auth_link"]._serialized_options = b"\330\241'\001"
    _globals["_LINKSESSIONSTATUS"]._serialized_start = 942
    _globals["_LINKSESSIONSTATUS"]._serialized_end = 1166
    _globals["_LINKTOKEN"]._serialized_start = 147
    _globals["_LINKTOKEN"]._serialized_end = 417
    _globals["_GETLINKSESSIONREQUEST"]._serialized_start = 419
    _globals["_GETLINKSESSIONREQUEST"]._serialized_end = 512
    _globals["_GETLINKSESSIONRESPONSE"]._serialized_start = 515
    _globals["_GETLINKSESSIONRESPONSE"]._serialized_end = 721
    _globals["_CREATELINKSESSIONREQUEST"]._serialized_start = 723
    _globals["_CREATELINKSESSIONREQUEST"]._serialized_end = 749
    _globals["_CREATELINKSESSIONRESPONSE"]._serialized_start = 752
    _globals["_CREATELINKSESSIONRESPONSE"]._serialized_end = 939
# @@protoc_insertion_point(module_scope)
