GRPC_RESULT_METADATA_COL_PREFIX: str = "__chalk__.__result_metadata__."
"""
Column name prefix in which the GRPC server returns metadata about a given feature
(e.g. is it a cache hit, what the resolver fqn was, etc.)
"""
