# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/offline_queries.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.aggregate.v1 import service_pb2 as chalk_dot_aggregate_dot_v1_dot_service__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.server.v1 import datasets_pb2 as chalk_dot_server_dot_v1_dot_datasets__pb2
from chalk._gen.chalk.server.v1 import performance_summary_pb2 as chalk_dot_server_dot_v1_dot_performance__summary__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n%chalk/server/v1/offline_queries.proto\x12\x0f\x63halk.server.v1\x1a chalk/aggregate/v1/service.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1e\x63halk/server/v1/datasets.proto\x1a)chalk/server/v1/performance_summary.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x8d\x12\n\x10OfflineQueryMeta\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12!\n\x0coperation_id\x18\x02 \x01(\tR\x0boperationId\x12%\n\x0e\x65nvironment_id\x18\x03 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x04 \x01(\tR\x0c\x64\x65ploymentId\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12:\n\nquery_meta\x18\x06 \x01(\x0b\x32\x16.google.protobuf.ValueH\x00R\tqueryMeta\x88\x01\x01\x12\'\n\rquery_plan_id\x18\x07 \x01(\tH\x01R\x0bqueryPlanId\x88\x01\x01\x12$\n\x0b\x62ranch_name\x18\x08 \x01(\tH\x02R\nbranchName\x88\x01\x01\x12"\n\ndataset_id\x18\t \x01(\tH\x03R\tdatasetId\x88\x01\x01\x12&\n\x0c\x64\x61taset_name\x18\n \x01(\tH\x04R\x0b\x64\x61tasetName\x88\x01\x01\x12"\n\nhas_errors\x18\x0b \x01(\x08H\x05R\thasErrors\x88\x01\x01\x12\x1e\n\x08\x61gent_id\x18\x0c \x01(\tH\x06R\x07\x61gentId\x88\x01\x01\x12\x1e\n\x08trace_id\x18\r \x01(\tH\x07R\x07traceId\x88\x01\x01\x12*\n\x0e\x63orrelation_id\x18\x0e \x01(\tH\x08R\rcorrelationId\x88\x01\x01\x12=\n\x0c\x63ompleted_at\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0b\x63ompletedAt\x12@\n\x06status\x18\x10 \x01(\x0e\x32#.chalk.server.v1.OfflineQueryStatusH\tR\x06status\x88\x01\x01\x12&\n\x0fhas_plan_stages\x18\x11 \x01(\x08R\rhasPlanStages\x12,\n\x0ftotal_computers\x18\x12 \x01(\x05H\nR\x0etotalComputers\x88\x01\x01\x12;\n\x17num_completed_computers\x18\x13 \x01(\x05H\x0bR\x15numCompletedComputers\x88\x01\x01\x12.\n\x10total_partitions\x18\x14 \x01(\x05H\x0cR\x0ftotalPartitions\x88\x01\x01\x12=\n\x18num_completed_partitions\x18\x15 \x01(\x05H\rR\x16numCompletedPartitions\x88\x01\x01\x12\x32\n\x12recompute_features\x18\x16 \x01(\tH\x0eR\x11recomputeFeatures\x88\x01\x01\x12+\n\x0fspine_sql_query\x18\x17 \x01(\tH\x0fR\rspineSqlQuery\x88\x01\x01\x12\x35\n\x07\x66ilters\x18\x18 \x01(\x0b\x32\x16.google.protobuf.ValueH\x10R\x07\x66ilters\x88\x01\x01\x12\x44\n\x0fplanner_options\x18\x19 \x01(\x0b\x32\x16.google.protobuf.ValueH\x11R\x0eplannerOptions\x88\x01\x01\x12\x44\n\x0finvoker_options\x18\x1a \x01(\x0b\x32\x16.google.protobuf.ValueH\x12R\x0einvokerOptions\x88\x01\x01\x12\x45\n\nquery_type\x18\x1b \x01(\x0e\x32!.chalk.server.v1.OfflineQueryKindH\x13R\tqueryType\x88\x01\x01\x12\x12\n\x04tags\x18\x1c \x03(\tR\x04tags\x12\x34\n\x16required_resolver_tags\x18\x1d \x03(\tR\x14requiredResolverTags\x12\x37\n\x15\x61ggregate_backfill_id\x18\x1e \x01(\tH\x14R\x13\x61ggregateBackfillId\x88\x01\x01\x12\x33\n\x06output\x18\x1f \x01(\x0b\x32\x16.google.protobuf.ValueH\x15R\x06output\x88\x01\x01\x12\x44\n\x0frequired_output\x18  \x01(\x0b\x32\x16.google.protobuf.ValueH\x16R\x0erequiredOutput\x88\x01\x01\x12/\n\x11raw_body_filename\x18! \x01(\tH\x17R\x0frawBodyFilename\x88\x01\x01\x12T\n\x10\x64\x61taset_revision\x18" \x01(\x0b\x32$.chalk.server.v1.DatasetRevisionMetaH\x18R\x0f\x64\x61tasetRevision\x88\x01\x01\x12R\n\x0btime_series\x18# \x03(\x0b\x32\x31.chalk.aggregate.v1.PlanAggregateBackfillResponseR\ntimeSeries\x12/\n\x11\x65valuation_run_id\x18$ \x01(\tH\x19R\x0f\x65valuationRunId\x88\x01\x01\x42\r\n\x0b_query_metaB\x10\n\x0e_query_plan_idB\x0e\n\x0c_branch_nameB\r\n\x0b_dataset_idB\x0f\n\r_dataset_nameB\r\n\x0b_has_errorsB\x0b\n\t_agent_idB\x0b\n\t_trace_idB\x11\n\x0f_correlation_idB\t\n\x07_statusB\x12\n\x10_total_computersB\x1a\n\x18_num_completed_computersB\x13\n\x11_total_partitionsB\x1b\n\x19_num_completed_partitionsB\x15\n\x13_recompute_featuresB\x12\n\x10_spine_sql_queryB\n\n\x08_filtersB\x12\n\x10_planner_optionsB\x12\n\x10_invoker_optionsB\r\n\x0b_query_typeB\x18\n\x16_aggregate_backfill_idB\t\n\x07_outputB\x12\n\x10_required_outputB\x14\n\x12_raw_body_filenameB\x13\n\x11_dataset_revisionB\x14\n\x12_evaluation_run_id"\xf7\x05\n\x19ListOfflineQueriesRequest\x12\x1b\n\x06\x63ursor\x18\x01 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x12"\n\nstart_date\x18\x03 \x01(\tH\x02R\tstartDate\x88\x01\x01\x12\x1e\n\x08\x65nd_date\x18\x04 \x01(\tH\x03R\x07\x65ndDate\x88\x01\x01\x12 \n\tid_filter\x18\x05 \x01(\tH\x04R\x08idFilter\x88\x01\x01\x12+\n\x0f\x61gent_id_filter\x18\x06 \x01(\tH\x05R\ragentIdFilter\x88\x01\x01\x12(\n\rbranch_filter\x18\x07 \x01(\tH\x06R\x0c\x62ranchFilter\x88\x01\x01\x12G\n\x0bkind_filter\x18\x08 \x01(\x0e\x32!.chalk.server.v1.OfflineQueryKindH\x07R\nkindFilter\x88\x01\x01\x12M\n\rstatus_filter\x18\t \x01(\x0e\x32#.chalk.server.v1.OfflineQueryStatusH\x08R\x0cstatusFilter\x88\x01\x01\x12H\n\x1e\x61ggregation_backfill_id_filter\x18\n \x01(\tH\tR\x1b\x61ggregationBackfillIdFilter\x88\x01\x01\x12<\n\x18\x65valuation_run_id_filter\x18\x0b \x01(\tH\nR\x15\x65valuationRunIdFilter\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limitB\r\n\x0b_start_dateB\x0b\n\t_end_dateB\x0c\n\n_id_filterB\x12\n\x10_agent_id_filterB\x10\n\x0e_branch_filterB\x0e\n\x0c_kind_filterB\x10\n\x0e_status_filterB!\n\x1f_aggregation_backfill_id_filterB\x1b\n\x19_evaluation_run_id_filter"\x90\x01\n\x1aListOfflineQueriesResponse\x12J\n\x0foffline_queries\x18\x01 \x03(\x0b\x32!.chalk.server.v1.OfflineQueryMetaR\x0eofflineQueries\x12\x1b\n\x06\x63ursor\x18\x02 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x42\t\n\x07_cursor"B\n\x16GetOfflineQueryRequest\x12(\n\x10offline_query_id\x18\x01 \x01(\tR\x0eofflineQueryId"a\n\x17GetOfflineQueryResponse\x12\x46\n\roffline_query\x18\x01 \x01(\x0b\x32!.chalk.server.v1.OfflineQueryMetaR\x0cofflineQuery*\x87\x02\n\x12OfflineQueryStatus\x12$\n OFFLINE_QUERY_STATUS_UNSPECIFIED\x10\x00\x12 \n\x1cOFFLINE_QUERY_STATUS_UNKNOWN\x10\x01\x12 \n\x1cOFFLINE_QUERY_STATUS_WORKING\x10\x02\x12\x1f\n\x1bOFFLINE_QUERY_STATUS_FAILED\x10\x03\x12"\n\x1eOFFLINE_QUERY_STATUS_COMPLETED\x10\x04\x12!\n\x1dOFFLINE_QUERY_STATUS_CANCELED\x10\x05\x12\x1f\n\x1bOFFLINE_QUERY_STATUS_QUEUED\x10\x06*\xaa\x02\n\x10OfflineQueryKind\x12"\n\x1eOFFLINE_QUERY_KIND_UNSPECIFIED\x10\x00\x12\x1e\n\x1aOFFLINE_QUERY_KIND_UNKNOWN\x10\x01\x12*\n&OFFLINE_QUERY_KIND_ASYNC_OFFLINE_QUERY\x10\x02\x12)\n%OFFLINE_QUERY_KIND_CRON_OFFLINE_QUERY\x10\x03\x12$\n OFFLINE_QUERY_KIND_OFFLINE_QUERY\x10\x04\x12(\n$OFFLINE_QUERY_KIND_DATASET_INGESTION\x10\x05\x12+\n\'OFFLINE_QUERY_KIND_AGGREGATION_BACKFILL\x10\x06\x32\xbf\x03\n\x1bOfflineQueryMetadataService\x12u\n\x12ListOfflineQueries\x12*.chalk.server.v1.ListOfflineQueriesRequest\x1a+.chalk.server.v1.ListOfflineQueriesResponse"\x06\x90\x02\x01\x80}\x0b\x12l\n\x0fGetOfflineQuery\x12\'.chalk.server.v1.GetOfflineQueryRequest\x1a(.chalk.server.v1.GetOfflineQueryResponse"\x06\x90\x02\x01\x80}\x0b\x12\xba\x01\n)ListOfflineQueryShardPerformanceSummaries\x12\x41.chalk.server.v1.ListOfflineQueryShardPerformanceSummariesRequest\x1a\x42.chalk.server.v1.ListOfflineQueryShardPerformanceSummariesResponse"\x06\x90\x02\x01\x80}\x0b\x42\x9c\x01\n\x13\x63om.chalk.server.v1B\x13OfflineQueriesProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.offline_queries_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\023OfflineQueriesProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name["ListOfflineQueries"]._options = None
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name[
        "ListOfflineQueries"
    ]._serialized_options = b"\220\002\001\200}\013"
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name["GetOfflineQuery"]._options = None
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name[
        "GetOfflineQuery"
    ]._serialized_options = b"\220\002\001\200}\013"
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name[
        "ListOfflineQueryShardPerformanceSummaries"
    ]._options = None
    _globals["_OFFLINEQUERYMETADATASERVICE"].methods_by_name[
        "ListOfflineQueryShardPerformanceSummaries"
    ]._serialized_options = b"\220\002\001\200}\013"
    _globals["_OFFLINEQUERYSTATUS"]._serialized_start = 3660
    _globals["_OFFLINEQUERYSTATUS"]._serialized_end = 3923
    _globals["_OFFLINEQUERYKIND"]._serialized_start = 3926
    _globals["_OFFLINEQUERYKIND"]._serialized_end = 4224
    _globals["_OFFLINEQUERYMETA"]._serialized_start = 264
    _globals["_OFFLINEQUERYMETA"]._serialized_end = 2581
    _globals["_LISTOFFLINEQUERIESREQUEST"]._serialized_start = 2584
    _globals["_LISTOFFLINEQUERIESREQUEST"]._serialized_end = 3343
    _globals["_LISTOFFLINEQUERIESRESPONSE"]._serialized_start = 3346
    _globals["_LISTOFFLINEQUERIESRESPONSE"]._serialized_end = 3490
    _globals["_GETOFFLINEQUERYREQUEST"]._serialized_start = 3492
    _globals["_GETOFFLINEQUERYREQUEST"]._serialized_end = 3558
    _globals["_GETOFFLINEQUERYRESPONSE"]._serialized_start = 3560
    _globals["_GETOFFLINEQUERYRESPONSE"]._serialized_end = 3657
    _globals["_OFFLINEQUERYMETADATASERVICE"]._serialized_start = 4227
    _globals["_OFFLINEQUERYMETADATASERVICE"]._serialized_end = 4674
# @@protoc_insertion_point(module_scope)
