# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v2/feature_values.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.arrow.v1 import arrow_pb2 as chalk_dot_arrow_dot_v1_dot_arrow__pb2
from chalk._gen.chalk.common.v1 import operation_kind_pb2 as chalk_dot_common_dot_v1_dot_operation__kind__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n$chalk/engine/v2/feature_values.proto\x12\x0f\x63halk.engine.v2\x1a\x1a\x63halk/arrow/v1/arrow.proto\x1a$chalk/common/v1/operation_kind.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x85\x02\n\x13\x46\x65\x61tureValueFilters\x12!\n\x0cresolver_fqn\x18\x01 \x03(\tR\x0bresolverFqn\x12#\n\rdeployment_id\x18\x02 \x03(\tR\x0c\x64\x65ploymentId\x12!\n\x0coperation_id\x18\x03 \x03(\tR\x0boperationId\x12\x45\n\x0eoperation_kind\x18\x04 \x03(\x0e\x32\x1e.chalk.common.v1.OperationKindR\roperationKind\x12<\n\x0bprimary_key\x18\x05 \x03(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\nprimaryKey"\xb4\x01\n\x19GetFeatureValuesPageToken\x12?\n\rtimestamp_hwm\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0ctimestampHwm\x12(\n\x10operation_id_hwm\x18\x02 \x01(\tR\x0eoperationIdHwm\x12,\n\x12observation_id_hwm\x18\x03 \x01(\tR\x10observationIdHwm"\xaa\x03\n\x17GetFeatureValuesRequest\x12\x1f\n\x0b\x66\x65\x61ture_fqn\x18\x01 \x01(\tR\nfeatureFqn\x12R\n\x0etimestamp_type\x18\x02 \x01(\x0e\x32+.chalk.engine.v2.FeatureValuesTimestampTypeR\rtimestampType\x12N\n\x15lower_bound_inclusive\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x13lowerBoundInclusive\x12N\n\x15upper_bound_exclusive\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x13upperBoundExclusive\x12>\n\x07\x66ilters\x18\x05 \x01(\x0b\x32$.chalk.engine.v2.FeatureValueFiltersR\x07\x66ilters\x12\x1b\n\tpage_size\x18\x06 \x01(\x05R\x08pageSize\x12\x1d\n\npage_token\x18\x07 \x01(\tR\tpageToken"\x88\x01\n\x18GetFeatureValuesResponse\x12&\n\x0fnext_page_token\x18\x01 \x01(\tR\rnextPageToken\x12\x1d\n\ntotal_size\x18\x02 \x01(\x05R\ttotalSize\x12\x1a\n\x07parquet\x18\x03 \x01(\x0cH\x00R\x07parquetB\t\n\x07payload*\xa9\x01\n\x1a\x46\x65\x61tureValuesTimestampType\x12-\n)FEATURE_VALUES_TIMESTAMP_TYPE_UNSPECIFIED\x10\x00\x12-\n)FEATURE_VALUES_TIMESTAMP_TYPE_INSERTED_AT\x10\x01\x12-\n)FEATURE_VALUES_TIMESTAMP_TYPE_OBSERVED_AT\x10\x02\x42\x87\x01\n\x13\x63om.chalk.engine.v2B\x12\x46\x65\x61tureValuesProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V2\xca\x02\x0f\x43halk\\Engine\\V2\xe2\x02\x1b\x43halk\\Engine\\V2\\GPBMetadata\xea\x02\x11\x43halk::Engine::V2b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v2.feature_values_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v2B\022FeatureValuesProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V2\312\002\017Chalk\\Engine\\V2\342\002\033Chalk\\Engine\\V2\\GPBMetadata\352\002\021Chalk::Engine::V2"
    _globals["_FEATUREVALUESTIMESTAMPTYPE"]._serialized_start = 1172
    _globals["_FEATUREVALUESTIMESTAMPTYPE"]._serialized_end = 1341
    _globals["_FEATUREVALUEFILTERS"]._serialized_start = 157
    _globals["_FEATUREVALUEFILTERS"]._serialized_end = 418
    _globals["_GETFEATUREVALUESPAGETOKEN"]._serialized_start = 421
    _globals["_GETFEATUREVALUESPAGETOKEN"]._serialized_end = 601
    _globals["_GETFEATUREVALUESREQUEST"]._serialized_start = 604
    _globals["_GETFEATUREVALUESREQUEST"]._serialized_end = 1030
    _globals["_GETFEATUREVALUESRESPONSE"]._serialized_start = 1033
    _globals["_GETFEATUREVALUESRESPONSE"]._serialized_end = 1169
# @@protoc_insertion_point(module_scope)
