# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/pod_request.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.chart.v1 import densetimeserieschart_pb2 as chalk_dot_chart_dot_v1_dot_densetimeserieschart__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!chalk/server/v1/pod_request.proto\x12\x0f\x63halk.server.v1\x1a)chalk/chart/v1/densetimeserieschart.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xcc\x03\n\x1aGetPodRequestChartsRequest\x12V\n\x19start_timestamp_inclusive\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x17startTimestampInclusive\x12W\n\x17\x65nd_timestamp_exclusive\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x15\x65ndTimestampExclusive\x88\x01\x01\x12>\n\rwindow_period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod\x12\x44\n\x08grouping\x18\x04 \x01(\x0e\x32#.chalk.server.v1.PodRequestGroupingH\x01R\x08grouping\x88\x01\x01\x12N\n\x0eresource_types\x18\x05 \x03(\x0e\x32\'.chalk.server.v1.PodRequestResourceTypeR\rresourceTypesB\x1a\n\x18_end_timestamp_exclusiveB\x0b\n\t_grouping"[\n\x1bGetPodRequestChartsResponse\x12<\n\x06\x63harts\x18\x01 \x03(\x0b\x32$.chalk.chart.v1.DenseTimeSeriesChartR\x06\x63harts*\xc8\x01\n\x12PodRequestGrouping\x12$\n POD_REQUEST_GROUPING_UNSPECIFIED\x10\x00\x12"\n\x1ePOD_REQUEST_GROUPING_NAMESPACE\x10\x01\x12 \n\x1cPOD_REQUEST_GROUPING_CLUSTER\x10\x02\x12$\n POD_REQUEST_GROUPING_ENVIRONMENT\x10\x03\x12 \n\x1cPOD_REQUEST_GROUPING_SERVICE\x10\x04*\xbd\x01\n\x16PodRequestResourceType\x12)\n%POD_REQUEST_RESOURCE_TYPE_UNSPECIFIED\x10\x00\x12!\n\x1dPOD_REQUEST_RESOURCE_TYPE_CPU\x10\x01\x12$\n POD_REQUEST_RESOURCE_TYPE_MEMORY\x10\x02\x12/\n+POD_REQUEST_RESOURCE_TYPE_EPHEMERAL_STORAGE\x10\x03\x42\x98\x01\n\x13\x63om.chalk.server.v1B\x0fPodRequestProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.pod_request_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\017PodRequestProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_PODREQUESTGROUPING"]._serialized_start = 719
    _globals["_PODREQUESTGROUPING"]._serialized_end = 919
    _globals["_PODREQUESTRESOURCETYPE"]._serialized_start = 922
    _globals["_PODREQUESTRESOURCETYPE"]._serialized_end = 1111
    _globals["_GETPODREQUESTCHARTSREQUEST"]._serialized_start = 163
    _globals["_GETPODREQUESTCHARTSREQUEST"]._serialized_end = 623
    _globals["_GETPODREQUESTCHARTSRESPONSE"]._serialized_start = 625
    _globals["_GETPODREQUESTCHARTSRESPONSE"]._serialized_end = 716
# @@protoc_insertion_point(module_scope)
