# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/topic.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/server/v1/topic.proto\x12\x0f\x63halk.server.v1"\'\n\x08SQSTopic\x12\x1b\n\tqueue_url\x18\x01 \x01(\tR\x08queueUrl"G\n\x0bPubSubTopic\x12\x1d\n\nproject_id\x18\x01 \x01(\tR\tprojectId\x12\x19\n\x08topic_id\x18\x02 \x01(\tR\x07topicId"\x8d\x01\n\x05Topic\x12\x38\n\tsqs_topic\x18\x01 \x01(\x0b\x32\x19.chalk.server.v1.SQSTopicH\x00R\x08sqsTopic\x12\x41\n\x0cpubsub_topic\x18\x02 \x01(\x0b\x32\x1c.chalk.server.v1.PubSubTopicH\x00R\x0bpubsubTopicB\x07\n\x05topicB\x93\x01\n\x13\x63om.chalk.server.v1B\nTopicProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.topic_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\nTopicProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_SQSTOPIC"]._serialized_start = 48
    _globals["_SQSTOPIC"]._serialized_end = 87
    _globals["_PUBSUBTOPIC"]._serialized_start = 89
    _globals["_PUBSUBTOPIC"]._serialized_end = 160
    _globals["_TOPIC"]._serialized_start = 163
    _globals["_TOPIC"]._serialized_end = 304
# @@protoc_insertion_point(module_scope)
