"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.http_engine_proxy_pb2 import (
    CreateOfflineQueryRequest,
    CreateOfflineQueryResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class HTTPEngineProxyServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    CreateOfflineQuery: UnaryUnaryMultiCallable[
        CreateOfflineQueryRequest,
        CreateOfflineQueryResponse,
    ]

class HTTPEngineProxyServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def CreateOfflineQuery(
        self,
        request: CreateOfflineQueryRequest,
        context: ServicerContext,
    ) -> CreateOfflineQueryResponse: ...

def add_HTTPEngineProxyServiceServicer_to_server(servicer: HTTPEngineProxyServiceServicer, server: Server) -> None: ...
