# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/nodepools/v1/karpenter.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.kubernetes.v1 import nodes_pb2 as chalk_dot_kubernetes_dot_v1_dot_nodes__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n"chalk/nodepools/v1/karpenter.proto\x12\x12\x63halk.nodepools.v1\x1a\x1f\x63halk/kubernetes/v1/nodes.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xb0\x01\n!KarpenterNodepoolDisruptionBudget\x12\x14\n\x05nodes\x18\x01 \x01(\tR\x05nodes\x12\x1f\n\x08schedule\x18\x02 \x01(\tH\x00R\x08schedule\x88\x01\x01\x12:\n\x08\x64uration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationH\x01R\x08\x64uration\x88\x01\x01\x42\x0b\n\t_scheduleB\x0b\n\t_duration"\xa2\x02\n\x1bKarpenterNodepoolDisruption\x12K\n\x11\x63onsolidate_after\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x10\x63onsolidateAfter\x88\x01\x01\x12\x36\n\x14\x63onsolidation_policy\x18\x02 \x01(\tH\x01R\x13\x63onsolidationPolicy\x88\x01\x01\x12O\n\x07\x62udgets\x18\x03 \x03(\x0b\x32\x35.chalk.nodepools.v1.KarpenterNodepoolDisruptionBudgetR\x07\x62udgetsB\x14\n\x12_consolidate_afterB\x17\n\x15_consolidation_policy"r\n\x15KarpenterNodeClassRef\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x17\n\x04kind\x18\x02 \x01(\tH\x00R\x04kind\x88\x01\x01\x12\x19\n\x05group\x18\x03 \x01(\tH\x01R\x05group\x88\x01\x01\x42\x07\n\x05_kindB\x08\n\x06_group"h\n KarpenterNodeSelectorRequirement\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x1a\n\x08operator\x18\x02 \x01(\tR\x08operator\x12\x16\n\x06values\x18\x03 \x03(\tR\x06values"\xa8\x04\n\x1dKarpenterNodepoolTemplateSpec\x12@\n\x06taints\x18\x01 \x03(\x0b\x32(.chalk.kubernetes.v1.KubernetesNodeTaintR\x06taints\x12O\n\x0estartup_taints\x18\x02 \x03(\x0b\x32(.chalk.kubernetes.v1.KubernetesNodeTaintR\rstartupTaints\x12X\n\x0crequirements\x18\x03 \x03(\x0b\x32\x34.chalk.nodepools.v1.KarpenterNodeSelectorRequirementR\x0crequirements\x12O\n\x0enode_class_ref\x18\x04 \x01(\x0b\x32).chalk.nodepools.v1.KarpenterNodeClassRefR\x0cnodeClassRef\x12\x41\n\x0c\x65xpire_after\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x0b\x65xpireAfter\x88\x01\x01\x12X\n\x18termination_grace_period\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationH\x01R\x16terminationGracePeriod\x88\x01\x01\x42\x0f\n\r_expire_afterB\x1b\n\x19_termination_grace_period"\xe3\x02\n!KarpenterNodepoolTemplateMetadata\x12Y\n\x06labels\x18\x01 \x03(\x0b\x32\x41.chalk.nodepools.v1.KarpenterNodepoolTemplateMetadata.LabelsEntryR\x06labels\x12h\n\x0b\x61nnotations\x18\x02 \x03(\x0b\x32\x46.chalk.nodepools.v1.KarpenterNodepoolTemplateMetadata.AnnotationsEntryR\x0b\x61nnotations\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a>\n\x10\x41nnotationsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xb5\x01\n\x19KarpenterNodepoolTemplate\x12Q\n\x08metadata\x18\x01 \x01(\x0b\x32\x35.chalk.nodepools.v1.KarpenterNodepoolTemplateMetadataR\x08metadata\x12\x45\n\x04spec\x18\x02 \x01(\x0b\x32\x31.chalk.nodepools.v1.KarpenterNodepoolTemplateSpecR\x04spec"\xe5\x02\n\x15KarpenterNodepoolSpec\x12O\n\ndisruption\x18\x01 \x01(\x0b\x32/.chalk.nodepools.v1.KarpenterNodepoolDisruptionR\ndisruption\x12I\n\x08template\x18\x02 \x01(\x0b\x32-.chalk.nodepools.v1.KarpenterNodepoolTemplateR\x08template\x12M\n\x06limits\x18\x03 \x03(\x0b\x32\x35.chalk.nodepools.v1.KarpenterNodepoolSpec.LimitsEntryR\x06limits\x12\x1b\n\x06weight\x18\x04 \x01(\x05H\x00R\x06weight\x88\x01\x01\x1a\x39\n\x0bLimitsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\t\n\x07_weight"\x87\x04\n\x19KarpenterNodepoolMetadata\x12`\n\x0b\x61nnotations\x18\x01 \x03(\x0b\x32>.chalk.nodepools.v1.KarpenterNodepoolMetadata.AnnotationsEntryR\x0b\x61nnotations\x12I\n\x12\x63reation_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x11\x63reationTimestamp\x12\x1e\n\ngeneration\x18\x03 \x01(\x03R\ngeneration\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12)\n\x10resource_version\x18\x05 \x01(\tR\x0fresourceVersion\x12\x10\n\x03uid\x18\x06 \x01(\tR\x03uid\x12Q\n\x06labels\x18\x07 \x03(\x0b\x32\x39.chalk.nodepools.v1.KarpenterNodepoolMetadata.LabelsEntryR\x06labels\x1a>\n\x10\x41nnotationsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xcd\x01\n\x1aKarpenterNodepoolCondition\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\x12\x16\n\x06status\x18\x02 \x01(\tR\x06status\x12\x30\n\x14last_transition_time\x18\x04 \x01(\x03R\x12lastTransitionTime\x12\x1b\n\x06reason\x18\x05 \x01(\tH\x00R\x06reason\x88\x01\x01\x12\x1d\n\x07message\x18\x06 \x01(\tH\x01R\x07message\x88\x01\x01\x42\t\n\x07_reasonB\n\n\x08_message"\x81\x02\n\x17KarpenterNodepoolStatus\x12X\n\tresources\x18\x01 \x03(\x0b\x32:.chalk.nodepools.v1.KarpenterNodepoolStatus.ResourcesEntryR\tresources\x12N\n\nconditions\x18\x02 \x03(\x0b\x32..chalk.nodepools.v1.KarpenterNodepoolConditionR\nconditions\x1a<\n\x0eResourcesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\x97\x02\n\x11KarpenterNodepool\x12\x1f\n\x0b\x61pi_version\x18\x01 \x01(\tR\napiVersion\x12\x12\n\x04kind\x18\x02 \x01(\tR\x04kind\x12I\n\x08metadata\x18\x03 \x01(\x0b\x32-.chalk.nodepools.v1.KarpenterNodepoolMetadataR\x08metadata\x12=\n\x04spec\x18\x04 \x01(\x0b\x32).chalk.nodepools.v1.KarpenterNodepoolSpecR\x04spec\x12\x43\n\x06status\x18\x05 \x01(\x0b\x32+.chalk.nodepools.v1.KarpenterNodepoolStatusR\x06statusB\xac\x01\n\x16\x63om.chalk.nodepools.v1B\x0eKarpenterProtoP\x01Z\x18nodepools/v1;nodepoolsv1\xa2\x02\x03\x43NX\xaa\x02\x12\x43halk.Nodepools.V1\xca\x02\x12\x43halk\\Nodepools\\V1\xe2\x02\x1e\x43halk\\Nodepools\\V1\\GPBMetadata\xea\x02\x14\x43halk::Nodepools::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.nodepools.v1.karpenter_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\026com.chalk.nodepools.v1B\016KarpenterProtoP\001Z\030nodepools/v1;nodepoolsv1\242\002\003CNX\252\002\022Chalk.Nodepools.V1\312\002\022Chalk\\Nodepools\\V1\342\002\036Chalk\\Nodepools\\V1\\GPBMetadata\352\002\024Chalk::Nodepools::V1"
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_LABELSENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_LABELSENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_ANNOTATIONSENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_ANNOTATIONSENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLSPEC_LIMITSENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLSPEC_LIMITSENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLMETADATA_ANNOTATIONSENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLMETADATA_ANNOTATIONSENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLMETADATA_LABELSENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLMETADATA_LABELSENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLSTATUS_RESOURCESENTRY"]._options = None
    _globals["_KARPENTERNODEPOOLSTATUS_RESOURCESENTRY"]._serialized_options = b"8\001"
    _globals["_KARPENTERNODEPOOLDISRUPTIONBUDGET"]._serialized_start = 157
    _globals["_KARPENTERNODEPOOLDISRUPTIONBUDGET"]._serialized_end = 333
    _globals["_KARPENTERNODEPOOLDISRUPTION"]._serialized_start = 336
    _globals["_KARPENTERNODEPOOLDISRUPTION"]._serialized_end = 626
    _globals["_KARPENTERNODECLASSREF"]._serialized_start = 628
    _globals["_KARPENTERNODECLASSREF"]._serialized_end = 742
    _globals["_KARPENTERNODESELECTORREQUIREMENT"]._serialized_start = 744
    _globals["_KARPENTERNODESELECTORREQUIREMENT"]._serialized_end = 848
    _globals["_KARPENTERNODEPOOLTEMPLATESPEC"]._serialized_start = 851
    _globals["_KARPENTERNODEPOOLTEMPLATESPEC"]._serialized_end = 1403
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA"]._serialized_start = 1406
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA"]._serialized_end = 1761
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_LABELSENTRY"]._serialized_start = 1640
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_LABELSENTRY"]._serialized_end = 1697
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_ANNOTATIONSENTRY"]._serialized_start = 1699
    _globals["_KARPENTERNODEPOOLTEMPLATEMETADATA_ANNOTATIONSENTRY"]._serialized_end = 1761
    _globals["_KARPENTERNODEPOOLTEMPLATE"]._serialized_start = 1764
    _globals["_KARPENTERNODEPOOLTEMPLATE"]._serialized_end = 1945
    _globals["_KARPENTERNODEPOOLSPEC"]._serialized_start = 1948
    _globals["_KARPENTERNODEPOOLSPEC"]._serialized_end = 2305
    _globals["_KARPENTERNODEPOOLSPEC_LIMITSENTRY"]._serialized_start = 2237
    _globals["_KARPENTERNODEPOOLSPEC_LIMITSENTRY"]._serialized_end = 2294
    _globals["_KARPENTERNODEPOOLMETADATA"]._serialized_start = 2308
    _globals["_KARPENTERNODEPOOLMETADATA"]._serialized_end = 2827
    _globals["_KARPENTERNODEPOOLMETADATA_ANNOTATIONSENTRY"]._serialized_start = 1699
    _globals["_KARPENTERNODEPOOLMETADATA_ANNOTATIONSENTRY"]._serialized_end = 1761
    _globals["_KARPENTERNODEPOOLMETADATA_LABELSENTRY"]._serialized_start = 1640
    _globals["_KARPENTERNODEPOOLMETADATA_LABELSENTRY"]._serialized_end = 1697
    _globals["_KARPENTERNODEPOOLCONDITION"]._serialized_start = 2830
    _globals["_KARPENTERNODEPOOLCONDITION"]._serialized_end = 3035
    _globals["_KARPENTERNODEPOOLSTATUS"]._serialized_start = 3038
    _globals["_KARPENTERNODEPOOLSTATUS"]._serialized_end = 3295
    _globals["_KARPENTERNODEPOOLSTATUS_RESOURCESENTRY"]._serialized_start = 3235
    _globals["_KARPENTERNODEPOOLSTATUS_RESOURCESENTRY"]._serialized_end = 3295
    _globals["_KARPENTERNODEPOOL"]._serialized_start = 3298
    _globals["_KARPENTERNODEPOOL"]._serialized_end = 3577
# @@protoc_insertion_point(module_scope)
