# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/timeserieschart.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n%chalk/server/v1/timeserieschart.proto\x12\x0f\x63halk.server.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"2\n\x07PointV2\x12\x19\n\x05value\x18\x01 \x01(\x01H\x00R\x05value\x88\x01\x01:\x02\x18\x01\x42\x08\n\x06_value"p\n\x0cTimeSeriesV2\x12\x30\n\x06points\x18\x01 \x03(\x0b\x32\x18.chalk.server.v1.PointV2R\x06points\x12\x14\n\x05label\x18\x02 \x01(\tR\x05label\x12\x14\n\x05units\x18\x03 \x01(\tR\x05units:\x02\x18\x01"\xdb\x01\n\x11TimeSeriesChartV2\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12\x35\n\x06series\x18\x02 \x03(\x0b\x32\x1d.chalk.server.v1.TimeSeriesV2R\x06series\x12\x35\n\x08x_series\x18\x03 \x03(\x0b\x32\x1a.google.protobuf.TimestampR\x07xSeries\x12>\n\rwindow_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod:\x02\x18\x01\x42\x9d\x01\n\x13\x63om.chalk.server.v1B\x14TimeserieschartProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.timeserieschart_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\024TimeserieschartProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_POINTV2"]._options = None
    _globals["_POINTV2"]._serialized_options = b"\030\001"
    _globals["_TIMESERIESV2"]._options = None
    _globals["_TIMESERIESV2"]._serialized_options = b"\030\001"
    _globals["_TIMESERIESCHARTV2"]._options = None
    _globals["_TIMESERIESCHARTV2"]._serialized_options = b"\030\001"
    _globals["_POINTV2"]._serialized_start = 123
    _globals["_POINTV2"]._serialized_end = 173
    _globals["_TIMESERIESV2"]._serialized_start = 175
    _globals["_TIMESERIESV2"]._serialized_end = 287
    _globals["_TIMESERIESCHARTV2"]._serialized_start = 290
    _globals["_TIMESERIESCHARTV2"]._serialized_end = 509
# @@protoc_insertion_point(module_scope)
