# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/deployment.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n chalk/server/v1/deployment.proto\x12\x0f\x63halk.server.v1\x1a\x1fgoogle/protobuf/timestamp.proto"\x88\x01\n\x0eInstanceSizing\x12(\n\rmin_instances\x18\x01 \x01(\rH\x00R\x0cminInstances\x88\x01\x01\x12(\n\rmax_instances\x18\x02 \x01(\rH\x01R\x0cmaxInstances\x88\x01\x01\x42\x10\n\x0e_min_instancesB\x10\n\x0e_max_instances"\xde\x0b\n\nDeployment\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12\x39\n\x06status\x18\x03 \x01(\x0e\x32!.chalk.server.v1.DeploymentStatusR\x06status\x12\'\n\x0f\x64\x65ployment_tags\x18\x04 \x03(\tR\x0e\x64\x65ploymentTags\x12$\n\x0e\x63loud_build_id\x18\x05 \x01(\tR\x0c\x63loudBuildId\x12!\n\x0ctriggered_by\x18\x06 \x01(\tR\x0btriggeredBy\x12\x38\n\x15requirements_filepath\x18\x07 \x01(\tH\x00R\x14requirementsFilepath\x88\x01\x01\x12\x34\n\x13\x64ockerfile_filepath\x18\x08 \x01(\tH\x01R\x12\x64ockerfileFilepath\x88\x01\x01\x12\x1d\n\x07runtime\x18\t \x01(\tH\x02R\x07runtime\x88\x01\x01\x12\'\n\x0f\x63halkpy_version\x18\n \x01(\tR\x0e\x63halkpyVersion\x12.\n\x13raw_dependency_hash\x18\x0b \x01(\tR\x11rawDependencyHash\x12\x37\n\x15\x66inal_dependency_hash\x18\x0c \x01(\tH\x03R\x13\x66inalDependencyHash\x88\x01\x01\x12\x37\n\x15is_preview_deployment\x18\r \x01(\x08H\x04R\x13isPreviewDeployment\x88\x01\x01\x12\x39\n\ncreated_at\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x1d\n\ngit_commit\x18\x10 \x01(\tR\tgitCommit\x12\x15\n\x06git_pr\x18\x11 \x01(\tR\x05gitPr\x12\x1d\n\ngit_branch\x18\x12 \x01(\tR\tgitBranch\x12(\n\x10git_author_email\x18\x13 \x01(\tR\x0egitAuthorEmail\x12\x16\n\x06\x62ranch\x18\x14 \x01(\tR\x06\x62ranch\x12)\n\x10project_settings\x18\x15 \x01(\tR\x0fprojectSettings\x12\x32\n\x12requirements_files\x18\x16 \x01(\tH\x05R\x11requirementsFiles\x88\x01\x01\x12\x17\n\x07git_tag\x18\x17 \x01(\tR\x06gitTag\x12$\n\x0e\x62\x61se_image_sha\x18\x18 \x01(\tR\x0c\x62\x61seImageSha\x12\x46\n\x11status_changed_at\x18\x19 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0fstatusChangedAt\x12;\n\x17pinned_platform_version\x18\x1a \x01(\tH\x06R\x15pinnedPlatformVersion\x88\x01\x01\x12\x39\n\x16preview_deployment_tag\x18\x1b \x01(\tH\x07R\x14previewDeploymentTag\x88\x01\x01\x12T\n\x0eprofiling_mode\x18\x1c \x01(\x0e\x32(.chalk.server.v1.DeploymentProfilingModeH\x08R\rprofilingMode\x88\x01\x01\x42\x18\n\x16_requirements_filepathB\x16\n\x14_dockerfile_filepathB\n\n\x08_runtimeB\x18\n\x16_final_dependency_hashB\x18\n\x16_is_preview_deploymentB\x15\n\x13_requirements_filesB\x1a\n\x18_pinned_platform_versionB\x19\n\x17_preview_deployment_tagB\x11\n\x0f_profiling_mode*\xde\x03\n\x10\x44\x65ploymentStatus\x12!\n\x1d\x44\x45PLOYMENT_STATUS_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_UNKNOWN\x10\x01\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_PENDING\x10\x02\x12\x1c\n\x18\x44\x45PLOYMENT_STATUS_QUEUED\x10\x03\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_WORKING\x10\x04\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_SUCCESS\x10\x05\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_FAILURE\x10\x06\x12$\n DEPLOYMENT_STATUS_INTERNAL_ERROR\x10\x07\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_TIMEOUT\x10\x08\x12\x1f\n\x1b\x44\x45PLOYMENT_STATUS_CANCELLED\x10\t\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_EXPIRED\x10\n\x12!\n\x1d\x44\x45PLOYMENT_STATUS_BOOT_ERRORS\x10\x0b\x12%\n!DEPLOYMENT_STATUS_AWAITING_SOURCE\x10\x0c\x12\x1f\n\x1b\x44\x45PLOYMENT_STATUS_DEPLOYING\x10\r*\x8a\x01\n\x17\x44\x65ploymentProfilingMode\x12)\n%DEPLOYMENT_PROFILING_MODE_UNSPECIFIED\x10\x00\x12"\n\x1e\x44\x45PLOYMENT_PROFILING_MODE_NONE\x10\x01\x12 \n\x1c\x44\x45PLOYMENT_PROFILING_MODE_O2\x10\x02\x42\x98\x01\n\x13\x63om.chalk.server.v1B\x0f\x44\x65ploymentProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.deployment_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\017DeploymentProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_DEPLOYMENTSTATUS"]._serialized_start = 1731
    _globals["_DEPLOYMENTSTATUS"]._serialized_end = 2209
    _globals["_DEPLOYMENTPROFILINGMODE"]._serialized_start = 2212
    _globals["_DEPLOYMENTPROFILINGMODE"]._serialized_end = 2350
    _globals["_INSTANCESIZING"]._serialized_start = 87
    _globals["_INSTANCESIZING"]._serialized_end = 223
    _globals["_DEPLOYMENT"]._serialized_start = 226
    _globals["_DEPLOYMENT"]._serialized_end = 1728
# @@protoc_insertion_point(module_scope)
