"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.cli_pb2 import (
    GetVersionsRequest,
    GetVersionsResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class CommandLineInterfaceServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    GetVersions: UnaryUnaryMultiCallable[
        GetVersionsRequest,
        GetVersionsResponse,
    ]

class CommandLineInterfaceServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def GetVersions(
        self,
        request: GetVersionsRequest,
        context: ServicerContext,
    ) -> GetVersionsResponse: ...

def add_CommandLineInterfaceServiceServicer_to_server(
    servicer: CommandLineInterfaceServiceServicer, server: Server
) -> None: ...
