# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/http_engine_proxy.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import chalk_error_pb2 as chalk_dot_common_dot_v1_dot_chalk__error__pb2
from chalk._gen.chalk.common.v1 import dataset_response_pb2 as chalk_dot_common_dot_v1_dot_dataset__response__pb2
from chalk._gen.chalk.common.v1 import offline_query_pb2 as chalk_dot_common_dot_v1_dot_offline__query__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\'chalk/server/v1/http_engine_proxy.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a!chalk/common/v1/chalk_error.proto\x1a&chalk/common/v1/dataset_response.proto\x1a#chalk/common/v1/offline_query.proto"[\n\x19\x43reateOfflineQueryRequest\x12>\n\x07request\x18\x01 \x01(\x0b\x32$.chalk.common.v1.OfflineQueryRequestR\x07request"\xd1\x02\n\x1a\x43reateOfflineQueryResponse\x12#\n\x0bis_finished\x18\x01 \x01(\x08\x42\x02\x18\x01R\nisFinished\x12\x1c\n\x07version\x18\x02 \x01(\x05\x42\x02\x18\x01R\x07version\x12\x16\n\x04urls\x18\x03 \x03(\tB\x02\x18\x01R\x04urls\x12\x37\n\x06\x65rrors\x18\x04 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorB\x02\x18\x01R\x06\x65rrors\x12\x46\n\x07\x63olumns\x18\x05 \x01(\x0b\x32#.chalk.common.v1.ColumnMetadataListB\x02\x18\x01H\x00R\x07\x63olumns\x88\x01\x01\x12K\n\x10\x64\x61taset_response\x18\x06 \x01(\x0b\x32 .chalk.common.v1.DatasetResponseR\x0f\x64\x61tasetResponseB\n\n\x08_columns2\x8c\x01\n\x16HTTPEngineProxyService\x12r\n\x12\x43reateOfflineQuery\x12*.chalk.server.v1.CreateOfflineQueryRequest\x1a+.chalk.server.v1.CreateOfflineQueryResponse"\x03\x80}\x04\x42\x9d\x01\n\x13\x63om.chalk.server.v1B\x14HttpEngineProxyProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.http_engine_proxy_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\024HttpEngineProxyProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["is_finished"]._options = None
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["is_finished"]._serialized_options = b"\030\001"
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["version"]._options = None
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["version"]._serialized_options = b"\030\001"
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["urls"]._options = None
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["urls"]._serialized_options = b"\030\001"
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["errors"]._options = None
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["errors"]._serialized_options = b"\030\001"
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["columns"]._options = None
    _globals["_CREATEOFFLINEQUERYRESPONSE"].fields_by_name["columns"]._serialized_options = b"\030\001"
    _globals["_HTTPENGINEPROXYSERVICE"].methods_by_name["CreateOfflineQuery"]._options = None
    _globals["_HTTPENGINEPROXYSERVICE"].methods_by_name["CreateOfflineQuery"]._serialized_options = b"\200}\004"
    _globals["_CREATEOFFLINEQUERYREQUEST"]._serialized_start = 205
    _globals["_CREATEOFFLINEQUERYREQUEST"]._serialized_end = 296
    _globals["_CREATEOFFLINEQUERYRESPONSE"]._serialized_start = 299
    _globals["_CREATEOFFLINEQUERYRESPONSE"]._serialized_end = 636
    _globals["_HTTPENGINEPROXYSERVICE"]._serialized_start = 639
    _globals["_HTTPENGINEPROXYSERVICE"]._serialized_end = 779
# @@protoc_insertion_point(module_scope)
