"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.scheduled_query_run_pb2 import (
    GetScheduledQueryRunRequest,
    GetScheduledQueryRunResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class ScheduledQueryServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    GetScheduledQueryRun: UnaryUnaryMultiCallable[
        GetScheduledQueryRunRequest,
        GetScheduledQueryRunResponse,
    ]

class ScheduledQueryServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def GetScheduledQueryRun(
        self,
        request: GetScheduledQueryRunRequest,
        context: ServicerContext,
    ) -> GetScheduledQueryRunResponse: ...

def add_ScheduledQueryServiceServicer_to_server(servicer: ScheduledQueryServiceServicer, server: Server) -> None: ...
