# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/auth/v1/audit.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x19\x63halk/auth/v1/audit.proto\x12\rchalk.auth.v1\x1a google/protobuf/descriptor.proto"a\n\x0c\x41uditOptions\x12/\n\x05level\x18\x01 \x01(\x0e\x32\x19.chalk.auth.v1.AuditLevelR\x05level\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription*V\n\nAuditLevel\x12\x1b\n\x17\x41UDIT_LEVEL_UNSPECIFIED\x10\x00\x12\x16\n\x12\x41UDIT_LEVEL_ERRORS\x10\x01\x12\x13\n\x0f\x41UDIT_LEVEL_ALL\x10\x02:V\n\x05\x61udit\x12\x1e.google.protobuf.MethodOptions\x18\xb1\xea\x01 \x01(\x0b\x32\x1b.chalk.auth.v1.AuditOptionsR\x05\x61udit\x88\x01\x01\x42\x85\x01\n\x11\x63om.chalk.auth.v1B\nAuditProtoP\x01Z\x0e\x61uth/v1;authv1\xa2\x02\x03\x43\x41X\xaa\x02\rChalk.Auth.V1\xca\x02\rChalk\\Auth\\V1\xe2\x02\x19\x43halk\\Auth\\V1\\GPBMetadata\xea\x02\x0f\x43halk::Auth::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.auth.v1.audit_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\021com.chalk.auth.v1B\nAuditProtoP\001Z\016auth/v1;authv1\242\002\003CAX\252\002\rChalk.Auth.V1\312\002\rChalk\\Auth\\V1\342\002\031Chalk\\Auth\\V1\\GPBMetadata\352\002\017Chalk::Auth::V1"
    _globals["_AUDITLEVEL"]._serialized_start = 177
    _globals["_AUDITLEVEL"]._serialized_end = 263
    _globals["_AUDITOPTIONS"]._serialized_start = 78
    _globals["_AUDITOPTIONS"]._serialized_end = 175
# @@protoc_insertion_point(module_scope)
