# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/arrow/v1/arrow.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1a\x63halk/arrow/v1/arrow.proto\x12\x0e\x63halk.arrow.v1"\xb8\x01\n\x06Schema\x12/\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x07\x63olumns\x12@\n\x08metadata\x18\x02 \x03(\x0b\x32$.chalk.arrow.v1.Schema.MetadataEntryR\x08metadata\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xa2\x02\n\x05\x46ield\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x38\n\narrow_type\x18\x02 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeR\tarrowType\x12\x1a\n\x08nullable\x18\x03 \x01(\x08R\x08nullable\x12\x31\n\x08\x63hildren\x18\x04 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x08\x63hildren\x12?\n\x08metadata\x18\x05 \x03(\x0b\x32#.chalk.arrow.v1.Field.MetadataEntryR\x08metadata\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01")\n\x0f\x46ixedSizeBinary\x12\x16\n\x06length\x18\x01 \x01(\x05R\x06length"^\n\tTimestamp\x12\x35\n\ttime_unit\x18\x01 \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitR\x08timeUnit\x12\x1a\n\x08timezone\x18\x02 \x01(\tR\x08timezone"=\n\x07\x44\x65\x63imal\x12\x1c\n\tprecision\x18\x01 \x01(\x05R\tprecision\x12\x14\n\x05scale\x18\x02 \x01(\x05R\x05scale"<\n\x04List\x12\x34\n\nfield_type\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\tfieldType"b\n\rFixedSizeList\x12\x34\n\nfield_type\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\tfieldType\x12\x1b\n\tlist_size\x18\x02 \x01(\x05R\x08listSize"G\n\x06Struct\x12=\n\x0fsub_field_types\x18\x01 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\rsubFieldTypes"\x90\x01\n\x03Map\x12\x32\n\tkey_field\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\x08keyField\x12\x34\n\nitem_field\x18\x02 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\titemField\x12\x1f\n\x0bkeys_sorted\x18\x03 \x01(\x08R\nkeysSorted"`\n\x0fScalarListValue\x12\x1d\n\narrow_data\x18\x01 \x01(\x0cR\tarrowData\x12.\n\x06schema\x18\x02 \x01(\x0b\x32\x16.chalk.arrow.v1.SchemaR\x06schema"\x8a\x01\n\x11ScalarTime32Value\x12\x30\n\x13time32_second_value\x18\x01 \x01(\x05H\x00R\x11time32SecondValue\x12:\n\x18time32_millisecond_value\x18\x02 \x01(\x05H\x00R\x16time32MillisecondValueB\x07\n\x05value"\x92\x01\n\x11ScalarTime64Value\x12:\n\x18time64_microsecond_value\x18\x01 \x01(\x03H\x00R\x16time64MicrosecondValue\x12\x38\n\x17time64_nanosecond_value\x18\x02 \x01(\x03H\x00R\x15time64NanosecondValueB\x07\n\x05value"\x8f\x02\n\x14ScalarTimestampValue\x12\x36\n\x16time_microsecond_value\x18\x01 \x01(\x03H\x00R\x14timeMicrosecondValue\x12\x34\n\x15time_nanosecond_value\x18\x02 \x01(\x03H\x00R\x13timeNanosecondValue\x12,\n\x11time_second_value\x18\x03 \x01(\x03H\x00R\x0ftimeSecondValue\x12\x36\n\x16time_millisecond_value\x18\x04 \x01(\x03H\x00R\x14timeMillisecondValue\x12\x1a\n\x08timezone\x18\x05 \x01(\tR\x08timezoneB\x07\n\x05value"|\n\x0bStructValue\x12>\n\x0c\x66ield_values\x18\x02 \x03(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x0b\x66ieldValues\x12-\n\x06\x66ields\x18\x03 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x06\x66ields"G\n\x15ScalarFixedSizeBinary\x12\x16\n\x06values\x18\x01 \x01(\x0cR\x06values\x12\x16\n\x06length\x18\x02 \x01(\x05R\x06length"X\n\x0c\x44\x65\x63imalValue\x12\x14\n\x05value\x18\x01 \x01(\x0cR\x05value\x12\x1c\n\tprecision\x18\x02 \x01(\x05R\tprecision\x12\x14\n\x05scale\x18\x03 \x01(\x05R\x05scale"]\n\tExtension\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12<\n\x0cstorage_type\x18\x02 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeR\x0bstorageType"\x94\x01\n\x0e\x45xtensionValue\x12@\n\x0e\x65xtension_type\x18\x01 \x01(\x0b\x32\x19.chalk.arrow.v1.ExtensionR\rextensionType\x12@\n\rstorage_value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x0cstorageValue"\xef\x0e\n\x0bScalarValue\x12:\n\nnull_value\x18\x01 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeH\x00R\tnullValue\x12\x1f\n\nbool_value\x18\x02 \x01(\x08H\x00R\tboolValue\x12%\n\rfloat64_value\x18\x03 \x01(\x01H\x00R\x0c\x66loat64Value\x12!\n\x0bint64_value\x18\x04 \x01(\x03H\x00R\nint64Value\x12*\n\x10large_utf8_value\x18\x05 \x01(\tH\x00R\x0elargeUtf8Value\x12O\n\x0ftimestamp_value\x18\x06 \x01(\x0b\x32$.chalk.arrow.v1.ScalarTimestampValueH\x00R\x0etimestampValue\x12$\n\rdate_64_value\x18\x07 \x01(\x03H\x00R\x0b\x64\x61te64Value\x12@\n\x0cstruct_value\x18\x08 \x01(\x0b\x32\x1b.chalk.arrow.v1.StructValueH\x00R\x0bstructValue\x12K\n\x10large_list_value\x18\t \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x0elargeListValue\x12\x46\n\x0ctime64_value\x18\n \x01(\x0b\x32!.chalk.arrow.v1.ScalarTime64ValueH\x00R\x0btime64Value\x12\x34\n\x15\x64uration_second_value\x18\x10 \x01(\x03H\x00R\x13\x64urationSecondValue\x12>\n\x1a\x64uration_millisecond_value\x18\x11 \x01(\x03H\x00R\x18\x64urationMillisecondValue\x12>\n\x1a\x64uration_microsecond_value\x18\x12 \x01(\x03H\x00R\x18\x64urationMicrosecondValue\x12<\n\x19\x64uration_nanosecond_value\x18\x13 \x01(\x03H\x00R\x17\x64urationNanosecondValue\x12\x1f\n\nutf8_value\x18\x14 \x01(\tH\x00R\tutf8Value\x12\x1f\n\nint8_value\x18\x15 \x01(\x05H\x00R\tint8Value\x12!\n\x0bint16_value\x18\x16 \x01(\x05H\x00R\nint16Value\x12!\n\x0bint32_value\x18\x17 \x01(\x05H\x00R\nint32Value\x12!\n\x0buint8_value\x18\x18 \x01(\rH\x00R\nuint8Value\x12#\n\x0cuint16_value\x18\x19 \x01(\rH\x00R\x0buint16Value\x12#\n\x0cuint32_value\x18\x1a \x01(\rH\x00R\x0buint32Value\x12#\n\x0cuint64_value\x18\x1b \x01(\x04H\x00R\x0buint64Value\x12%\n\rfloat16_value\x18\x1c \x01(\x02H\x00R\x0c\x66loat16Value\x12%\n\rfloat32_value\x18\x1d \x01(\x02H\x00R\x0c\x66loat32Value\x12$\n\rdate_32_value\x18\x1e \x01(\x05H\x00R\x0b\x64\x61te32Value\x12\x46\n\x0ctime32_value\x18\x1f \x01(\x0b\x32!.chalk.arrow.v1.ScalarTime32ValueH\x00R\x0btime32Value\x12@\n\nlist_value\x18  \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\tlistValue\x12T\n\x15\x66ixed_size_list_value\x18! \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x12\x66ixedSizeListValue\x12>\n\tmap_value\x18" \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x08mapValue\x12#\n\x0c\x62inary_value\x18# \x01(\x0cH\x00R\x0b\x62inaryValue\x12.\n\x12large_binary_value\x18$ \x01(\x0cH\x00R\x10largeBinaryValue\x12^\n\x17\x66ixed_size_binary_value\x18% \x01(\x0b\x32%.chalk.arrow.v1.ScalarFixedSizeBinaryH\x00R\x14\x66ixedSizeBinaryValue\x12I\n\x10\x64\x65\x63imal128_value\x18& \x01(\x0b\x32\x1c.chalk.arrow.v1.DecimalValueH\x00R\x0f\x64\x65\x63imal128Value\x12I\n\x10\x64\x65\x63imal256_value\x18\' \x01(\x0b\x32\x1c.chalk.arrow.v1.DecimalValueH\x00R\x0f\x64\x65\x63imal256Value\x12I\n\x0f\x65xtension_value\x18( \x01(\x0b\x32\x1e.chalk.arrow.v1.ExtensionValueH\x00R\x0e\x65xtensionValueB\x07\n\x05value"\x89\x0e\n\tArrowType\x12\x32\n\x04none\x18\x01 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04none\x12\x32\n\x04\x62ool\x18\x02 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04\x62ool\x12\x38\n\x07\x66loat64\x18\x03 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat64\x12\x34\n\x05int64\x18\x04 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int64\x12=\n\nlarge_utf8\x18\x05 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\tlargeUtf8\x12\x39\n\ttimestamp\x18\x06 \x01(\x0b\x32\x19.chalk.arrow.v1.TimestampH\x00R\ttimestamp\x12\x36\n\x06\x64\x61te64\x18\x07 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x64\x61te64\x12\x30\n\x06struct\x18\x08 \x01(\x0b\x32\x16.chalk.arrow.v1.StructH\x00R\x06struct\x12\x35\n\nlarge_list\x18\t \x01(\x0b\x32\x14.chalk.arrow.v1.ListH\x00R\tlargeList\x12\x32\n\x06time64\x18\n \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x06time64\x12\x36\n\x08\x64uration\x18\x10 \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x08\x64uration\x12\x32\n\x04utf8\x18\x11 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04utf8\x12\x32\n\x04int8\x18\x12 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04int8\x12\x34\n\x05int16\x18\x13 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int16\x12\x34\n\x05int32\x18\x14 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int32\x12\x34\n\x05uint8\x18\x15 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05uint8\x12\x36\n\x06uint16\x18\x16 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint16\x12\x36\n\x06uint32\x18\x17 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint32\x12\x36\n\x06uint64\x18\x18 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint64\x12\x38\n\x07\x66loat16\x18\x19 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat16\x12\x38\n\x07\x66loat32\x18\x1a \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat32\x12\x36\n\x06\x64\x61te32\x18\x1b \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x64\x61te32\x12\x32\n\x06time32\x18\x1c \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x06time32\x12*\n\x04list\x18\x1d \x01(\x0b\x32\x14.chalk.arrow.v1.ListH\x00R\x04list\x12G\n\x0f\x66ixed_size_list\x18\x1e \x01(\x0b\x32\x1d.chalk.arrow.v1.FixedSizeListH\x00R\rfixedSizeList\x12\x36\n\x06\x62inary\x18\x1f \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x62inary\x12\x41\n\x0clarge_binary\x18  \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x0blargeBinary\x12,\n\x11\x66ixed_size_binary\x18! \x01(\x05H\x00R\x0f\x66ixedSizeBinary\x12:\n\x0b\x64\x65\x63imal_128\x18" \x01(\x0b\x32\x17.chalk.arrow.v1.DecimalH\x00R\ndecimal128\x12:\n\x0b\x64\x65\x63imal_256\x18# \x01(\x0b\x32\x17.chalk.arrow.v1.DecimalH\x00R\ndecimal256\x12\'\n\x03map\x18$ \x01(\x0b\x32\x13.chalk.arrow.v1.MapH\x00R\x03map\x12\x39\n\textension\x18% \x01(\x0b\x32\x19.chalk.arrow.v1.ExtensionH\x00R\textensionB\x11\n\x0f\x61rrow_type_enum"\x0e\n\x0c\x45mptyMessage*\x8b\x01\n\x08TimeUnit\x12\x19\n\x15TIME_UNIT_UNSPECIFIED\x10\x00\x12\x14\n\x10TIME_UNIT_SECOND\x10\x01\x12\x19\n\x15TIME_UNIT_MILLISECOND\x10\x02\x12\x19\n\x15TIME_UNIT_MICROSECOND\x10\x03\x12\x18\n\x14TIME_UNIT_NANOSECOND\x10\x04*\x9a\x01\n\x12\x46\x65\x61therCompression\x12#\n\x1f\x46\x45\x41THER_COMPRESSION_UNSPECIFIED\x10\x00\x12$\n FEATHER_COMPRESSION_UNCOMPRESSED\x10\x01\x12\x1b\n\x17\x46\x45\x41THER_COMPRESSION_LZ4\x10\x02\x12\x1c\n\x18\x46\x45\x41THER_COMPRESSION_ZSTD\x10\x03\x42z\n\x12\x63om.chalk.arrow.v1B\nArrowProtoP\x01\xa2\x02\x03\x43\x41X\xaa\x02\x0e\x43halk.Arrow.V1\xca\x02\x0e\x43halk\\Arrow\\V1\xe2\x02\x1a\x43halk\\Arrow\\V1\\GPBMetadata\xea\x02\x10\x43halk::Arrow::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.arrow.v1.arrow_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\022com.chalk.arrow.v1B\nArrowProtoP\001\242\002\003CAX\252\002\016Chalk.Arrow.V1\312\002\016Chalk\\Arrow\\V1\342\002\032Chalk\\Arrow\\V1\\GPBMetadata\352\002\020Chalk::Arrow::V1"
    _globals["_SCHEMA_METADATAENTRY"]._options = None
    _globals["_SCHEMA_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_FIELD_METADATAENTRY"]._options = None
    _globals["_FIELD_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_TIMEUNIT"]._serialized_start = 6034
    _globals["_TIMEUNIT"]._serialized_end = 6173
    _globals["_FEATHERCOMPRESSION"]._serialized_start = 6176
    _globals["_FEATHERCOMPRESSION"]._serialized_end = 6330
    _globals["_SCHEMA"]._serialized_start = 47
    _globals["_SCHEMA"]._serialized_end = 231
    _globals["_SCHEMA_METADATAENTRY"]._serialized_start = 172
    _globals["_SCHEMA_METADATAENTRY"]._serialized_end = 231
    _globals["_FIELD"]._serialized_start = 234
    _globals["_FIELD"]._serialized_end = 524
    _globals["_FIELD_METADATAENTRY"]._serialized_start = 172
    _globals["_FIELD_METADATAENTRY"]._serialized_end = 231
    _globals["_FIXEDSIZEBINARY"]._serialized_start = 526
    _globals["_FIXEDSIZEBINARY"]._serialized_end = 567
    _globals["_TIMESTAMP"]._serialized_start = 569
    _globals["_TIMESTAMP"]._serialized_end = 663
    _globals["_DECIMAL"]._serialized_start = 665
    _globals["_DECIMAL"]._serialized_end = 726
    _globals["_LIST"]._serialized_start = 728
    _globals["_LIST"]._serialized_end = 788
    _globals["_FIXEDSIZELIST"]._serialized_start = 790
    _globals["_FIXEDSIZELIST"]._serialized_end = 888
    _globals["_STRUCT"]._serialized_start = 890
    _globals["_STRUCT"]._serialized_end = 961
    _globals["_MAP"]._serialized_start = 964
    _globals["_MAP"]._serialized_end = 1108
    _globals["_SCALARLISTVALUE"]._serialized_start = 1110
    _globals["_SCALARLISTVALUE"]._serialized_end = 1206
    _globals["_SCALARTIME32VALUE"]._serialized_start = 1209
    _globals["_SCALARTIME32VALUE"]._serialized_end = 1347
    _globals["_SCALARTIME64VALUE"]._serialized_start = 1350
    _globals["_SCALARTIME64VALUE"]._serialized_end = 1496
    _globals["_SCALARTIMESTAMPVALUE"]._serialized_start = 1499
    _globals["_SCALARTIMESTAMPVALUE"]._serialized_end = 1770
    _globals["_STRUCTVALUE"]._serialized_start = 1772
    _globals["_STRUCTVALUE"]._serialized_end = 1896
    _globals["_SCALARFIXEDSIZEBINARY"]._serialized_start = 1898
    _globals["_SCALARFIXEDSIZEBINARY"]._serialized_end = 1969
    _globals["_DECIMALVALUE"]._serialized_start = 1971
    _globals["_DECIMALVALUE"]._serialized_end = 2059
    _globals["_EXTENSION"]._serialized_start = 2061
    _globals["_EXTENSION"]._serialized_end = 2154
    _globals["_EXTENSIONVALUE"]._serialized_start = 2157
    _globals["_EXTENSIONVALUE"]._serialized_end = 2305
    _globals["_SCALARVALUE"]._serialized_start = 2308
    _globals["_SCALARVALUE"]._serialized_end = 4211
    _globals["_ARROWTYPE"]._serialized_start = 4214
    _globals["_ARROWTYPE"]._serialized_end = 6015
    _globals["_EMPTYMESSAGE"]._serialized_start = 6017
    _globals["_EMPTYMESSAGE"]._serialized_end = 6031
# @@protoc_insertion_point(module_scope)
