# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/kubernetes/v1/events.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n chalk/kubernetes/v1/events.proto\x12\x13\x63halk.kubernetes.v1"`\n\x1a\x43halkKubernetesEventSeries\x12\x14\n\x05\x63ount\x18\x01 \x01(\x05R\x05\x63ount\x12,\n\x12last_observed_time\x18\x02 \x01(\x03R\x10lastObservedTime"\xa7\x02\n\x14\x43halkKubernetesEvent\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reason\x12\x16\n\x06\x61\x63tion\x18\x03 \x01(\tR\x06\x61\x63tion\x12\x31\n\x14reporting_controller\x18\x04 \x01(\tR\x13reportingController\x12\x1d\n\nevent_time\x18\x05 \x01(\x03R\teventTime\x12\x17\n\x04note\x18\x06 \x01(\tH\x00R\x04note\x88\x01\x01\x12L\n\x06series\x18\x07 \x01(\x0b\x32/.chalk.kubernetes.v1.ChalkKubernetesEventSeriesH\x01R\x06series\x88\x01\x01\x42\x07\n\x05_noteB\t\n\x07_seriesB\xb0\x01\n\x17\x63om.chalk.kubernetes.v1B\x0b\x45ventsProtoP\x01Z\x1akubernetes/v1;kubernetesv1\xa2\x02\x03\x43KX\xaa\x02\x13\x43halk.Kubernetes.V1\xca\x02\x13\x43halk\\Kubernetes\\V1\xe2\x02\x1f\x43halk\\Kubernetes\\V1\\GPBMetadata\xea\x02\x15\x43halk::Kubernetes::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.kubernetes.v1.events_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\027com.chalk.kubernetes.v1B\013EventsProtoP\001Z\032kubernetes/v1;kubernetesv1\242\002\003CKX\252\002\023Chalk.Kubernetes.V1\312\002\023Chalk\\Kubernetes\\V1\342\002\037Chalk\\Kubernetes\\V1\\GPBMetadata\352\002\025Chalk::Kubernetes::V1"
    _globals["_CHALKKUBERNETESEVENTSERIES"]._serialized_start = 57
    _globals["_CHALKKUBERNETESEVENTSERIES"]._serialized_end = 153
    _globals["_CHALKKUBERNETESEVENT"]._serialized_start = 156
    _globals["_CHALKKUBERNETESEVENT"]._serialized_end = 451
# @@protoc_insertion_point(module_scope)
