# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/prompt.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/server/v1/prompt.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a google/protobuf/field_mask.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto">\n\x0ePromptTemplate\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message"\xbb\x02\n\x0eProviderConfig\x12\x1a\n\x08provider\x18\x01 \x01(\tR\x08provider\x12\x1d\n\nmodel_name\x18\x02 \x01(\tR\tmodelName\x12O\n\nparameters\x18\x03 \x03(\x0b\x32/.chalk.server.v1.ProviderConfig.ParametersEntryR\nparameters\x12\x30\n\x11structured_output\x18\x04 \x01(\tH\x00R\x10structuredOutput\x88\x01\x01\x1aU\n\x0fParametersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\x14\n\x12_structured_output"\xf7\x04\n\rPromptVariant\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x03 \x01(\tR\x0c\x64\x65ploymentId\x12+\n\x0fnamed_prompt_id\x18\x04 \x01(\tH\x00R\rnamedPromptId\x88\x01\x01\x12/\n\x11\x65valuation_run_id\x18\x05 \x01(\tH\x01R\x0f\x65valuationRunId\x88\x01\x01\x12$\n\x0b\x63ommit_hash\x18\x06 \x01(\tH\x02R\ncommitHash\x88\x01\x01\x12=\n\ttemplates\x18\x07 \x03(\x0b\x32\x1f.chalk.server.v1.PromptTemplateR\ttemplates\x12\x1c\n\tvariables\x18\x08 \x03(\tR\tvariables\x12H\n\x0fprovider_config\x18\t \x01(\x0b\x32\x1f.chalk.server.v1.ProviderConfigR\x0eproviderConfig\x12&\n\x0cvariant_hash\x18\n \x01(\tH\x03R\x0bvariantHash\x88\x01\x01\x12"\n\ncreated_by\x18\x0b \x01(\tH\x04R\tcreatedBy\x88\x01\x01\x12\x39\n\ncreated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAtB\x12\n\x10_named_prompt_idB\x14\n\x12_evaluation_run_idB\x0e\n\x0c_commit_hashB\x0f\n\r_variant_hashB\r\n\x0b_created_by"\xb5\x04\n\x0bNamedPrompt\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x03 \x01(\tR\x0c\x64\x65ploymentId\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x05 \x01(\tR\x0b\x64\x65scription\x12\x12\n\x04tags\x18\x06 \x03(\tR\x04tags\x12"\n\ncreated_by\x18\x07 \x01(\tH\x00R\tcreatedBy\x88\x01\x01\x12\x39\n\ncreated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12>\n\nupdated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01R\tupdatedAt\x88\x01\x01\x12@\n\x0b\x61rchived_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x02R\narchivedAt\x88\x01\x01\x12W\n\x15latest_prompt_variant\x18\x0b \x01(\x0b\x32\x1e.chalk.server.v1.PromptVariantH\x03R\x13latestPromptVariant\x88\x01\x01\x42\r\n\x0b_created_byB\r\n\x0b_updated_atB\x0e\n\x0c_archived_atB\x18\n\x16_latest_prompt_variant"\xfa\x08\n\x13PromptEvaluationRun\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x03 \x01(\tR\x0c\x64\x65ploymentId\x12\x1f\n\x0b\x62ranch_name\x18\x04 \x01(\tR\nbranchName\x12\x1d\n\ndataset_id\x18\x05 \x01(\tR\tdatasetId\x12.\n\x13\x64\x61taset_revision_id\x18\x06 \x01(\tR\x11\x64\x61tasetRevisionId\x12.\n\x10reference_output\x18\x07 \x01(\tH\x00R\x0freferenceOutput\x88\x01\x01\x12\x1e\n\nevaluators\x18\x08 \x03(\tR\nevaluators\x12*\n\x11offline_query_ids\x18\t \x03(\tR\x0fofflineQueryIds\x12\x37\n\x18related_named_prompt_ids\x18\n \x03(\tR\x15relatedNamedPromptIds\x12\x34\n\x16related_evaluation_ids\x18\x0b \x03(\tR\x14relatedEvaluationIds\x12g\n\x11\x61ggregate_metrics\x18\x0c \x03(\x0b\x32:.chalk.server.v1.PromptEvaluationRun.AggregateMetricsEntryR\x10\x61ggregateMetrics\x12O\n\tmeta_data\x18\r \x03(\x0b\x32\x32.chalk.server.v1.PromptEvaluationRun.MetaDataEntryR\x08metaData\x12"\n\ncreated_by\x18\x0e \x01(\tH\x01R\tcreatedBy\x88\x01\x01\x12\x39\n\ncreated_at\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12&\n\x0c\x64\x61taset_name\x18\x10 \x01(\tH\x02R\x0b\x64\x61tasetName\x88\x01\x01\x12;\n\x1arelated_named_prompt_names\x18\x11 \x03(\tR\x17relatedNamedPromptNames\x12G\n\x0fprompt_variants\x18\x12 \x03(\x0b\x32\x1e.chalk.server.v1.PromptVariantR\x0epromptVariants\x1a[\n\x15\x41ggregateMetricsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x1aS\n\rMetaDataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\x13\n\x11_reference_outputB\r\n\x0b_created_byB\x0f\n\r_dataset_name"f\n\x17ListNamedPromptsRequest\x12\x1b\n\x06\x63ursor\x18\x01 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limit"\x93\x01\n\x18ListNamedPromptsResponse\x12\x41\n\rnamed_prompts\x18\x01 \x03(\x0b\x32\x1c.chalk.server.v1.NamedPromptR\x0cnamedPrompts\x12$\n\x0bnext_cursor\x18\x02 \x01(\tH\x00R\nnextCursor\x88\x01\x01\x42\x0e\n\x0c_next_cursor"?\n\x15GetNamedPromptRequest\x12&\n\x0fnamed_prompt_id\x18\x01 \x01(\tR\rnamedPromptId"o\n\x16GetNamedPromptResponse\x12\x44\n\x0cnamed_prompt\x18\x01 \x01(\x0b\x32\x1c.chalk.server.v1.NamedPromptH\x00R\x0bnamedPrompt\x88\x01\x01\x42\x0f\n\r_named_prompt"\xa1\x01\n\x16PromptVariantOperation\x12=\n\ttemplates\x18\x01 \x03(\x0b\x32\x1f.chalk.server.v1.PromptTemplateR\ttemplates\x12H\n\x0fprovider_config\x18\x02 \x01(\x0b\x32\x1f.chalk.server.v1.ProviderConfigR\x0eproviderConfig"\xb4\x01\n\x18\x43reateNamedPromptRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12\x12\n\x04tags\x18\x03 \x03(\tR\x04tags\x12N\n\x0eprompt_variant\x18\x04 \x01(\x0b\x32\'.chalk.server.v1.PromptVariantOperationR\rpromptVariant"\\\n\x19\x43reateNamedPromptResponse\x12?\n\x0cnamed_prompt\x18\x01 \x01(\x0b\x32\x1c.chalk.server.v1.NamedPromptR\x0bnamedPrompt"\xc3\x02\n\x1aUpdateNamedPromptOperation\x12\x17\n\x04name\x18\x01 \x01(\tH\x00R\x04name\x88\x01\x01\x12%\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x01R\x0b\x64\x65scription\x88\x01\x01\x12\x12\n\x04tags\x18\x03 \x03(\tR\x04tags\x12S\n\x0eprompt_variant\x18\x04 \x01(\x0b\x32\'.chalk.server.v1.PromptVariantOperationH\x02R\rpromptVariant\x88\x01\x01\x12@\n\x0b\x61rchived_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x03R\narchivedAt\x88\x01\x01\x42\x07\n\x05_nameB\x0e\n\x0c_descriptionB\x11\n\x0f_prompt_variantB\x0e\n\x0c_archived_at"\xc4\x01\n\x18UpdateNamedPromptRequest\x12&\n\x0fnamed_prompt_id\x18\x01 \x01(\tR\rnamedPromptId\x12\x43\n\x06update\x18\x02 \x01(\x0b\x32+.chalk.server.v1.UpdateNamedPromptOperationR\x06update\x12;\n\x0bupdate_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskR\nupdateMask"\\\n\x19UpdateNamedPromptResponse\x12?\n\x0cnamed_prompt\x18\x01 \x01(\x0b\x32\x1c.chalk.server.v1.NamedPromptR\x0bnamedPrompt"\xf0\x01\n\x19ListPromptVariantsRequest\x12+\n\x0fnamed_prompt_id\x18\x01 \x01(\tH\x00R\rnamedPromptId\x88\x01\x01\x12/\n\x11\x65valuation_run_id\x18\x02 \x01(\tH\x01R\x0f\x65valuationRunId\x88\x01\x01\x12\x1b\n\x06\x63ursor\x18\x03 \x01(\tH\x02R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x04 \x01(\x05H\x03R\x05limit\x88\x01\x01\x42\x12\n\x10_named_prompt_idB\x14\n\x12_evaluation_run_idB\t\n\x07_cursorB\x08\n\x06_limit"\x9b\x01\n\x1aListPromptVariantsResponse\x12G\n\x0fprompt_variants\x18\x01 \x03(\x0b\x32\x1e.chalk.server.v1.PromptVariantR\x0epromptVariants\x12$\n\x0bnext_cursor\x18\x02 \x01(\tH\x00R\nnextCursor\x88\x01\x01\x42\x0e\n\x0c_next_cursor"\xaf\x01\n\x1fListPromptEvaluationRunsRequest\x12\x1b\n\x06\x63ursor\x18\x01 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x12+\n\x0fnamed_prompt_id\x18\x03 \x01(\tH\x02R\rnamedPromptId\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limitB\x12\n\x10_named_prompt_id"\xa7\x01\n ListPromptEvaluationRunsResponse\x12M\n\x0f\x65valuation_runs\x18\x01 \x03(\x0b\x32$.chalk.server.v1.PromptEvaluationRunR\x0e\x65valuationRuns\x12$\n\x0bnext_cursor\x18\x02 \x01(\tH\x00R\nnextCursor\x88\x01\x01\x42\x0e\n\x0c_next_cursor"\xdb\x02\n\x1dGetPromptEvaluationRunRequest\x12,\n\x11\x65valuation_run_id\x18\x01 \x01(\tH\x00R\x0f\x65valuationRunId\x12h\n\x0fprefill_options\x18\x02 \x01(\x0b\x32=.chalk.server.v1.GetPromptEvaluationRunRequest.PrefillOptionsH\x00R\x0eprefillOptions\x1a\x98\x01\n\x0ePrefillOptions\x12+\n\x0fnamed_prompt_id\x18\x01 \x01(\tH\x00R\rnamedPromptId\x88\x01\x01\x12/\n\x11\x65valuation_run_id\x18\x02 \x01(\tH\x01R\x0f\x65valuationRunId\x88\x01\x01\x42\x12\n\x10_named_prompt_idB\x14\n\x12_evaluation_run_idB\x07\n\x05query"\x85\x01\n\x1eGetPromptEvaluationRunResponse\x12P\n\x0e\x65valuation_run\x18\x01 \x01(\x0b\x32$.chalk.server.v1.PromptEvaluationRunH\x00R\revaluationRun\x88\x01\x01\x42\x11\n\x0f_evaluation_run"\xeb\x04\n CreatePromptEvaluationRunRequest\x12\x1f\n\x0b\x62ranch_name\x18\x01 \x01(\tR\nbranchName\x12\x1d\n\ndataset_id\x18\x02 \x01(\tR\tdatasetId\x12.\n\x13\x64\x61taset_revision_id\x18\x03 \x01(\tR\x11\x64\x61tasetRevisionId\x12.\n\x10reference_output\x18\x04 \x01(\tH\x00R\x0freferenceOutput\x88\x01\x01\x12\x1e\n\nevaluators\x18\x05 \x03(\tR\nevaluators\x12\x37\n\x18related_named_prompt_ids\x18\x06 \x03(\tR\x15relatedNamedPromptIds\x12\x34\n\x16related_evaluation_ids\x18\x07 \x03(\tR\x14relatedEvaluationIds\x12\\\n\tmeta_data\x18\x08 \x03(\x0b\x32?.chalk.server.v1.CreatePromptEvaluationRunRequest.MetaDataEntryR\x08metaData\x12P\n\x0fprompt_variants\x18\t \x03(\x0b\x32\'.chalk.server.v1.PromptVariantOperationR\x0epromptVariants\x1aS\n\rMetaDataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\x13\n\x11_reference_output"p\n!CreatePromptEvaluationRunResponse\x12K\n\x0e\x65valuation_run\x18\x01 \x01(\x0b\x32$.chalk.server.v1.PromptEvaluationRunR\revaluationRun2\xdc\x07\n\rPromptService\x12o\n\x10ListNamedPrompts\x12(.chalk.server.v1.ListNamedPromptsRequest\x1a).chalk.server.v1.ListNamedPromptsResponse"\x06\x90\x02\x01\x80}\x0b\x12i\n\x0eGetNamedPrompt\x12&.chalk.server.v1.GetNamedPromptRequest\x1a\'.chalk.server.v1.GetNamedPromptResponse"\x06\x90\x02\x01\x80}\r\x12o\n\x11\x43reateNamedPrompt\x12).chalk.server.v1.CreateNamedPromptRequest\x1a*.chalk.server.v1.CreateNamedPromptResponse"\x03\x80}\x0c\x12o\n\x11UpdateNamedPrompt\x12).chalk.server.v1.UpdateNamedPromptRequest\x1a*.chalk.server.v1.UpdateNamedPromptResponse"\x03\x80}\x0c\x12u\n\x12ListPromptVariants\x12*.chalk.server.v1.ListPromptVariantsRequest\x1a+.chalk.server.v1.ListPromptVariantsResponse"\x06\x90\x02\x01\x80}\x0b\x12\x87\x01\n\x18ListPromptEvaluationRuns\x12\x30.chalk.server.v1.ListPromptEvaluationRunsRequest\x1a\x31.chalk.server.v1.ListPromptEvaluationRunsResponse"\x06\x90\x02\x01\x80}\x04\x12\x81\x01\n\x16GetPromptEvaluationRun\x12..chalk.server.v1.GetPromptEvaluationRunRequest\x1a/.chalk.server.v1.GetPromptEvaluationRunResponse"\x06\x90\x02\x01\x80}\x04\x12\x87\x01\n\x19\x43reatePromptEvaluationRun\x12\x31.chalk.server.v1.CreatePromptEvaluationRunRequest\x1a\x32.chalk.server.v1.CreatePromptEvaluationRunResponse"\x03\x80}\x04\x42\x94\x01\n\x13\x63om.chalk.server.v1B\x0bPromptProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.prompt_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\013PromptProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_PROVIDERCONFIG_PARAMETERSENTRY"]._options = None
    _globals["_PROVIDERCONFIG_PARAMETERSENTRY"]._serialized_options = b"8\001"
    _globals["_PROMPTEVALUATIONRUN_AGGREGATEMETRICSENTRY"]._options = None
    _globals["_PROMPTEVALUATIONRUN_AGGREGATEMETRICSENTRY"]._serialized_options = b"8\001"
    _globals["_PROMPTEVALUATIONRUN_METADATAENTRY"]._options = None
    _globals["_PROMPTEVALUATIONRUN_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST_METADATAENTRY"]._options = None
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_PROMPTSERVICE"].methods_by_name["ListNamedPrompts"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["ListNamedPrompts"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_PROMPTSERVICE"].methods_by_name["GetNamedPrompt"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["GetNamedPrompt"]._serialized_options = b"\220\002\001\200}\r"
    _globals["_PROMPTSERVICE"].methods_by_name["CreateNamedPrompt"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["CreateNamedPrompt"]._serialized_options = b"\200}\014"
    _globals["_PROMPTSERVICE"].methods_by_name["UpdateNamedPrompt"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["UpdateNamedPrompt"]._serialized_options = b"\200}\014"
    _globals["_PROMPTSERVICE"].methods_by_name["ListPromptVariants"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["ListPromptVariants"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_PROMPTSERVICE"].methods_by_name["ListPromptEvaluationRuns"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name[
        "ListPromptEvaluationRuns"
    ]._serialized_options = b"\220\002\001\200}\004"
    _globals["_PROMPTSERVICE"].methods_by_name["GetPromptEvaluationRun"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["GetPromptEvaluationRun"]._serialized_options = b"\220\002\001\200}\004"
    _globals["_PROMPTSERVICE"].methods_by_name["CreatePromptEvaluationRun"]._options = None
    _globals["_PROMPTSERVICE"].methods_by_name["CreatePromptEvaluationRun"]._serialized_options = b"\200}\004"
    _globals["_PROMPTTEMPLATE"]._serialized_start = 179
    _globals["_PROMPTTEMPLATE"]._serialized_end = 241
    _globals["_PROVIDERCONFIG"]._serialized_start = 244
    _globals["_PROVIDERCONFIG"]._serialized_end = 559
    _globals["_PROVIDERCONFIG_PARAMETERSENTRY"]._serialized_start = 452
    _globals["_PROVIDERCONFIG_PARAMETERSENTRY"]._serialized_end = 537
    _globals["_PROMPTVARIANT"]._serialized_start = 562
    _globals["_PROMPTVARIANT"]._serialized_end = 1193
    _globals["_NAMEDPROMPT"]._serialized_start = 1196
    _globals["_NAMEDPROMPT"]._serialized_end = 1761
    _globals["_PROMPTEVALUATIONRUN"]._serialized_start = 1764
    _globals["_PROMPTEVALUATIONRUN"]._serialized_end = 2910
    _globals["_PROMPTEVALUATIONRUN_AGGREGATEMETRICSENTRY"]._serialized_start = 2681
    _globals["_PROMPTEVALUATIONRUN_AGGREGATEMETRICSENTRY"]._serialized_end = 2772
    _globals["_PROMPTEVALUATIONRUN_METADATAENTRY"]._serialized_start = 2774
    _globals["_PROMPTEVALUATIONRUN_METADATAENTRY"]._serialized_end = 2857
    _globals["_LISTNAMEDPROMPTSREQUEST"]._serialized_start = 2912
    _globals["_LISTNAMEDPROMPTSREQUEST"]._serialized_end = 3014
    _globals["_LISTNAMEDPROMPTSRESPONSE"]._serialized_start = 3017
    _globals["_LISTNAMEDPROMPTSRESPONSE"]._serialized_end = 3164
    _globals["_GETNAMEDPROMPTREQUEST"]._serialized_start = 3166
    _globals["_GETNAMEDPROMPTREQUEST"]._serialized_end = 3229
    _globals["_GETNAMEDPROMPTRESPONSE"]._serialized_start = 3231
    _globals["_GETNAMEDPROMPTRESPONSE"]._serialized_end = 3342
    _globals["_PROMPTVARIANTOPERATION"]._serialized_start = 3345
    _globals["_PROMPTVARIANTOPERATION"]._serialized_end = 3506
    _globals["_CREATENAMEDPROMPTREQUEST"]._serialized_start = 3509
    _globals["_CREATENAMEDPROMPTREQUEST"]._serialized_end = 3689
    _globals["_CREATENAMEDPROMPTRESPONSE"]._serialized_start = 3691
    _globals["_CREATENAMEDPROMPTRESPONSE"]._serialized_end = 3783
    _globals["_UPDATENAMEDPROMPTOPERATION"]._serialized_start = 3786
    _globals["_UPDATENAMEDPROMPTOPERATION"]._serialized_end = 4109
    _globals["_UPDATENAMEDPROMPTREQUEST"]._serialized_start = 4112
    _globals["_UPDATENAMEDPROMPTREQUEST"]._serialized_end = 4308
    _globals["_UPDATENAMEDPROMPTRESPONSE"]._serialized_start = 4310
    _globals["_UPDATENAMEDPROMPTRESPONSE"]._serialized_end = 4402
    _globals["_LISTPROMPTVARIANTSREQUEST"]._serialized_start = 4405
    _globals["_LISTPROMPTVARIANTSREQUEST"]._serialized_end = 4645
    _globals["_LISTPROMPTVARIANTSRESPONSE"]._serialized_start = 4648
    _globals["_LISTPROMPTVARIANTSRESPONSE"]._serialized_end = 4803
    _globals["_LISTPROMPTEVALUATIONRUNSREQUEST"]._serialized_start = 4806
    _globals["_LISTPROMPTEVALUATIONRUNSREQUEST"]._serialized_end = 4981
    _globals["_LISTPROMPTEVALUATIONRUNSRESPONSE"]._serialized_start = 4984
    _globals["_LISTPROMPTEVALUATIONRUNSRESPONSE"]._serialized_end = 5151
    _globals["_GETPROMPTEVALUATIONRUNREQUEST"]._serialized_start = 5154
    _globals["_GETPROMPTEVALUATIONRUNREQUEST"]._serialized_end = 5501
    _globals["_GETPROMPTEVALUATIONRUNREQUEST_PREFILLOPTIONS"]._serialized_start = 5340
    _globals["_GETPROMPTEVALUATIONRUNREQUEST_PREFILLOPTIONS"]._serialized_end = 5492
    _globals["_GETPROMPTEVALUATIONRUNRESPONSE"]._serialized_start = 5504
    _globals["_GETPROMPTEVALUATIONRUNRESPONSE"]._serialized_end = 5637
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST"]._serialized_start = 5640
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST"]._serialized_end = 6259
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST_METADATAENTRY"]._serialized_start = 2774
    _globals["_CREATEPROMPTEVALUATIONRUNREQUEST_METADATAENTRY"]._serialized_end = 2857
    _globals["_CREATEPROMPTEVALUATIONRUNRESPONSE"]._serialized_start = 6261
    _globals["_CREATEPROMPTEVALUATIONRUNRESPONSE"]._serialized_end = 6373
    _globals["_PROMPTSERVICE"]._serialized_start = 6376
    _globals["_PROMPTSERVICE"]._serialized_end = 7364
# @@protoc_insertion_point(module_scope)
