"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.engine.v1.plan_server_pb2 import (
    ExecuteQueryRequest,
    ExecuteQueryResponse,
    GetPlanRequest,
    GetPlanResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class PlanServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    GetPlan: UnaryUnaryMultiCallable[
        GetPlanRequest,
        GetPlanResponse,
    ]
    ExecuteQuery: UnaryUnaryMultiCallable[
        ExecuteQueryRequest,
        ExecuteQueryResponse,
    ]
    """Fall-back for executing queries on the python engine"""

class PlanServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def GetPlan(
        self,
        request: GetPlanRequest,
        context: ServicerContext,
    ) -> GetPlanResponse: ...
    @abstractmethod
    def ExecuteQuery(
        self,
        request: ExecuteQueryRequest,
        context: ServicerContext,
    ) -> ExecuteQueryResponse:
        """Fall-back for executing queries on the python engine"""

def add_PlanServiceServicer_to_server(servicer: PlanServiceServicer, server: Server) -> None: ...
