# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from chalk._gen.chalk.server.v1 import builder_pb2 as chalk_dot_server_dot_v1_dot_builder__pb2


class BuilderServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetSearchConfig = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetSearchConfig",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigResponse.FromString,
        )
        self.ActivateDeployment = channel.unary_unary(
            "/chalk.server.v1.BuilderService/ActivateDeployment",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentResponse.FromString,
        )
        self.IndexDeployment = channel.unary_unary(
            "/chalk.server.v1.BuilderService/IndexDeployment",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentResponse.FromString,
        )
        self.DeployKubeComponents = channel.unary_unary(
            "/chalk.server.v1.BuilderService/DeployKubeComponents",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsResponse.FromString,
        )
        self.RebuildDeployment = channel.unary_unary(
            "/chalk.server.v1.BuilderService/RebuildDeployment",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentResponse.FromString,
        )
        self.RedeployDeployment = channel.unary_unary(
            "/chalk.server.v1.BuilderService/RedeployDeployment",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentResponse.FromString,
        )
        self.UploadSource = channel.unary_unary(
            "/chalk.server.v1.BuilderService/UploadSource",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceResponse.FromString,
        )
        self.LintSource = channel.unary_unary(
            "/chalk.server.v1.BuilderService/LintSource",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceResponse.FromString,
        )
        self.GetDeploymentSteps = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetDeploymentSteps",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsResponse.FromString,
        )
        self.GetDeploymentLogs = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetDeploymentLogs",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsResponse.FromString,
        )
        self.GetClusterTimescaleDB = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetClusterTimescaleDB",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBResponse.FromString,
        )
        self.GetClusterGateway = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetClusterGateway",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayResponse.FromString,
        )
        self.GetClusterBackgroundPersistence = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetClusterBackgroundPersistence",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceResponse.FromString,
        )
        self.CreateClusterTimescaleDB = channel.unary_unary(
            "/chalk.server.v1.BuilderService/CreateClusterTimescaleDB",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBResponse.FromString,
        )
        self.MigrateClusterTimescaleDB = channel.unary_unary(
            "/chalk.server.v1.BuilderService/MigrateClusterTimescaleDB",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBResponse.FromString,
        )
        self.CreateClusterGateway = channel.unary_unary(
            "/chalk.server.v1.BuilderService/CreateClusterGateway",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayResponse.FromString,
        )
        self.CreateClusterBackgroundPersistence = channel.unary_unary(
            "/chalk.server.v1.BuilderService/CreateClusterBackgroundPersistence",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceResponse.FromString,
        )
        self.UpdateEnvironmentVariables = channel.unary_unary(
            "/chalk.server.v1.BuilderService/UpdateEnvironmentVariables",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesResponse.FromString,
        )
        self.StartBranch = channel.unary_unary(
            "/chalk.server.v1.BuilderService/StartBranch",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchResponse.FromString,
        )
        self.GetNodepools = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetNodepools",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsResponse.FromString,
        )
        self.AddNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/AddNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolResponse.FromString,
        )
        self.UpdateNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/UpdateNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolResponse.FromString,
        )
        self.DeleteNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/DeleteNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolResponse.FromString,
        )
        self.GetKarpenterNodepools = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetKarpenterNodepools",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsResponse.FromString,
        )
        self.AddKarpenterNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/AddKarpenterNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolResponse.FromString,
        )
        self.UpdateKarpenterNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/UpdateKarpenterNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolResponse.FromString,
        )
        self.DeleteKarpenterNodepool = channel.unary_unary(
            "/chalk.server.v1.BuilderService/DeleteKarpenterNodepool",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolResponse.FromString,
        )
        self.GetKarpenterInstallationMetadata = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetKarpenterInstallationMetadata",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataResponse.FromString,
        )
        self.GetTagWeights = channel.unary_unary(
            "/chalk.server.v1.BuilderService/GetTagWeights",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsResponse.FromString,
        )
        self.SetTagWeights = channel.unary_unary(
            "/chalk.server.v1.BuilderService/SetTagWeights",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsResponse.FromString,
        )


class BuilderServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetSearchConfig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ActivateDeployment(self, request, context):
        """Takes an existing (past) deployment and promotes the k8s resources / other things associated with it.
        Useful for debugging in local development where the auto activation doesn't work b/c no pubsub.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def IndexDeployment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeployKubeComponents(self, request, context):
        """Intermediate step in the deployment activation process. Allows for partial migration to the new
        go-api-server builder service.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RebuildDeployment(self, request, context):
        """Takes an existing (past) deployment and re-creates the image associated with it,
        publishing the image as 'new_image_tag'.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RedeployDeployment(self, request, context):
        """Triggers a new build with the source code from this deployment and deploys the result"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadSource(self, request, context):
        """Triggers a new build with the provided source code archive and deploys the result"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def LintSource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetDeploymentSteps(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetDeploymentLogs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetClusterTimescaleDB(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetClusterGateway(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetClusterBackgroundPersistence(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateClusterTimescaleDB(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def MigrateClusterTimescaleDB(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateClusterGateway(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateClusterBackgroundPersistence(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateEnvironmentVariables(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def StartBranch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetNodepools(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AddNodepool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateNodepool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteNodepool(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetKarpenterNodepools(self, request, context):
        """to be deprecated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AddKarpenterNodepool(self, request, context):
        """to be deprecated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateKarpenterNodepool(self, request, context):
        """to be deprecated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteKarpenterNodepool(self, request, context):
        """to be deprecated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetKarpenterInstallationMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetTagWeights(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SetTagWeights(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_BuilderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetSearchConfig": grpc.unary_unary_rpc_method_handler(
            servicer.GetSearchConfig,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigResponse.SerializeToString,
        ),
        "ActivateDeployment": grpc.unary_unary_rpc_method_handler(
            servicer.ActivateDeployment,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentResponse.SerializeToString,
        ),
        "IndexDeployment": grpc.unary_unary_rpc_method_handler(
            servicer.IndexDeployment,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentResponse.SerializeToString,
        ),
        "DeployKubeComponents": grpc.unary_unary_rpc_method_handler(
            servicer.DeployKubeComponents,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsResponse.SerializeToString,
        ),
        "RebuildDeployment": grpc.unary_unary_rpc_method_handler(
            servicer.RebuildDeployment,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentResponse.SerializeToString,
        ),
        "RedeployDeployment": grpc.unary_unary_rpc_method_handler(
            servicer.RedeployDeployment,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentResponse.SerializeToString,
        ),
        "UploadSource": grpc.unary_unary_rpc_method_handler(
            servicer.UploadSource,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceResponse.SerializeToString,
        ),
        "LintSource": grpc.unary_unary_rpc_method_handler(
            servicer.LintSource,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceResponse.SerializeToString,
        ),
        "GetDeploymentSteps": grpc.unary_unary_rpc_method_handler(
            servicer.GetDeploymentSteps,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsResponse.SerializeToString,
        ),
        "GetDeploymentLogs": grpc.unary_unary_rpc_method_handler(
            servicer.GetDeploymentLogs,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsResponse.SerializeToString,
        ),
        "GetClusterTimescaleDB": grpc.unary_unary_rpc_method_handler(
            servicer.GetClusterTimescaleDB,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBResponse.SerializeToString,
        ),
        "GetClusterGateway": grpc.unary_unary_rpc_method_handler(
            servicer.GetClusterGateway,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayResponse.SerializeToString,
        ),
        "GetClusterBackgroundPersistence": grpc.unary_unary_rpc_method_handler(
            servicer.GetClusterBackgroundPersistence,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceResponse.SerializeToString,
        ),
        "CreateClusterTimescaleDB": grpc.unary_unary_rpc_method_handler(
            servicer.CreateClusterTimescaleDB,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBResponse.SerializeToString,
        ),
        "MigrateClusterTimescaleDB": grpc.unary_unary_rpc_method_handler(
            servicer.MigrateClusterTimescaleDB,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBResponse.SerializeToString,
        ),
        "CreateClusterGateway": grpc.unary_unary_rpc_method_handler(
            servicer.CreateClusterGateway,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayResponse.SerializeToString,
        ),
        "CreateClusterBackgroundPersistence": grpc.unary_unary_rpc_method_handler(
            servicer.CreateClusterBackgroundPersistence,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceResponse.SerializeToString,
        ),
        "UpdateEnvironmentVariables": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateEnvironmentVariables,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesResponse.SerializeToString,
        ),
        "StartBranch": grpc.unary_unary_rpc_method_handler(
            servicer.StartBranch,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchResponse.SerializeToString,
        ),
        "GetNodepools": grpc.unary_unary_rpc_method_handler(
            servicer.GetNodepools,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsResponse.SerializeToString,
        ),
        "AddNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.AddNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolResponse.SerializeToString,
        ),
        "UpdateNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolResponse.SerializeToString,
        ),
        "DeleteNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolResponse.SerializeToString,
        ),
        "GetKarpenterNodepools": grpc.unary_unary_rpc_method_handler(
            servicer.GetKarpenterNodepools,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsResponse.SerializeToString,
        ),
        "AddKarpenterNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.AddKarpenterNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolResponse.SerializeToString,
        ),
        "UpdateKarpenterNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateKarpenterNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolResponse.SerializeToString,
        ),
        "DeleteKarpenterNodepool": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteKarpenterNodepool,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolResponse.SerializeToString,
        ),
        "GetKarpenterInstallationMetadata": grpc.unary_unary_rpc_method_handler(
            servicer.GetKarpenterInstallationMetadata,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataResponse.SerializeToString,
        ),
        "GetTagWeights": grpc.unary_unary_rpc_method_handler(
            servicer.GetTagWeights,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsResponse.SerializeToString,
        ),
        "SetTagWeights": grpc.unary_unary_rpc_method_handler(
            servicer.SetTagWeights,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("chalk.server.v1.BuilderService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class BuilderService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetSearchConfig(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetSearchConfig",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetSearchConfigResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ActivateDeployment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/ActivateDeployment",
            chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.ActivateDeploymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def IndexDeployment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/IndexDeployment",
            chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.IndexDeploymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeployKubeComponents(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/DeployKubeComponents",
            chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.DeployKubeComponentsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RebuildDeployment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/RebuildDeployment",
            chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.RebuildDeploymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RedeployDeployment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/RedeployDeployment",
            chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.RedeployDeploymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadSource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/UploadSource",
            chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.UploadSourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def LintSource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/LintSource",
            chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.LintSourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetDeploymentSteps(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetDeploymentSteps",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentStepsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetDeploymentLogs(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetDeploymentLogs",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetDeploymentLogsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetClusterTimescaleDB(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetClusterTimescaleDB",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterTimescaleDBResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetClusterGateway(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetClusterGateway",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterGatewayResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetClusterBackgroundPersistence(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetClusterBackgroundPersistence",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetClusterBackgroundPersistenceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateClusterTimescaleDB(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/CreateClusterTimescaleDB",
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterTimescaleDBResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def MigrateClusterTimescaleDB(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/MigrateClusterTimescaleDB",
            chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.MigrateClusterTimescaleDBResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateClusterGateway(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/CreateClusterGateway",
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterGatewayResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateClusterBackgroundPersistence(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/CreateClusterBackgroundPersistence",
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateClusterBackgroundPersistenceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateEnvironmentVariables(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/UpdateEnvironmentVariables",
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateEnvironmentVariablesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def StartBranch(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/StartBranch",
            chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.StartBranchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetNodepools(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetNodepools",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetNodepoolsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AddNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/AddNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.AddNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/UpdateNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/DeleteNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.DeleteNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetKarpenterNodepools(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetKarpenterNodepools",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterNodepoolsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AddKarpenterNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/AddKarpenterNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.AddKarpenterNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateKarpenterNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/UpdateKarpenterNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.UpdateKarpenterNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteKarpenterNodepool(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/DeleteKarpenterNodepool",
            chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.DeleteKarpenterNodepoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetKarpenterInstallationMetadata(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetKarpenterInstallationMetadata",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKarpenterInstallationMetadataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetTagWeights(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/GetTagWeights",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetTagWeightsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SetTagWeights(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.BuilderService/SetTagWeights",
            chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.SetTagWeightsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class ClusterBuilderServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateKafkaTopics = channel.unary_unary(
            "/chalk.server.v1.ClusterBuilderService/CreateKafkaTopics",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsResponse.FromString,
        )
        self.GetKafkaTopics = channel.unary_unary(
            "/chalk.server.v1.ClusterBuilderService/GetKafkaTopics",
            request_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsResponse.FromString,
        )


class ClusterBuilderServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateKafkaTopics(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetKafkaTopics(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ClusterBuilderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CreateKafkaTopics": grpc.unary_unary_rpc_method_handler(
            servicer.CreateKafkaTopics,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsResponse.SerializeToString,
        ),
        "GetKafkaTopics": grpc.unary_unary_rpc_method_handler(
            servicer.GetKafkaTopics,
            request_deserializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("chalk.server.v1.ClusterBuilderService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ClusterBuilderService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateKafkaTopics(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.ClusterBuilderService/CreateKafkaTopics",
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.CreateKafkaTopicsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetKafkaTopics(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.ClusterBuilderService/GetKafkaTopics",
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_builder__pb2.GetKafkaTopicsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
