# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/queries.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.chart.v1 import densetimeserieschart_pb2 as chalk_dot_chart_dot_v1_dot_densetimeserieschart__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1d\x63halk/server/v1/queries.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a)chalk/chart/v1/densetimeserieschart.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"F\n!GetQueryPerformanceSummaryRequest\x12!\n\x0coperation_id\x18\x01 \x01(\tR\x0boperationId"\x95\x01\n"GetQueryPerformanceSummaryResponse\x12!\n\x0coperation_id\x18\x01 \x01(\tR\x0boperationId\x12\x34\n\x13performance_summary\x18\x02 \x01(\tH\x00R\x12performanceSummary\x88\x01\x01\x42\x16\n\x14_performance_summary"\x8c\x01\n\x18ListQueryErrorsPageToken\x12$\n\x0enumeric_id_hwm\x18\x01 \x01(\x03R\x0cnumericIdHwm\x12J\n\x13\x65rror_timestamp_hwm\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x11\x65rrorTimestampHwm"\xee\x01\n\x11QueryErrorFilters\x12&\n\x0coperation_id\x18\x01 \x01(\tH\x00R\x0boperationId\x88\x01\x01\x12$\n\x0b\x66\x65\x61ture_fqn\x18\x02 \x01(\tH\x01R\nfeatureFqn\x88\x01\x01\x12&\n\x0cresolver_fqn\x18\x03 \x01(\tH\x02R\x0bresolverFqn\x88\x01\x01\x12"\n\nquery_name\x18\x04 \x01(\tH\x03R\tqueryName\x88\x01\x01\x42\x0f\n\r_operation_idB\x0e\n\x0c_feature_fqnB\x0f\n\r_resolver_fqnB\r\n\x0b_query_name"\x8e\x07\n\x0eQueryErrorMeta\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12\x12\n\x04\x63ode\x18\x02 \x01(\tR\x04\x63ode\x12\x1a\n\x08\x63\x61tegory\x18\x03 \x01(\tR\x08\x63\x61tegory\x12\x18\n\x07message\x18\x04 \x01(\tR\x07message\x12\x33\n\x13\x64isplay_primary_key\x18\x05 \x01(\tH\x00R\x11\x64isplayPrimaryKey\x88\x01\x01\x12:\n\x17\x64isplay_primary_key_fqn\x18\x06 \x01(\tH\x01R\x14\x64isplayPrimaryKeyFqn\x88\x01\x01\x12\x1d\n\x07\x66\x65\x61ture\x18\x07 \x01(\tH\x02R\x07\x66\x65\x61ture\x88\x01\x01\x12\x1f\n\x08resolver\x18\x08 \x01(\tH\x03R\x08resolver\x88\x01\x01\x12"\n\nquery_name\x18\t \x01(\tH\x04R\tqueryName\x88\x01\x01\x12*\n\x0e\x65xception_kind\x18\n \x01(\tH\x05R\rexceptionKind\x88\x01\x01\x12\x30\n\x11\x65xception_message\x18\x0b \x01(\tH\x06R\x10\x65xceptionMessage\x88\x01\x01\x12\x36\n\x14\x65xception_stacktrace\x18\x0c \x01(\tH\x07R\x13\x65xceptionStacktrace\x88\x01\x01\x12G\n\x1d\x65xception_internal_stacktrace\x18\r \x01(\tH\x08R\x1b\x65xceptionInternalStacktrace\x88\x01\x01\x12&\n\x0coperation_id\x18\x0e \x01(\tH\tR\x0boperationId\x88\x01\x01\x12(\n\rdeployment_id\x18\x0f \x01(\tH\nR\x0c\x64\x65ploymentId\x88\x01\x01\x12\x39\n\ncreated_at\x18\x10 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAtB\x16\n\x14_display_primary_keyB\x1a\n\x18_display_primary_key_fqnB\n\n\x08_featureB\x0b\n\t_resolverB\r\n\x0b_query_nameB\x11\n\x0f_exception_kindB\x14\n\x12_exception_messageB\x17\n\x15_exception_stacktraceB \n\x1e_exception_internal_stacktraceB\x0f\n\r_operation_idB\x10\n\x0e_deployment_id"\xab\x02\n\x16ListQueryErrorsRequest\x12\x39\n\nstart_date\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartDate\x12\x35\n\x08\x65nd_date\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndDate\x12<\n\x07\x66ilters\x18\x03 \x01(\x0b\x32".chalk.server.v1.QueryErrorFiltersR\x07\x66ilters\x12 \n\tpage_size\x18\x04 \x01(\x05H\x00R\x08pageSize\x88\x01\x01\x12"\n\npage_token\x18\x05 \x01(\tH\x01R\tpageToken\x88\x01\x01\x42\x0c\n\n_page_sizeB\r\n\x0b_page_token"\x9e\x01\n\x17ListQueryErrorsResponse\x12\x42\n\x0cquery_errors\x18\x01 \x03(\x0b\x32\x1f.chalk.server.v1.QueryErrorMetaR\x0bqueryErrors\x12+\n\x0fnext_page_token\x18\x02 \x01(\tH\x00R\rnextPageToken\x88\x01\x01\x42\x12\n\x10_next_page_token"\xe7\x02\n\x1aGetQueryErrorsChartRequest\x12V\n\x19start_timestamp_inclusive\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x17startTimestampInclusive\x12W\n\x17\x65nd_timestamp_exclusive\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x15\x65ndTimestampExclusive\x88\x01\x01\x12>\n\rwindow_period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod\x12<\n\x07\x66ilters\x18\x04 \x01(\x0b\x32".chalk.server.v1.QueryErrorFiltersR\x07\x66iltersB\x1a\n\x18_end_timestamp_exclusive"Y\n\x1bGetQueryErrorsChartResponse\x12:\n\x05\x63hart\x18\x01 \x01(\x0b\x32$.chalk.chart.v1.DenseTimeSeriesChartR\x05\x63hart"9\n\x13GetQueryPlanRequest\x12"\n\rquery_plan_id\x18\x01 \x01(\tR\x0bqueryPlanId"\xc1\x01\n\tQueryPlan\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x03 \x01(\tR\x0c\x64\x65ploymentId\x12\x1d\n\nquery_plan\x18\x04 \x01(\tR\tqueryPlan\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt"Q\n\x14GetQueryPlanResponse\x12\x39\n\nquery_plan\x18\x01 \x01(\x0b\x32\x1a.chalk.server.v1.QueryPlanR\tqueryPlan2\xe1\x03\n\x0eQueriesService\x12\x8a\x01\n\x1aGetQueryPerformanceSummary\x12\x32.chalk.server.v1.GetQueryPerformanceSummaryRequest\x1a\x33.chalk.server.v1.GetQueryPerformanceSummaryResponse"\x03\x80}\x06\x12i\n\x0fListQueryErrors\x12\'.chalk.server.v1.ListQueryErrorsRequest\x1a(.chalk.server.v1.ListQueryErrorsResponse"\x03\x80}\x06\x12u\n\x13GetQueryErrorsChart\x12+.chalk.server.v1.GetQueryErrorsChartRequest\x1a,.chalk.server.v1.GetQueryErrorsChartResponse"\x03\x80}\x06\x12`\n\x0cGetQueryPlan\x12$.chalk.server.v1.GetQueryPlanRequest\x1a%.chalk.server.v1.GetQueryPlanResponse"\x03\x80}\x06\x42\x95\x01\n\x13\x63om.chalk.server.v1B\x0cQueriesProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.queries_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\014QueriesProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryPerformanceSummary"]._options = None
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryPerformanceSummary"]._serialized_options = b"\200}\006"
    _globals["_QUERIESSERVICE"].methods_by_name["ListQueryErrors"]._options = None
    _globals["_QUERIESSERVICE"].methods_by_name["ListQueryErrors"]._serialized_options = b"\200}\006"
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryErrorsChart"]._options = None
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryErrorsChart"]._serialized_options = b"\200}\006"
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryPlan"]._options = None
    _globals["_QUERIESSERVICE"].methods_by_name["GetQueryPlan"]._serialized_options = b"\200}\006"
    _globals["_GETQUERYPERFORMANCESUMMARYREQUEST"]._serialized_start = 191
    _globals["_GETQUERYPERFORMANCESUMMARYREQUEST"]._serialized_end = 261
    _globals["_GETQUERYPERFORMANCESUMMARYRESPONSE"]._serialized_start = 264
    _globals["_GETQUERYPERFORMANCESUMMARYRESPONSE"]._serialized_end = 413
    _globals["_LISTQUERYERRORSPAGETOKEN"]._serialized_start = 416
    _globals["_LISTQUERYERRORSPAGETOKEN"]._serialized_end = 556
    _globals["_QUERYERRORFILTERS"]._serialized_start = 559
    _globals["_QUERYERRORFILTERS"]._serialized_end = 797
    _globals["_QUERYERRORMETA"]._serialized_start = 800
    _globals["_QUERYERRORMETA"]._serialized_end = 1710
    _globals["_LISTQUERYERRORSREQUEST"]._serialized_start = 1713
    _globals["_LISTQUERYERRORSREQUEST"]._serialized_end = 2012
    _globals["_LISTQUERYERRORSRESPONSE"]._serialized_start = 2015
    _globals["_LISTQUERYERRORSRESPONSE"]._serialized_end = 2173
    _globals["_GETQUERYERRORSCHARTREQUEST"]._serialized_start = 2176
    _globals["_GETQUERYERRORSCHARTREQUEST"]._serialized_end = 2535
    _globals["_GETQUERYERRORSCHARTRESPONSE"]._serialized_start = 2537
    _globals["_GETQUERYERRORSCHARTRESPONSE"]._serialized_end = 2626
    _globals["_GETQUERYPLANREQUEST"]._serialized_start = 2628
    _globals["_GETQUERYPLANREQUEST"]._serialized_end = 2685
    _globals["_QUERYPLAN"]._serialized_start = 2688
    _globals["_QUERYPLAN"]._serialized_end = 2881
    _globals["_GETQUERYPLANRESPONSE"]._serialized_start = 2883
    _globals["_GETQUERYPLANRESPONSE"]._serialized_end = 2964
    _globals["_QUERIESSERVICE"]._serialized_start = 2967
    _globals["_QUERIESSERVICE"]._serialized_end = 3448
# @@protoc_insertion_point(module_scope)
