# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/kubernetes/v1/nodes.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1f\x63halk/kubernetes/v1/nodes.proto\x12\x13\x63halk.kubernetes.v1"\x97\x01\n\x13KubernetesNodeTaint\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x19\n\x05value\x18\x02 \x01(\tH\x00R\x05value\x88\x01\x01\x12\x16\n\x06\x65\x66\x66\x65\x63t\x18\x03 \x01(\tR\x06\x65\x66\x66\x65\x63t\x12"\n\ntime_added\x18\x04 \x01(\x03H\x01R\ttimeAdded\x88\x01\x01\x42\x08\n\x06_valueB\r\n\x0b_time_added"\x93\x02\n\x12KubernetesNodeSpec\x12\x1e\n\x08pod_cidr\x18\x01 \x01(\tH\x00R\x07podCidr\x88\x01\x01\x12@\n\x06taints\x18\x02 \x03(\x0b\x32(.chalk.kubernetes.v1.KubernetesNodeTaintR\x06taints\x12\x1b\n\tpod_cidrs\x18\x03 \x03(\tR\x08podCidrs\x12$\n\x0bprovider_id\x18\x04 \x01(\tH\x01R\nproviderId\x88\x01\x01\x12)\n\runschedulable\x18\x05 \x01(\x08H\x02R\runschedulable\x88\x01\x01\x42\x0b\n\t_pod_cidrB\x0e\n\x0c_provider_idB\x10\n\x0e_unschedulable"\xfa\x01\n\x17KubernetesNodeCondition\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\x12\x16\n\x06status\x18\x02 \x01(\tR\x06status\x12.\n\x13last_heartbeat_time\x18\x03 \x01(\x03R\x11lastHeartbeatTime\x12\x30\n\x14last_transition_time\x18\x04 \x01(\x03R\x12lastTransitionTime\x12\x1b\n\x06reason\x18\x05 \x01(\tH\x00R\x06reason\x88\x01\x01\x12\x1d\n\x07message\x18\x06 \x01(\tH\x01R\x07message\x88\x01\x01\x42\t\n\x07_reasonB\n\n\x08_message"O\n\x18KubernetesAttachedVolume\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1f\n\x0b\x64\x65vice_path\x18\x02 \x01(\tR\ndevicePath"\xe4\x01\n\x14KubernetesNodeStatus\x12L\n\nconditions\x18\x01 \x03(\x0b\x32,.chalk.kubernetes.v1.KubernetesNodeConditionR\nconditions\x12$\n\x0evolumes_in_use\x18\x02 \x03(\tR\x0cvolumesInUse\x12X\n\x10volumes_attached\x18\x03 \x03(\x0b\x32-.chalk.kubernetes.v1.KubernetesAttachedVolumeR\x0fvolumesAttached"\xde\x08\n\x12KubernetesNodeData\x12\x12\n\x04team\x18\x01 \x01(\tR\x04team\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x10\n\x03uid\x18\x03 \x01(\tR\x03uid\x12#\n\rinstance_type\x18\x04 \x01(\tR\x0cinstanceType\x12\x16\n\x06region\x18\x05 \x01(\tR\x06region\x12\x12\n\x04zone\x18\x06 \x01(\tR\x04zone\x12-\n\x12\x63reation_timestamp\x18\x08 \x01(\x03R\x11\x63reationTimestamp\x12-\n\x12\x64\x65letion_timestamp\x18\t \x01(\x03R\x11\x64\x65letionTimestamp\x12-\n\x12observed_timestamp\x18\n \x01(\x03R\x11observedTimestamp\x12K\n\x06labels\x18\x0b \x03(\x0b\x32\x33.chalk.kubernetes.v1.KubernetesNodeData.LabelsEntryR\x06labels\x12Z\n\x0b\x61nnotations\x18\x0c \x03(\x0b\x32\x38.chalk.kubernetes.v1.KubernetesNodeData.AnnotationsEntryR\x0b\x61nnotations\x12\x1d\n\nmachine_id\x18\r \x01(\tR\tmachineId\x12\x1f\n\x0bsystem_uuid\x18\x0e \x01(\tR\nsystemUuid\x12\x17\n\x07\x62oot_id\x18\x0f \x01(\tR\x06\x62ootId\x12$\n\runschedulable\x18\x10 \x01(\x08R\runschedulable\x12\x1c\n\tnamespace\x18\x11 \x01(\tR\tnamespace\x12\x1f\n\x0binstance_id\x18\x12 \x01(\tR\ninstanceId\x12\x18\n\x07\x63luster\x18\x13 \x01(\tR\x07\x63luster\x12\x1b\n\ttotal_cpu\x18\x14 \x01(\tR\x08totalCpu\x12!\n\x0ctotal_memory\x18\x15 \x01(\tR\x0btotalMemory\x12\'\n\x0f\x61llocatable_cpu\x18\x16 \x01(\tR\x0e\x61llocatableCpu\x12-\n\x12\x61llocatable_memory\x18\x17 \x01(\tR\x11\x61llocatableMemory\x12@\n\x04spec\x18\x18 \x01(\x0b\x32\'.chalk.kubernetes.v1.KubernetesNodeSpecH\x00R\x04spec\x88\x01\x01\x12\x46\n\x06status\x18\x19 \x01(\x0b\x32).chalk.kubernetes.v1.KubernetesNodeStatusH\x01R\x06status\x88\x01\x01\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a>\n\x10\x41nnotationsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x07\n\x05_specB\t\n\x07_statusB\xaf\x01\n\x17\x63om.chalk.kubernetes.v1B\nNodesProtoP\x01Z\x1akubernetes/v1;kubernetesv1\xa2\x02\x03\x43KX\xaa\x02\x13\x43halk.Kubernetes.V1\xca\x02\x13\x43halk\\Kubernetes\\V1\xe2\x02\x1f\x43halk\\Kubernetes\\V1\\GPBMetadata\xea\x02\x15\x43halk::Kubernetes::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.kubernetes.v1.nodes_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\027com.chalk.kubernetes.v1B\nNodesProtoP\001Z\032kubernetes/v1;kubernetesv1\242\002\003CKX\252\002\023Chalk.Kubernetes.V1\312\002\023Chalk\\Kubernetes\\V1\342\002\037Chalk\\Kubernetes\\V1\\GPBMetadata\352\002\025Chalk::Kubernetes::V1"
    _globals["_KUBERNETESNODEDATA_LABELSENTRY"]._options = None
    _globals["_KUBERNETESNODEDATA_LABELSENTRY"]._serialized_options = b"8\001"
    _globals["_KUBERNETESNODEDATA_ANNOTATIONSENTRY"]._options = None
    _globals["_KUBERNETESNODEDATA_ANNOTATIONSENTRY"]._serialized_options = b"8\001"
    _globals["_KUBERNETESNODETAINT"]._serialized_start = 57
    _globals["_KUBERNETESNODETAINT"]._serialized_end = 208
    _globals["_KUBERNETESNODESPEC"]._serialized_start = 211
    _globals["_KUBERNETESNODESPEC"]._serialized_end = 486
    _globals["_KUBERNETESNODECONDITION"]._serialized_start = 489
    _globals["_KUBERNETESNODECONDITION"]._serialized_end = 739
    _globals["_KUBERNETESATTACHEDVOLUME"]._serialized_start = 741
    _globals["_KUBERNETESATTACHEDVOLUME"]._serialized_end = 820
    _globals["_KUBERNETESNODESTATUS"]._serialized_start = 823
    _globals["_KUBERNETESNODESTATUS"]._serialized_end = 1051
    _globals["_KUBERNETESNODEDATA"]._serialized_start = 1054
    _globals["_KUBERNETESNODEDATA"]._serialized_end = 2172
    _globals["_KUBERNETESNODEDATA_LABELSENTRY"]._serialized_start = 2031
    _globals["_KUBERNETESNODEDATA_LABELSENTRY"]._serialized_end = 2088
    _globals["_KUBERNETESNODEDATA_ANNOTATIONSENTRY"]._serialized_start = 2090
    _globals["_KUBERNETESNODEDATA_ANNOTATIONSENTRY"]._serialized_end = 2152
# @@protoc_insertion_point(module_scope)
