# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/artifacts/v1/cdc.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/artifacts/v1/cdc.proto\x12\x12\x63halk.artifacts.v1"?\n\x11\x43\x44\x43TableReference\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n\x06schema\x18\x02 \x01(\tR\x06schema"u\n\tCDCSource\x12)\n\x10integration_name\x18\x01 \x01(\tR\x0fintegrationName\x12=\n\x06tables\x18\x02 \x03(\x0b\x32%.chalk.artifacts.v1.CDCTableReferenceR\x06tablesB\x8c\x01\n\x16\x63om.chalk.artifacts.v1B\x08\x43\x64\x63ProtoP\x01\xa2\x02\x03\x43\x41X\xaa\x02\x12\x43halk.Artifacts.V1\xca\x02\x12\x43halk\\Artifacts\\V1\xe2\x02\x1e\x43halk\\Artifacts\\V1\\GPBMetadata\xea\x02\x14\x43halk::Artifacts::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.artifacts.v1.cdc_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\026com.chalk.artifacts.v1B\010CdcProtoP\001\242\002\003CAX\252\002\022Chalk.Artifacts.V1\312\002\022Chalk\\Artifacts\\V1\342\002\036Chalk\\Artifacts\\V1\\GPBMetadata\352\002\024Chalk::Artifacts::V1"
    _globals["_CDCTABLEREFERENCE"]._serialized_start = 52
    _globals["_CDCTABLEREFERENCE"]._serialized_end = 115
    _globals["_CDCSOURCE"]._serialized_start = 117
    _globals["_CDCSOURCE"]._serialized_end = 234
# @@protoc_insertion_point(module_scope)
