# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/common/v1/upload_features.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.common.v1 import chalk_error_pb2 as chalk_dot_common_dot_v1_dot_chalk__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n%chalk/common/v1/upload_features.proto\x12\x0f\x63halk.common.v1\x1a!chalk/common/v1/chalk_error.proto":\n\x15UploadFeaturesRequest\x12!\n\x0cinputs_table\x18\x01 \x01(\x0cR\x0binputsTable"p\n\x16UploadFeaturesResponse\x12\x33\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorR\x06\x65rrors\x12!\n\x0coperation_id\x18\x02 \x01(\tR\x0boperationIdB\x88\x01\n\x13\x63om.chalk.common.v1B\x13UploadFeaturesProtoP\x01\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43halk.Common.V1\xca\x02\x0f\x43halk\\Common\\V1\xe2\x02\x1b\x43halk\\Common\\V1\\GPBMetadata\xea\x02\x11\x43halk::Common::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.common.v1.upload_features_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.common.v1B\023UploadFeaturesProtoP\001\242\002\003CCX\252\002\017Chalk.Common.V1\312\002\017Chalk\\Common\\V1\342\002\033Chalk\\Common\\V1\\GPBMetadata\352\002\021Chalk::Common::V1"
    _globals["_UPLOADFEATURESREQUEST"]._serialized_start = 93
    _globals["_UPLOADFEATURESREQUEST"]._serialized_end = 151
    _globals["_UPLOADFEATURESRESPONSE"]._serialized_start = 153
    _globals["_UPLOADFEATURESRESPONSE"]._serialized_end = 265
# @@protoc_insertion_point(module_scope)
