# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/log.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x19\x63halk/server/v1/log.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x9c\x01\n\x08LogEntry\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1f\n\x08severity\x18\x02 \x01(\tH\x00R\x08severity\x88\x01\x01\x12\x38\n\ttimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12\x18\n\x07message\x18\x04 \x01(\tR\x07messageB\x0b\n\t_severity"C\n\x19SearchLogEntriesPageToken\x12&\n\x0fnext_page_token\x18\x01 \x01(\tR\rnextPageToken"\x9d\x01\n\x17SearchLogEntriesRequest\x12\x19\n\x05query\x18\x01 \x01(\tH\x00R\x05query\x88\x01\x01\x12N\n\npage_token\x18\x02 \x01(\x0b\x32*.chalk.server.v1.SearchLogEntriesPageTokenH\x01R\tpageToken\x88\x01\x01\x42\x08\n\x06_queryB\r\n\x0b_page_token"\xc3\x01\n\x18SearchLogEntriesResponse\x12:\n\x0blog_entries\x18\x01 \x03(\x0b\x32\x19.chalk.server.v1.LogEntryR\nlogEntries\x12W\n\x0fnext_page_token\x18\x02 \x01(\x0b\x32*.chalk.server.v1.SearchLogEntriesPageTokenH\x00R\rnextPageToken\x88\x01\x01\x42\x12\n\x10_next_page_token2\x83\x01\n\x10LogSearchService\x12o\n\x10SearchLogEntries\x12(.chalk.server.v1.SearchLogEntriesRequest\x1a).chalk.server.v1.SearchLogEntriesResponse"\x06\x90\x02\x01\x80}\x06\x42\x91\x01\n\x13\x63om.chalk.server.v1B\x08LogProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.log_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\010LogProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_LOGSEARCHSERVICE"].methods_by_name["SearchLogEntries"]._options = None
    _globals["_LOGSEARCHSERVICE"].methods_by_name["SearchLogEntries"]._serialized_options = b"\220\002\001\200}\006"
    _globals["_LOGENTRY"]._serialized_start = 113
    _globals["_LOGENTRY"]._serialized_end = 269
    _globals["_SEARCHLOGENTRIESPAGETOKEN"]._serialized_start = 271
    _globals["_SEARCHLOGENTRIESPAGETOKEN"]._serialized_end = 338
    _globals["_SEARCHLOGENTRIESREQUEST"]._serialized_start = 341
    _globals["_SEARCHLOGENTRIESREQUEST"]._serialized_end = 498
    _globals["_SEARCHLOGENTRIESRESPONSE"]._serialized_start = 501
    _globals["_SEARCHLOGENTRIESRESPONSE"]._serialized_end = 696
    _globals["_LOGSEARCHSERVICE"]._serialized_start = 699
    _globals["_LOGSEARCHSERVICE"]._serialized_end = 830
# @@protoc_insertion_point(module_scope)
