# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/datasets.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1e\x63halk/server/v1/datasets.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xb6\n\n\x13\x44\x61tasetRevisionMeta\x12\x1d\n\nnumeric_id\x18\x01 \x01(\x03R\tnumericId\x12(\n\x10offline_query_id\x18\x02 \x01(\tR\x0eofflineQueryId\x12\x1d\n\ndataset_id\x18\x03 \x01(\tR\tdatasetId\x12"\n\ngivens_uri\x18\x04 \x01(\tH\x00R\tgivensUri\x88\x01\x01\x12V\n\x0egivens_version\x18\x05 \x01(\x0e\x32*.chalk.server.v1.OfflineQueryGivensVersionH\x01R\rgivensVersion\x88\x01\x01\x12"\n\noutput_uri\x18\x06 \x01(\tH\x02R\toutputUri\x88\x01\x01\x12K\n\x0eoutput_version\x18\x07 \x01(\x0e\x32\x1f.chalk.server.v1.DatasetVersionH\x03R\routputVersion\x88\x01\x01\x12$\n\x0b\x62ranch_name\x18\x08 \x01(\tH\x04R\nbranchName\x88\x01\x01\x12\x1e\n\x08num_rows\x18\t \x01(\x03H\x05R\x07numRows\x88\x01\x01\x12\x33\n\x13physical_size_bytes\x18\n \x01(\x03H\x06R\x11physicalSizeBytes\x88\x01\x01\x12%\n\x0eoutput_columns\x18\x0b \x03(\tR\routputColumns\x12\x1f\n\x0boutput_fqns\x18\x0c \x03(\tR\noutputFqns\x12\x1e\n\x08\x61gent_id\x18\r \x01(\tH\x07R\x07\x61gentId\x88\x01\x01\x12\x42\n\x0c\x63ompleted_at\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x08R\x0b\x63ompletedAt\x88\x01\x01\x12"\n\nnum_shards\x18\x0f \x01(\x03H\tR\tnumShards\x88\x01\x01\x12(\n\rnum_computers\x18\x10 \x01(\x03H\nR\x0cnumComputers\x88\x01\x01\x12\x37\n\x08metadata\x18\x11 \x01(\x0b\x32\x16.google.protobuf.ValueH\x0bR\x08metadata\x88\x01\x01\x12\x43\n\x06status\x18\x12 \x01(\x0e\x32&.chalk.server.v1.DatasetRevisionStatusH\x0cR\x06status\x88\x01\x01\x12\x33\n\x13num_rows_calculated\x18\x13 \x01(\x03H\rR\x11numRowsCalculated\x88\x01\x01\x12H\n\x1ephysical_size_bytes_calculated\x18\x14 \x01(\x03H\x0eR\x1bphysicalSizeBytesCalculated\x88\x01\x01\x12>\n\ncreated_at\x18\x15 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x0fR\tcreatedAt\x88\x01\x01\x42\r\n\x0b_givens_uriB\x11\n\x0f_givens_versionB\r\n\x0b_output_uriB\x11\n\x0f_output_versionB\x0e\n\x0c_branch_nameB\x0b\n\t_num_rowsB\x16\n\x14_physical_size_bytesB\x0b\n\t_agent_idB\x0f\n\r_completed_atB\r\n\x0b_num_shardsB\x10\n\x0e_num_computersB\x0b\n\t_metadataB\t\n\x07_statusB\x16\n\x14_num_rows_calculatedB!\n\x1f_physical_size_bytes_calculatedB\r\n\x0b_created_at"\xae\x02\n\x0b\x44\x61tasetMeta\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12&\n\x0c\x64\x61taset_name\x18\x03 \x01(\tH\x00R\x0b\x64\x61tasetName\x88\x01\x01\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12[\n\x14most_recent_revision\x18\x07 \x01(\x0b\x32$.chalk.server.v1.DatasetRevisionMetaH\x01R\x12mostRecentRevision\x88\x01\x01\x42\x0f\n\r_dataset_nameB\x17\n\x15_most_recent_revision"\x8a\x01\n\x13ListDatasetsRequest\x12\x1b\n\x06\x63ursor\x18\x01 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x12\x1b\n\x06search\x18\x03 \x01(\tH\x02R\x06search\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limitB\t\n\x07_search"x\n\x14ListDatasetsResponse\x12\x38\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x1c.chalk.server.v1.DatasetMetaR\x08\x64\x61tasets\x12\x1b\n\x06\x63ursor\x18\x02 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x42\t\n\x07_cursor"#\n\x11GetDatasetRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"L\n\x12GetDatasetResponse\x12\x36\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x1c.chalk.server.v1.DatasetMetaR\x07\x64\x61taset"\x89\x01\n\x1bListDatasetRevisionsRequest\x12\x1d\n\ndataset_id\x18\x01 \x01(\tR\tdatasetId\x12\x1b\n\x06\x63ursor\x18\x02 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x03 \x01(\x05H\x01R\x05limit\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limit"\x8a\x01\n\x1cListDatasetRevisionsResponse\x12\x42\n\trevisions\x18\x01 \x03(\x0b\x32$.chalk.server.v1.DatasetRevisionMetaR\trevisions\x12\x1b\n\x06\x63ursor\x18\x02 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x42\t\n\x07_cursor"+\n\x19GetDatasetRevisionRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"^\n\x1aGetDatasetRevisionResponse\x12@\n\x08revision\x18\x01 \x01(\x0b\x32$.chalk.server.v1.DatasetRevisionMetaR\x08revision*\x9f\x02\n\x15\x44\x61tasetRevisionStatus\x12\'\n#DATASET_REVISION_STATUS_UNSPECIFIED\x10\x00\x12#\n\x1f\x44\x41TASET_REVISION_STATUS_UNKNOWN\x10\x01\x12#\n\x1f\x44\x41TASET_REVISION_STATUS_WORKING\x10\x02\x12%\n!DATASET_REVISION_STATUS_COMPLETED\x10\x03\x12"\n\x1e\x44\x41TASET_REVISION_STATUS_FAILED\x10\x04\x12$\n DATASET_REVISION_STATUS_CANCELED\x10\x05\x12"\n\x1e\x44\x41TASET_REVISION_STATUS_QUEUED\x10\x06*\xe9\x02\n\x0e\x44\x61tasetVersion\x12\x1f\n\x1b\x44\x41TASET_VERSION_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x44\x41TASET_VERSION_UNKNOWN\x10\x01\x12:\n6DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES\x10\x02\x12"\n\x1e\x44\x41TASET_VERSION_DATASET_WRITER\x10\x03\x12=\n9DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2\x10\x04\x12.\n*DATASET_VERSION_COMPUTE_RESOLVER_OUTPUT_V1\x10\x05\x12!\n\x1d\x44\x41TASET_VERSION_NATIVE_DTYPES\x10\x06\x12\'\n#DATASET_VERSION_NATIVE_COLUMN_NAMES\x10\x07*\xad\x02\n\x19OfflineQueryGivensVersion\x12,\n(OFFLINE_QUERY_GIVENS_VERSION_UNSPECIFIED\x10\x00\x12(\n$OFFLINE_QUERY_GIVENS_VERSION_UNKNOWN\x10\x01\x12>\n:OFFLINE_QUERY_GIVENS_VERSION_NATIVE_TS_FEATURE_FOR_ROOT_NS\x10\x02\x12\x33\n/OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME\x10\x03\x12\x43\n?OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME_WITH_URI_PREFIX\x10\x04\x32\xd0\x03\n\x16\x44\x61tasetMetadataService\x12\x63\n\x0cListDatasets\x12$.chalk.server.v1.ListDatasetsRequest\x1a%.chalk.server.v1.ListDatasetsResponse"\x06\x90\x02\x01\x80}\x04\x12]\n\nGetDataset\x12".chalk.server.v1.GetDatasetRequest\x1a#.chalk.server.v1.GetDatasetResponse"\x06\x90\x02\x01\x80}\x04\x12{\n\x14ListDatasetRevisions\x12,.chalk.server.v1.ListDatasetRevisionsRequest\x1a-.chalk.server.v1.ListDatasetRevisionsResponse"\x06\x90\x02\x01\x80}\x04\x12u\n\x12GetDatasetRevision\x12*.chalk.server.v1.GetDatasetRevisionRequest\x1a+.chalk.server.v1.GetDatasetRevisionResponse"\x06\x90\x02\x01\x80}\x04\x42\x96\x01\n\x13\x63om.chalk.server.v1B\rDatasetsProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.datasets_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\rDatasetsProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_DATASETMETADATASERVICE"].methods_by_name["ListDatasets"]._options = None
    _globals["_DATASETMETADATASERVICE"].methods_by_name["ListDatasets"]._serialized_options = b"\220\002\001\200}\004"
    _globals["_DATASETMETADATASERVICE"].methods_by_name["GetDataset"]._options = None
    _globals["_DATASETMETADATASERVICE"].methods_by_name["GetDataset"]._serialized_options = b"\220\002\001\200}\004"
    _globals["_DATASETMETADATASERVICE"].methods_by_name["ListDatasetRevisions"]._options = None
    _globals["_DATASETMETADATASERVICE"].methods_by_name[
        "ListDatasetRevisions"
    ]._serialized_options = b"\220\002\001\200}\004"
    _globals["_DATASETMETADATASERVICE"].methods_by_name["GetDatasetRevision"]._options = None
    _globals["_DATASETMETADATASERVICE"].methods_by_name[
        "GetDatasetRevision"
    ]._serialized_options = b"\220\002\001\200}\004"
    _globals["_DATASETREVISIONSTATUS"]._serialized_start = 2590
    _globals["_DATASETREVISIONSTATUS"]._serialized_end = 2877
    _globals["_DATASETVERSION"]._serialized_start = 2880
    _globals["_DATASETVERSION"]._serialized_end = 3241
    _globals["_OFFLINEQUERYGIVENSVERSION"]._serialized_start = 3244
    _globals["_OFFLINEQUERYGIVENSVERSION"]._serialized_end = 3545
    _globals["_DATASETREVISIONMETA"]._serialized_start = 148
    _globals["_DATASETREVISIONMETA"]._serialized_end = 1482
    _globals["_DATASETMETA"]._serialized_start = 1485
    _globals["_DATASETMETA"]._serialized_end = 1787
    _globals["_LISTDATASETSREQUEST"]._serialized_start = 1790
    _globals["_LISTDATASETSREQUEST"]._serialized_end = 1928
    _globals["_LISTDATASETSRESPONSE"]._serialized_start = 1930
    _globals["_LISTDATASETSRESPONSE"]._serialized_end = 2050
    _globals["_GETDATASETREQUEST"]._serialized_start = 2052
    _globals["_GETDATASETREQUEST"]._serialized_end = 2087
    _globals["_GETDATASETRESPONSE"]._serialized_start = 2089
    _globals["_GETDATASETRESPONSE"]._serialized_end = 2165
    _globals["_LISTDATASETREVISIONSREQUEST"]._serialized_start = 2168
    _globals["_LISTDATASETREVISIONSREQUEST"]._serialized_end = 2305
    _globals["_LISTDATASETREVISIONSRESPONSE"]._serialized_start = 2308
    _globals["_LISTDATASETREVISIONSRESPONSE"]._serialized_end = 2446
    _globals["_GETDATASETREVISIONREQUEST"]._serialized_start = 2448
    _globals["_GETDATASETREVISIONREQUEST"]._serialized_end = 2491
    _globals["_GETDATASETREVISIONRESPONSE"]._serialized_start = 2493
    _globals["_GETDATASETREVISIONRESPONSE"]._serialized_end = 2587
    _globals["_DATASETMETADATASERVICE"]._serialized_start = 3548
    _globals["_DATASETMETADATASERVICE"]._serialized_end = 4012
# @@protoc_insertion_point(module_scope)
