"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.metrics_pb2 import (
    GetOverviewSummaryMetricsRequest,
    GetOverviewSummaryMetricsResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class MetricsServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    GetOverviewSummaryMetrics: UnaryUnaryMultiCallable[
        GetOverviewSummaryMetricsRequest,
        GetOverviewSummaryMetricsResponse,
    ]

class MetricsServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def GetOverviewSummaryMetrics(
        self,
        request: GetOverviewSummaryMetricsRequest,
        context: ServicerContext,
    ) -> GetOverviewSummaryMetricsResponse: ...

def add_MetricsServiceServicer_to_server(servicer: MetricsServiceServicer, server: Server) -> None: ...
