# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/metrics.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1d\x63halk/server/v1/metrics.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto"\x88\x01\n GetOverviewSummaryMetricsRequest\x12$\n\x0brange_start\x18\x01 \x01(\tH\x00R\nrangeStart\x88\x01\x01\x12 \n\trange_end\x18\x02 \x01(\tH\x01R\x08rangeEnd\x88\x01\x01\x42\x0e\n\x0c_range_startB\x0c\n\n_range_end"A\n\x15OverviewSummaryMetric\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x14\n\x05value\x18\x02 \x01(\x02R\x05value"e\n!GetOverviewSummaryMetricsResponse\x12@\n\x07metrics\x18\x01 \x03(\x0b\x32&.chalk.server.v1.OverviewSummaryMetricR\x07metrics2\x9a\x01\n\x0eMetricsService\x12\x87\x01\n\x19GetOverviewSummaryMetrics\x12\x31.chalk.server.v1.GetOverviewSummaryMetricsRequest\x1a\x32.chalk.server.v1.GetOverviewSummaryMetricsResponse"\x03\x80}\x06\x42\x95\x01\n\x13\x63om.chalk.server.v1B\x0cMetricsProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.metrics_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\014MetricsProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_METRICSSERVICE"].methods_by_name["GetOverviewSummaryMetrics"]._options = None
    _globals["_METRICSSERVICE"].methods_by_name["GetOverviewSummaryMetrics"]._serialized_options = b"\200}\006"
    _globals["_GETOVERVIEWSUMMARYMETRICSREQUEST"]._serialized_start = 84
    _globals["_GETOVERVIEWSUMMARYMETRICSREQUEST"]._serialized_end = 220
    _globals["_OVERVIEWSUMMARYMETRIC"]._serialized_start = 222
    _globals["_OVERVIEWSUMMARYMETRIC"]._serialized_end = 287
    _globals["_GETOVERVIEWSUMMARYMETRICSRESPONSE"]._serialized_start = 289
    _globals["_GETOVERVIEWSUMMARYMETRICSRESPONSE"]._serialized_end = 390
    _globals["_METRICSSERVICE"]._serialized_start = 393
    _globals["_METRICSSERVICE"]._serialized_end = 547
# @@protoc_insertion_point(module_scope)
