# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v1/offline_store_service.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import feature_values_pb2 as chalk_dot_common_dot_v1_dot_feature__values__pb2
from chalk._gen.chalk.common.v1 import query_log_pb2 as chalk_dot_common_dot_v1_dot_query__log__pb2
from chalk._gen.chalk.common.v1 import query_values_pb2 as chalk_dot_common_dot_v1_dot_query__values__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n+chalk/engine/v1/offline_store_service.proto\x12\x0f\x63halk.engine.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a$chalk/common/v1/feature_values.proto\x1a\x1f\x63halk/common/v1/query_log.proto\x1a"chalk/common/v1/query_values.proto"\x1a\n\x18OfflineStoreServiceDummy2\xba\x05\n\x13OfflineStoreService\x12r\n\x12GetQueryLogEntries\x12*.chalk.common.v1.GetQueryLogEntriesRequest\x1a+.chalk.common.v1.GetQueryLogEntriesResponse"\x03\x80}\x04\x12\x66\n\x0eGetQueryValues\x12&.chalk.common.v1.GetQueryValuesRequest\x1a\'.chalk.common.v1.GetQueryValuesResponse"\x03\x80}\x04\x12~\n\x15GetFeatureValuesChart\x12-.chalk.common.v1.GetFeatureValuesChartRequest\x1a..chalk.common.v1.GetFeatureValuesChartResponse"\x06\x88\x02\x01\x80}\x04\x12\x9c\x01\n\x1fGetFeatureValuesTimeSeriesChart\x12\x37.chalk.common.v1.GetFeatureValuesTimeSeriesChartRequest\x1a\x38.chalk.common.v1.GetFeatureValuesTimeSeriesChartResponse"\x06\x88\x02\x01\x80}\x04\x12\xa2\x01\n!GetFeatureValuesTimeSeriesChartV2\x12\x39.chalk.common.v1.GetFeatureValuesTimeSeriesChartV2Request\x1a:.chalk.common.v1.GetFeatureValuesTimeSeriesChartV2Response"\x06\x88\x02\x01\x80}\x04\x1a\x03\x88\x02\x01\x42\x8d\x01\n\x13\x63om.chalk.engine.v1B\x18OfflineStoreServiceProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V1\xca\x02\x0f\x43halk\\Engine\\V1\xe2\x02\x1b\x43halk\\Engine\\V1\\GPBMetadata\xea\x02\x11\x43halk::Engine::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v1.offline_store_service_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v1B\030OfflineStoreServiceProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V1\312\002\017Chalk\\Engine\\V1\342\002\033Chalk\\Engine\\V1\\GPBMetadata\352\002\021Chalk::Engine::V1"
    _globals["_OFFLINESTORESERVICE"]._options = None
    _globals["_OFFLINESTORESERVICE"]._serialized_options = b"\210\002\001"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryLogEntries"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryLogEntries"]._serialized_options = b"\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryValues"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryValues"]._serialized_options = b"\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesChart"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesChart"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesTimeSeriesChart"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesTimeSeriesChart"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesTimeSeriesChartV2"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesTimeSeriesChartV2"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICEDUMMY"]._serialized_start = 204
    _globals["_OFFLINESTORESERVICEDUMMY"]._serialized_end = 230
    _globals["_OFFLINESTORESERVICE"]._serialized_start = 233
    _globals["_OFFLINESTORESERVICE"]._serialized_end = 931
# @@protoc_insertion_point(module_scope)
