# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v1/plan_server.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.common.v1 import online_query_pb2 as chalk_dot_common_dot_v1_dot_online__query__pb2
from chalk._gen.chalk.engine.v1 import plan_pb2 as chalk_dot_engine_dot_v1_dot_plan__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!chalk/engine/v1/plan_server.proto\x12\x0f\x63halk.engine.v1\x1a"chalk/common/v1/online_query.proto\x1a\x1a\x63halk/engine/v1/plan.proto"g\n\x0eGetPlanRequest\x12U\n\x14online_query_request\x18\x01 \x01(\x0b\x32#.chalk.common.v1.OnlineQueryRequestR\x12onlineQueryRequest"<\n\x0fGetPlanResponse\x12)\n\x04plan\x18\x01 \x01(\x0b\x32\x15.chalk.engine.v1.PlanR\x04plan"l\n\x13\x45xecuteQueryRequest\x12U\n\x14online_query_request\x18\x01 \x01(\x0b\x32#.chalk.common.v1.OnlineQueryRequestR\x12onlineQueryRequest"p\n\x14\x45xecuteQueryResponse\x12X\n\x15online_query_response\x18\x01 \x01(\x0b\x32$.chalk.common.v1.OnlineQueryResponseR\x13onlineQueryResponse2\xbc\x01\n\x0bPlanService\x12N\n\x07GetPlan\x12\x1f.chalk.engine.v1.GetPlanRequest\x1a .chalk.engine.v1.GetPlanResponse"\x00\x12]\n\x0c\x45xecuteQuery\x12$.chalk.engine.v1.ExecuteQueryRequest\x1a%.chalk.engine.v1.ExecuteQueryResponse"\x00\x42\x84\x01\n\x13\x63om.chalk.engine.v1B\x0fPlanServerProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V1\xca\x02\x0f\x43halk\\Engine\\V1\xe2\x02\x1b\x43halk\\Engine\\V1\\GPBMetadata\xea\x02\x11\x43halk::Engine::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v1.plan_server_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v1B\017PlanServerProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V1\312\002\017Chalk\\Engine\\V1\342\002\033Chalk\\Engine\\V1\\GPBMetadata\352\002\021Chalk::Engine::V1"
    _globals["_GETPLANREQUEST"]._serialized_start = 118
    _globals["_GETPLANREQUEST"]._serialized_end = 221
    _globals["_GETPLANRESPONSE"]._serialized_start = 223
    _globals["_GETPLANRESPONSE"]._serialized_end = 283
    _globals["_EXECUTEQUERYREQUEST"]._serialized_start = 285
    _globals["_EXECUTEQUERYREQUEST"]._serialized_end = 393
    _globals["_EXECUTEQUERYRESPONSE"]._serialized_start = 395
    _globals["_EXECUTEQUERYRESPONSE"]._serialized_end = 507
    _globals["_PLANSERVICE"]._serialized_start = 510
    _globals["_PLANSERVICE"]._serialized_end = 698
# @@protoc_insertion_point(module_scope)
