# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from chalk._gen.chalk.server.v1 import monitoring_pb2 as chalk_dot_server_dot_v1_dot_monitoring__pb2


class MonitoringServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.TestPagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/TestPagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationResponse.FromString,
        )
        self.AddPagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/AddPagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationResponse.FromString,
        )
        self.DeletePagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/DeletePagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationResponse.FromString,
        )
        self.SetDefaultPagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/SetDefaultPagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationResponse.FromString,
        )
        self.UpdatePagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/UpdatePagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationResponse.FromString,
        )
        self.GetAllPagerDutyIntegrations = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/GetAllPagerDutyIntegrations",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsResponse.FromString,
        )
        self.GetPagerDutyIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/GetPagerDutyIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationResponse.FromString,
        )
        self.TestIncidentIoIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/TestIncidentIoIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationResponse.FromString,
        )
        self.AddIncidentIoIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/AddIncidentIoIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationResponse.FromString,
        )
        self.DeleteIncidentIoIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/DeleteIncidentIoIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationResponse.FromString,
        )
        self.UpdateIncidentIoIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/UpdateIncidentIoIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationResponse.FromString,
        )
        self.GetAllIncidentIoIntegrations = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/GetAllIncidentIoIntegrations",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsResponse.FromString,
        )
        self.GetIncidentIoIntegration = channel.unary_unary(
            "/chalk.server.v1.MonitoringService/GetIncidentIoIntegration",
            request_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationRequest.SerializeToString,
            response_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationResponse.FromString,
        )


class MonitoringServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def TestPagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AddPagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeletePagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SetDefaultPagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdatePagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetAllPagerDutyIntegrations(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetPagerDutyIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def TestIncidentIoIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AddIncidentIoIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteIncidentIoIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateIncidentIoIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetAllIncidentIoIntegrations(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetIncidentIoIntegration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MonitoringServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "TestPagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.TestPagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationResponse.SerializeToString,
        ),
        "AddPagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.AddPagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationResponse.SerializeToString,
        ),
        "DeletePagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.DeletePagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationResponse.SerializeToString,
        ),
        "SetDefaultPagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.SetDefaultPagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationResponse.SerializeToString,
        ),
        "UpdatePagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.UpdatePagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationResponse.SerializeToString,
        ),
        "GetAllPagerDutyIntegrations": grpc.unary_unary_rpc_method_handler(
            servicer.GetAllPagerDutyIntegrations,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsResponse.SerializeToString,
        ),
        "GetPagerDutyIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.GetPagerDutyIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationResponse.SerializeToString,
        ),
        "TestIncidentIoIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.TestIncidentIoIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationResponse.SerializeToString,
        ),
        "AddIncidentIoIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.AddIncidentIoIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationResponse.SerializeToString,
        ),
        "DeleteIncidentIoIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteIncidentIoIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationResponse.SerializeToString,
        ),
        "UpdateIncidentIoIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateIncidentIoIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationResponse.SerializeToString,
        ),
        "GetAllIncidentIoIntegrations": grpc.unary_unary_rpc_method_handler(
            servicer.GetAllIncidentIoIntegrations,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsResponse.SerializeToString,
        ),
        "GetIncidentIoIntegration": grpc.unary_unary_rpc_method_handler(
            servicer.GetIncidentIoIntegration,
            request_deserializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationRequest.FromString,
            response_serializer=chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("chalk.server.v1.MonitoringService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class MonitoringService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def TestPagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/TestPagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.TestPagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AddPagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/AddPagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.AddPagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeletePagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/DeletePagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.DeletePagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SetDefaultPagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/SetDefaultPagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.SetDefaultPagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdatePagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/UpdatePagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdatePagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetAllPagerDutyIntegrations(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/GetAllPagerDutyIntegrations",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllPagerDutyIntegrationsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetPagerDutyIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/GetPagerDutyIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetPagerDutyIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def TestIncidentIoIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/TestIncidentIoIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.TestIncidentIoIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AddIncidentIoIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/AddIncidentIoIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.AddIncidentIoIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteIncidentIoIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/DeleteIncidentIoIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.DeleteIncidentIoIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateIncidentIoIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/UpdateIncidentIoIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.UpdateIncidentIoIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetAllIncidentIoIntegrations(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/GetAllIncidentIoIntegrations",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetAllIncidentIoIntegrationsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetIncidentIoIntegration(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.server.v1.MonitoringService/GetIncidentIoIntegration",
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationRequest.SerializeToString,
            chalk_dot_server_dot_v1_dot_monitoring__pb2.GetIncidentIoIntegrationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
