"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.protosql.v1.sql_service_pb2 import (
    ExecuteSqlQueryRequest,
    ExecuteSqlQueryResponse,
    PlanSqlQueryRequest,
    PlanSqlQueryResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class SqlServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    ExecuteSqlQuery: UnaryUnaryMultiCallable[
        ExecuteSqlQueryRequest,
        ExecuteSqlQueryResponse,
    ]
    PlanSqlQuery: UnaryUnaryMultiCallable[
        PlanSqlQueryRequest,
        PlanSqlQueryResponse,
    ]

class SqlServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def ExecuteSqlQuery(
        self,
        request: ExecuteSqlQueryRequest,
        context: ServicerContext,
    ) -> ExecuteSqlQueryResponse: ...
    @abstractmethod
    def PlanSqlQuery(
        self,
        request: PlanSqlQueryRequest,
        context: ServicerContext,
    ) -> PlanSqlQueryResponse: ...

def add_SqlServiceServicer_to_server(servicer: SqlServiceServicer, server: Server) -> None: ...
