from rich.color import Color

PASTELY_CYAN: Color = Color.from_rgb(54, 165, 135)
UNDERLYING_CYAN: Color = Color.from_rgb(102, 204, 177)
CITRUSY_YELLOW: Color = Color.from_rgb(245, 247, 122)
GRASSY_GREEN: Color = Color.from_rgb(187, 217, 109)
SHY_RED: Color = Color.from_rgb(237, 130, 100)
ANCIENT_GREEN: Color = Color.from_rgb(15, 37, 31)
CHALK_WEBSITE_GREEN: Color = Color.from_rgb(36, 58, 52)
SERENDIPITOUS_PURPLE: Color = Color.from_rgb(146, 137, 204)
SHADOWY_LAVENDER: Color = Color.from_rgb(170, 165, 196)
