# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/builder.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.graph.v1 import graph_pb2 as chalk_dot_graph_dot_v1_dot_graph__pb2
from chalk._gen.chalk.lsp.v1 import lsp_pb2 as chalk_dot_lsp_dot_v1_dot_lsp__pb2
from chalk._gen.chalk.nodepools.v1 import gke_pb2 as chalk_dot_nodepools_dot_v1_dot_gke__pb2
from chalk._gen.chalk.nodepools.v1 import karpenter_pb2 as chalk_dot_nodepools_dot_v1_dot_karpenter__pb2
from chalk._gen.chalk.server.v1 import deployment_pb2 as chalk_dot_server_dot_v1_dot_deployment__pb2
from chalk._gen.chalk.server.v1 import log_pb2 as chalk_dot_server_dot_v1_dot_log__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1d\x63halk/server/v1/builder.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1a\x63halk/graph/v1/graph.proto\x1a\x16\x63halk/lsp/v1/lsp.proto\x1a\x1c\x63halk/nodepools/v1/gke.proto\x1a"chalk/nodepools/v1/karpenter.proto\x1a chalk/server/v1/deployment.proto\x1a\x19\x63halk/server/v1/log.proto\x1a\x1fgoogle/protobuf/timestamp.proto"m\n\x18\x41\x63tivateDeploymentTarget\x12!\n\x0cservice_kind\x18\x01 \x01(\tR\x0bserviceKind\x12.\n\x13resource_group_name\x18\x02 \x01(\tR\x11resourceGroupName"\x96\x01\n\x19\x41\x63tivateDeploymentRequest\x12\x34\n\x16\x65xisting_deployment_id\x18\x01 \x01(\tR\x14\x65xistingDeploymentId\x12\x43\n\x07targets\x18\x02 \x03(\x0b\x32).chalk.server.v1.ActivateDeploymentTargetR\x07targets"\x1c\n\x1a\x41\x63tivateDeploymentResponse"N\n\x16IndexDeploymentRequest\x12\x34\n\x16\x65xisting_deployment_id\x18\x01 \x01(\tR\x14\x65xistingDeploymentId"\x19\n\x17IndexDeploymentResponse"\x98\x01\n\x1b\x44\x65ployKubeComponentsRequest\x12\x34\n\x16\x65xisting_deployment_id\x18\x01 \x01(\tR\x14\x65xistingDeploymentId\x12\x43\n\x07targets\x18\x02 \x03(\x0b\x32).chalk.server.v1.ActivateDeploymentTargetR\x07targets"\x1e\n\x1c\x44\x65ployKubeComponentsResponse"\xec\x01\n\x18RebuildDeploymentRequest\x12\x34\n\x16\x65xisting_deployment_id\x18\x01 \x01(\tR\x14\x65xistingDeploymentId\x12"\n\rnew_image_tag\x18\x02 \x01(\tR\x0bnewImageTag\x12\x33\n\x13\x62\x61se_image_override\x18\x03 \x01(\tH\x00R\x11\x62\x61seImageOverride\x88\x01\x01\x12)\n\x10\x65nable_profiling\x18\x04 \x01(\x08R\x0f\x65nableProfilingB\x16\n\x14_base_image_override"6\n\x19RebuildDeploymentResponse\x12\x19\n\x08\x62uild_id\x18\x01 \x01(\tR\x07\x62uildId"\xf2\x01\n\x19RedeployDeploymentRequest\x12\x34\n\x16\x65xisting_deployment_id\x18\x01 \x01(\tR\x14\x65xistingDeploymentId\x12)\n\x10\x65nable_profiling\x18\x02 \x01(\x08R\x0f\x65nableProfiling\x12\'\n\x0f\x64\x65ployment_tags\x18\x03 \x03(\tR\x0e\x64\x65ploymentTags\x12\x33\n\x13\x62\x61se_image_override\x18\x04 \x01(\tH\x00R\x11\x62\x61seImageOverride\x88\x01\x01\x42\x16\n\x14_base_image_override"`\n\x1aRedeployDeploymentResponse\x12\x1d\n\x08\x62uild_id\x18\x01 \x01(\tB\x02\x18\x01R\x07\x62uildId\x12#\n\rdeployment_id\x18\x02 \x01(\tR\x0c\x64\x65ploymentId"\xc8\x02\n\x13UploadSourceRequest\x12#\n\rdeployment_id\x18\x07 \x01(\tR\x0c\x64\x65ploymentId\x12\x18\n\x07\x61rchive\x18\x01 \x01(\x0cR\x07\x61rchive\x12\x1d\n\nno_promote\x18\x02 \x01(\x08R\tnoPromote\x12,\n\x0f\x64\x65pendency_hash\x18\x03 \x01(\tH\x00R\x0e\x64\x65pendencyHash\x88\x01\x01\x12\x33\n\x13\x62\x61se_image_override\x18\x04 \x01(\tH\x01R\x11\x62\x61seImageOverride\x88\x01\x01\x12\x19\n\x08use_grpc\x18\x05 \x01(\x08R\x07useGrpc\x12)\n\x10\x65nable_profiling\x18\x06 \x01(\x08R\x0f\x65nableProfilingB\x12\n\x10_dependency_hashB\x16\n\x14_base_image_override"Q\n\x14UploadSourceResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12!\n\x0cprogress_url\x18\x02 \x01(\tR\x0bprogressUrl"-\n\x11LintSourceRequest\x12\x18\n\x07\x61rchive\x18\x01 \x01(\x0cR\x07\x61rchive"f\n\x12LintSourceResponse\x12+\n\x05graph\x18\x01 \x01(\x0b\x32\x15.chalk.graph.v1.GraphR\x05graph\x12#\n\x03lsp\x18\x02 \x01(\x0b\x32\x11.chalk.lsp.v1.LSPR\x03lsp"@\n\x19GetDeploymentStepsRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"\xfa\x01\n\x13\x44\x65ploymentBuildStep\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tR\x0b\x64isplayName\x12>\n\x06status\x18\x03 \x01(\x0e\x32&.chalk.server.v1.DeploymentBuildStatusR\x06status\x12\x39\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime"\x95\x01\n\x1aGetDeploymentStepsResponse\x12:\n\x05steps\x18\x01 \x03(\x0b\x32$.chalk.server.v1.DeploymentBuildStepR\x05steps\x12;\n\ndeployment\x18\x02 \x01(\x0b\x32\x1b.chalk.server.v1.DeploymentR\ndeployment"?\n\x18GetDeploymentLogsRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"J\n\x19GetDeploymentLogsResponse\x12-\n\x04logs\x18\x01 \x03(\x0b\x32\x19.chalk.server.v1.LogEntryR\x04logs"E\n\x1cGetClusterTimescaleDBRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId"\x8a\x02\n\x1dGetClusterTimescaleDBResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0cspecs_string\x18\x02 \x01(\tB\x02\x18\x01R\x0bspecsString\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12<\n\x05specs\x18\x05 \x01(\x0b\x32&.chalk.server.v1.ClusterTimescaleSpecsR\x05specs"A\n\x18GetClusterGatewayRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId"\x82\x02\n\x19GetClusterGatewayResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12%\n\x0cspecs_string\x18\x02 \x01(\tB\x02\x18\x01R\x0bspecsString\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x38\n\x05specs\x18\x05 \x01(\x0b\x32".chalk.server.v1.EnvoyGatewaySpecsR\x05specs"\xa5\x02\n\x15\x42\x61\x63kgroundPersistence\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04kind\x18\x02 \x01(\tR\x04kind\x12%\n\x0cspecs_string\x18\x03 \x01(\tB\x02\x18\x01R\x0bspecsString\x12\x39\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12K\n\x05specs\x18\x06 \x01(\x0b\x32\x35.chalk.server.v1.BackgroundPersistenceDeploymentSpecsR\x05specs"O\n&GetClusterBackgroundPersistenceRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId"\x88\x01\n\'GetClusterBackgroundPersistenceResponse\x12]\n\x16\x62\x61\x63kground_persistence\x18\x01 \x01(\x0b\x32&.chalk.server.v1.BackgroundPersistenceR\x15\x62\x61\x63kgroundPersistence"\xda\x01\n\x1f\x43reateClusterTimescaleDBRequest\x12)\n\x0e\x65nvironment_id\x18\x01 \x03(\tB\x02\x18\x01R\renvironmentId\x12\'\n\x0f\x65nvironment_ids\x18\x03 \x03(\tR\x0e\x65nvironmentIds\x12%\n\x0cspecs_string\x18\x02 \x01(\tB\x02\x18\x01R\x0bspecsString\x12<\n\x05specs\x18\x04 \x01(\x0b\x32&.chalk.server.v1.ClusterTimescaleSpecsR\x05specs"\x85\x01\n\x12KubeResourceConfig\x12\x10\n\x03\x63pu\x18\x01 \x01(\tR\x03\x63pu\x12\x16\n\x06memory\x18\x02 \x01(\tR\x06memory\x12+\n\x11\x65phemeral_storage\x18\x03 \x01(\tR\x10\x65phemeralStorage\x12\x18\n\x07storage\x18\x04 \x01(\tR\x07storage"\xba\t\n\x15\x43lusterTimescaleSpecs\x12\'\n\x0ftimescale_image\x18\x01 \x01(\tR\x0etimescaleImage\x12#\n\rdatabase_name\x18\x02 \x01(\tR\x0c\x64\x61tabaseName\x12+\n\x11\x64\x61tabase_replicas\x18\x03 \x01(\x05R\x10\x64\x61tabaseReplicas\x12\x18\n\x07storage\x18\x04 \x01(\tR\x07storage\x12(\n\rstorage_class\x18\x05 \x01(\tH\x00R\x0cstorageClass\x88\x01\x01\x12\x1c\n\tnamespace\x18\x06 \x01(\tR\tnamespace\x12\x42\n\x07request\x18\x07 \x01(\x0b\x32#.chalk.server.v1.KubeResourceConfigH\x01R\x07request\x88\x01\x01\x12>\n\x05limit\x18\x08 \x01(\x0b\x32#.chalk.server.v1.KubeResourceConfigH\x02R\x05limit\x88\x01\x01\x12\x38\n\x18\x63onnection_pool_replicas\x18\t \x01(\x05R\x16\x63onnectionPoolReplicas\x12\x45\n\x1f\x63onnection_pool_max_connections\x18\n \x01(\tR\x1c\x63onnectionPoolMaxConnections\x12\x30\n\x14\x63onnection_pool_size\x18\x0b \x01(\tR\x12\x63onnectionPoolSize\x12\x30\n\x14\x63onnection_pool_mode\x18\x0c \x01(\tR\x12\x63onnectionPoolMode\x12#\n\rbackup_bucket\x18\r \x01(\tR\x0c\x62\x61\x63kupBucket\x12-\n\x13\x62\x61\x63kup_iam_role_arn\x18\x0e \x01(\tR\x10\x62\x61\x63kupIamRoleArn\x12\x1f\n\x0bsecret_name\x18\x0f \x01(\tR\nsecretName\x12\x1f\n\x08internal\x18\x10 \x01(\x08H\x03R\x08internal\x88\x01\x01\x12&\n\x0cservice_type\x18\x11 \x01(\tH\x04R\x0bserviceType\x88\x01\x01\x12o\n\x13postgres_parameters\x18\x12 \x03(\x0b\x32>.chalk.server.v1.ClusterTimescaleSpecs.PostgresParametersEntryR\x12postgresParameters\x12=\n\x1binclude_chalk_node_selector\x18\x13 \x01(\x08R\x18includeChalkNodeSelector\x12;\n\x1a\x62\x61\x63kup_gcp_service_account\x18\x14 \x01(\tR\x17\x62\x61\x63kupGcpServiceAccount\x12#\n\rinstance_type\x18\x15 \x01(\tR\x0cinstanceType\x1a\x45\n\x17PostgresParametersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x10\n\x0e_storage_classB\n\n\x08_requestB\x08\n\x06_limitB\x0b\n\t_internalB\x0f\n\r_service_type""\n CreateClusterTimescaleDBResponse"\xbf\x01\n MigrateClusterTimescaleDBRequest\x12\x30\n\x14\x63luster_timescale_id\x18\x01 \x01(\tR\x12\x63lusterTimescaleId\x12,\n\x0fmigration_image\x18\x02 \x01(\tH\x00R\x0emigrationImage\x88\x01\x01\x12\'\n\x0f\x65nvironment_ids\x18\x03 \x03(\tR\x0e\x65nvironmentIdsB\x12\n\x10_migration_image"#\n!MigrateClusterTimescaleDBResponse"\xd2\x01\n\x1b\x43reateClusterGatewayRequest\x12)\n\x0e\x65nvironment_id\x18\x01 \x03(\tB\x02\x18\x01R\renvironmentId\x12\'\n\x0f\x65nvironment_ids\x18\x03 \x03(\tR\x0e\x65nvironmentIds\x12%\n\x0cspecs_string\x18\x02 \x01(\tB\x02\x18\x01R\x0bspecsString\x12\x38\n\x05specs\x18\x04 \x01(\x0b\x32".chalk.server.v1.EnvoyGatewaySpecsR\x05specs"\xd6\x02\n\x11\x45nvoyGatewaySpecs\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12!\n\x0cgateway_name\x18\x02 \x01(\tR\x0bgatewayName\x12,\n\x12gateway_class_name\x18\x03 \x01(\tR\x10gatewayClassName\x12\x43\n\tlisteners\x18\x04 \x03(\x0b\x32%.chalk.server.v1.EnvoyGatewayListenerR\tlisteners\x12\x43\n\x06\x63onfig\x18\x05 \x01(\x0b\x32&.chalk.server.v1.GatewayProviderConfigH\x00R\x06\x63onfig\x88\x01\x01\x12=\n\x1binclude_chalk_node_selector\x18\x06 \x01(\x08R\x18includeChalkNodeSelectorB\t\n\x07_config"\xad\x01\n\x14\x45nvoyGatewayListener\x12\x12\n\x04port\x18\x01 \x01(\x05R\x04port\x12\x1a\n\x08protocol\x18\x02 \x01(\tR\x08protocol\x12\x12\n\x04name\x18\x03 \x01(\tR\x04name\x12Q\n\x0e\x61llowed_routes\x18\x04 \x01(\x0b\x32*.chalk.server.v1.EnvoyGatewayAllowedRoutesR\rallowedRoutes"k\n\x19\x45nvoyGatewayAllowedRoutes\x12N\n\nnamespaces\x18\x01 \x01(\x0b\x32..chalk.server.v1.EnvoyGatewayAllowedNamespacesR\nnamespaces"3\n\x1d\x45nvoyGatewayAllowedNamespaces\x12\x12\n\x04\x66rom\x18\x01 \x01(\tR\x04\x66rom"\xa5\x01\n\x15GatewayProviderConfig\x12\x43\n\x05\x65nvoy\x18\x01 \x01(\x0b\x32+.chalk.server.v1.EnvoyGatewayProviderConfigH\x00R\x05\x65nvoy\x12=\n\x03gcp\x18\x02 \x01(\x0b\x32).chalk.server.v1.GCPGatewayProviderConfigH\x00R\x03gcpB\x08\n\x06\x63onfig"\x84\x02\n\x1a\x45nvoyGatewayProviderConfig\x12.\n\x10timeout_duration\x18\x01 \x01(\tH\x00R\x0ftimeoutDuration\x88\x01\x01\x12&\n\x0c\x64ns_hostname\x18\x02 \x01(\tH\x01R\x0b\x64nsHostname\x88\x01\x01\x12\x1f\n\x08replicas\x18\x03 \x01(\x05H\x02R\x08replicas\x88\x01\x01\x12(\n\rmin_available\x18\x04 \x01(\x05H\x03R\x0cminAvailable\x88\x01\x01\x42\x13\n\x11_timeout_durationB\x0f\n\r_dns_hostnameB\x0b\n\t_replicasB\x10\n\x0e_min_available"=\n\x18GCPGatewayProviderConfig\x12!\n\x0c\x64ns_hostname\x18\x01 \x01(\tR\x0b\x64nsHostname"\x1e\n\x1c\x43reateClusterGatewayResponse"\xc8\x01\n)CreateClusterBackgroundPersistenceRequest\x12\'\n\x0f\x65nvironment_ids\x18\x01 \x03(\tR\x0e\x65nvironmentIds\x12%\n\x0cspecs_string\x18\x02 \x01(\tB\x02\x18\x01R\x0bspecsString\x12K\n\x05specs\x18\x03 \x01(\x0b\x32\x35.chalk.server.v1.BackgroundPersistenceDeploymentSpecsR\x05specs"\x85\r\n BackgroundPersistenceCommonSpecs\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12-\n\x13\x62us_writer_image_go\x18\x02 \x01(\tR\x10\x62usWriterImageGo\x12\x35\n\x17\x62us_writer_image_python\x18\x03 \x01(\tR\x14\x62usWriterImagePython\x12\x31\n\x15\x62us_writer_image_bswl\x18\x04 \x01(\tR\x12\x62usWriterImageBswl\x12\x30\n\x14service_account_name\x18\x05 \x01(\tR\x12serviceAccountName\x12\x1f\n\x0b\x62us_backend\x18\x06 \x01(\tR\nbusBackend\x12#\n\rsecret_client\x18\x07 \x01(\tR\x0csecretClient\x12T\n\'bigquery_parquet_upload_subscription_id\x18\x08 \x01(\tR#bigqueryParquetUploadSubscriptionId\x12V\n(bigquery_streaming_write_subscription_id\x18\t \x01(\tR$bigqueryStreamingWriteSubscriptionId\x12\x43\n\x1e\x62igquery_streaming_write_topic\x18\n \x01(\tR\x1b\x62igqueryStreamingWriteTopic\x12\x38\n\x16\x62igquery_upload_bucket\x18\x0b \x01(\tB\x02\x18\x01R\x14\x62igqueryUploadBucket\x12\x36\n\x15\x62igquery_upload_topic\x18\x0c \x01(\tB\x02\x18\x01R\x13\x62igqueryUploadTopic\x12\x30\n\x14google_cloud_project\x18\r \x01(\tR\x12googleCloudProject\x12&\n\x0fkafka_dlq_topic\x18\x0e \x01(\tR\rkafkaDlqTopic\x12=\n\x1bmetrics_bus_subscription_id\x18\x0f \x01(\tR\x18metricsBusSubscriptionId\x12/\n\x14metrics_bus_topic_id\x18\x10 \x01(\tR\x11metricsBusTopicId\x12:\n\x19operation_subscription_id\x18\x11 \x01(\tR\x17operationSubscriptionId\x12\x33\n\x16query_log_result_topic\x18\x12 \x01(\tR\x13queryLogResultTopic\x12\x39\n\x19query_log_subscription_id\x18\x13 \x01(\tR\x16queryLogSubscriptionId\x12J\n"result_bus_metrics_subscription_id\x18\x14 \x01(\tR\x1eresultBusMetricsSubscriptionId\x12U\n(result_bus_offline_store_subscription_id\x18\x15 \x01(\tR#resultBusOfflineStoreSubscriptionId\x12S\n\'result_bus_online_store_subscription_id\x18\x16 \x01(\tR"resultBusOnlineStoreSubscriptionId\x12-\n\x13result_bus_topic_id\x18\x17 \x01(\tR\x10resultBusTopicId\x12+\n\x12usage_bus_topic_id\x18\x18 \x01(\tR\x0fusageBusTopicId\x12?\n\x1cusage_events_subscription_id\x18\x19 \x01(\tR\x19usageEventsSubscriptionId\x12(\n\x10\x62q_upload_bucket\x18\x1a \x01(\tR\x0e\x62qUploadBucket\x12&\n\x0f\x62q_upload_topic\x18\x1b \x01(\tR\rbqUploadTopic\x12=\n\x1binclude_chalk_node_selector\x18\x1c \x01(\x08R\x18includeChalkNodeSelector\x12\x31\n\x15\x62us_writer_image_rust\x18\x1d \x01(\tR\x12\x62usWriterImageRust"\xc5\x02\n#BackgroundPersistenceWriterHpaSpecs\x12;\n\x1ahpa_pubsub_subscription_id\x18\x01 \x01(\tR\x17hpaPubsubSubscriptionId\x12-\n\x10hpa_min_replicas\x18\x02 \x01(\x05H\x00R\x0ehpaMinReplicas\x88\x01\x01\x12-\n\x10hpa_max_replicas\x18\x03 \x01(\x05H\x01R\x0ehpaMaxReplicas\x88\x01\x01\x12<\n\x18hpa_target_average_value\x18\x04 \x01(\x05H\x02R\x15hpaTargetAverageValue\x88\x01\x01\x42\x13\n\x11_hpa_min_replicasB\x13\n\x11_hpa_max_replicasB\x1b\n\x19_hpa_target_average_value"\xd4\x0b\n BackgroundPersistenceWriterSpecs\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12%\n\x0eimage_override\x18\x02 \x01(\tR\rimageOverride\x12V\n\thpa_specs\x18\x03 \x01(\x0b\x32\x34.chalk.server.v1.BackgroundPersistenceWriterHpaSpecsH\x00R\x08hpaSpecs\x88\x01\x01\x12\x1e\n\x08gke_spot\x18\x04 \x01(\x08H\x01R\x07gkeSpot\x88\x01\x01\x12\x37\n\x15load_writer_configmap\x18\x05 \x01(\x08H\x02R\x13loadWriterConfigmap\x88\x01\x01\x12\x18\n\x07version\x18\x06 \x01(\tR\x07version\x12\x42\n\x07request\x18\x07 \x01(\x0b\x32#.chalk.server.v1.KubeResourceConfigH\x03R\x07request\x88\x01\x01\x12>\n\x05limit\x18\x08 \x01(\x0b\x32#.chalk.server.v1.KubeResourceConfigH\x04R\x05limit\x88\x01\x01\x12.\n\x13\x62us_subscriber_type\x18\t \x01(\tR\x11\x62usSubscriberType\x12\x32\n\x15\x64\x65\x66\x61ult_replica_count\x18\n \x01(\x05R\x13\x64\x65\x66\x61ultReplicaCount\x12\x41\n\x1dkafka_consumer_group_override\x18\x0b \x01(\tR\x1akafkaConsumerGroupOverride\x12)\n\x0emax_batch_size\x18\x0c \x01(\x05H\x05R\x0cmaxBatchSize\x88\x01\x01\x12I\n\x1emessage_processing_concurrency\x18\r \x01(\x05H\x06R\x1cmessageProcessingConcurrency\x88\x01\x01\x12\x43\n\x1fmetadata_sql_ssl_ca_cert_secret\x18\x0e \x01(\tR\x1ametadataSqlSslCaCertSecret\x12K\n#metadata_sql_ssl_client_cert_secret\x18\x0f \x01(\tR\x1emetadataSqlSslClientCertSecret\x12I\n"metadata_sql_ssl_client_key_secret\x18\x10 \x01(\tR\x1dmetadataSqlSslClientKeySecret\x12\x35\n\x17metadata_sql_uri_secret\x18\x11 \x01(\tR\x14metadataSqlUriSecret\x12\x42\n\x1eoffline_store_inserter_db_type\x18\x12 \x01(\tR\x1aofflineStoreInserterDbType\x12\x30\n\x14storage_cache_prefix\x18\x13 \x01(\tR\x12storageCachePrefix\x12&\n\x0fusage_store_uri\x18\x14 \x01(\tR\rusageStoreUri\x12\x64\n-results_writer_skip_producing_feature_metrics\x18\x15 \x01(\x08H\x07R(resultsWriterSkipProducingFeatureMetrics\x88\x01\x01\x12>\n\x1cquery_table_write_drop_ratio\x18\x16 \x01(\tR\x18queryTableWriteDropRatioB\x0c\n\n_hpa_specsB\x0b\n\t_gke_spotB\x18\n\x16_load_writer_configmapB\n\n\x08_requestB\x08\n\x06_limitB\x11\n\x0f_max_batch_sizeB!\n\x1f_message_processing_concurrencyB0\n._results_writer_skip_producing_feature_metrics"\xe4\x05\n$BackgroundPersistenceDeploymentSpecs\x12k\n\x18\x63ommon_persistence_specs\x18\x01 \x01(\x0b\x32\x31.chalk.server.v1.BackgroundPersistenceCommonSpecsR\x16\x63ommonPersistenceSpecs\x12&\n\x0f\x61pi_server_host\x18\x02 \x01(\tR\rapiServerHost\x12*\n\x11kafka_sasl_secret\x18\x03 \x01(\tR\x0fkafkaSaslSecret\x12+\n\x11metadata_provider\x18\x04 \x01(\tR\x10metadataProvider\x12\x36\n\x17kafka_bootstrap_servers\x18\x05 \x01(\tR\x15kafkaBootstrapServers\x12\x36\n\x17kafka_security_protocol\x18\x06 \x01(\tR\x15kafkaSecurityProtocol\x12\x30\n\x14kafka_sasl_mechanism\x18\x07 \x01(\tR\x12kafkaSaslMechanism\x12,\n\x12redis_is_clustered\x18\x08 \x01(\tR\x10redisIsClustered\x12K\n"snowflake_storage_integration_name\x18\t \x01(\tR\x1fsnowflakeStorageIntegrationName\x12H\n!redis_lightning_supports_has_many\x18\n \x01(\x08R\x1dredisLightningSupportsHasMany\x12\x1a\n\x08insecure\x18\x0b \x01(\x08R\x08insecure\x12K\n\x07writers\x18\x0c \x03(\x0b\x32\x31.chalk.server.v1.BackgroundPersistenceWriterSpecsR\x07writers",\n*CreateClusterBackgroundPersistenceResponse"\x18\n\x16GetSearchConfigRequest"T\n\x17GetSearchConfigResponse\x12\x17\n\x07team_id\x18\x01 \x01(\tR\x06teamId\x12 \n\x0cteam_api_key\x18\x02 \x01(\tR\nteamApiKey"\xf0\x01\n!UpdateEnvironmentVariablesRequest\x12\x81\x01\n\x15\x65nvironment_variables\x18\x01 \x03(\x0b\x32L.chalk.server.v1.UpdateEnvironmentVariablesRequest.EnvironmentVariablesEntryR\x14\x65nvironmentVariables\x1aG\n\x19\x45nvironmentVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"$\n"UpdateEnvironmentVariablesResponse"\x14\n\x12StartBranchRequest"P\n\x13StartBranchResponse\x12\x39\n\x05state\x18\x01 \x01(\x0e\x32#.chalk.server.v1.BranchScalingStateR\x05state"\x9a\x01\n\nKafkaTopic\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1e\n\npartitions\x18\x02 \x01(\x05R\npartitions\x12%\n\x0breplication\x18\x03 \x01(\x05H\x00R\x0breplication\x88\x01\x01\x12!\n\x0cretention_ms\x18\x04 \x01(\x05R\x0bretentionMsB\x0e\n\x0c_replication"O\n\x18\x43reateKafkaTopicsRequest\x12\x33\n\x06topics\x18\x01 \x03(\x0b\x32\x1b.chalk.server.v1.KafkaTopicR\x06topics"\x1b\n\x19\x43reateKafkaTopicsResponse"\x17\n\x15GetKafkaTopicsRequest"M\n\x16GetKafkaTopicsResponse\x12\x33\n\x06topics\x18\x01 \x03(\x0b\x32\x1b.chalk.server.v1.KafkaTopicR\x06topics"\x15\n\x13GetNodepoolsRequest"\xb4\x01\n\x14GetNodepoolsResponse\x12V\n\x13karpenter_nodepools\x18\x01 \x03(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\x12karpenterNodepools\x12\x44\n\rgke_nodepools\x18\x02 \x03(\x0b\x32\x1f.chalk.nodepools.v1.GKENodePoolR\x0cgkeNodepools"\xbe\x01\n\x12\x41\x64\x64NodepoolRequest\x12V\n\x12karpenter_nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolH\x00R\x11karpenterNodepool\x12\x44\n\x0cgke_nodepool\x18\x02 \x01(\x0b\x32\x1f.chalk.nodepools.v1.GKENodePoolH\x00R\x0bgkeNodepoolB\n\n\x08nodepool"\xbf\x01\n\x13\x41\x64\x64NodepoolResponse\x12V\n\x12karpenter_nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolH\x00R\x11karpenterNodepool\x12\x44\n\x0cgke_nodepool\x18\x02 \x01(\x0b\x32\x1f.chalk.nodepools.v1.GKENodePoolH\x00R\x0bgkeNodepoolB\n\n\x08nodepool"\xd5\x01\n\x15UpdateNodepoolRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x44\n\x0cgke_nodepool\x18\x02 \x01(\x0b\x32\x1f.chalk.nodepools.v1.GKENodePoolH\x00R\x0bgkeNodepool\x12V\n\x12karpenter_nodepool\x18\x03 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolH\x00R\x11karpenterNodepoolB\n\n\x08nodepool"\xc2\x01\n\x16UpdateNodepoolResponse\x12V\n\x12karpenter_nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolH\x00R\x11karpenterNodepool\x12\x44\n\x0cgke_nodepool\x18\x02 \x01(\x0b\x32\x1f.chalk.nodepools.v1.GKENodePoolH\x00R\x0bgkeNodepoolB\n\n\x08nodepool"+\n\x15\x44\x65leteNodepoolRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name"\x18\n\x16\x44\x65leteNodepoolResponse"\x1e\n\x1cGetKarpenterNodepoolsRequest"d\n\x1dGetKarpenterNodepoolsResponse\x12\x43\n\tnodepools\x18\x01 \x03(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\tnodepools"`\n\x1b\x41\x64\x64KarpenterNodepoolRequest\x12\x41\n\x08nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\x08nodepool"a\n\x1c\x41\x64\x64KarpenterNodepoolResponse\x12\x41\n\x08nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\x08nodepool"w\n\x1eUpdateKarpenterNodepoolRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x41\n\x08nodepool\x18\x02 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\x08nodepool"d\n\x1fUpdateKarpenterNodepoolResponse\x12\x41\n\x08nodepool\x18\x01 \x01(\x0b\x32%.chalk.nodepools.v1.KarpenterNodepoolR\x08nodepool"4\n\x1e\x44\x65leteKarpenterNodepoolRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name"!\n\x1f\x44\x65leteKarpenterNodepoolResponse")\n\'GetKarpenterInstallationMetadataRequest"\xed\x01\n(GetKarpenterInstallationMetadataResponse\x12|\n\x11\x64\x65ployment_labels\x18\x01 \x03(\x0b\x32O.chalk.server.v1.GetKarpenterInstallationMetadataResponse.DeploymentLabelsEntryR\x10\x64\x65ploymentLabels\x1a\x43\n\x15\x44\x65ploymentLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xc1\x01\n\rDeploymentTag\x12\x10\n\x03tag\x18\x01 \x01(\tR\x03tag\x12\x1b\n\x06weight\x18\x02 \x01(\x05H\x00R\x06weight\x88\x01\x01\x12(\n\rdeployment_id\x18\x03 \x01(\tH\x01R\x0c\x64\x65ploymentId\x88\x01\x01\x12(\n\rmirror_weight\x18\x04 \x01(\x05H\x02R\x0cmirrorWeight\x88\x01\x01\x42\t\n\x07_weightB\x10\n\x0e_deployment_idB\x10\n\x0e_mirror_weight"\x16\n\x14GetTagWeightsRequest"K\n\x15GetTagWeightsResponse\x12\x32\n\x04tags\x18\x01 \x03(\x0b\x32\x1e.chalk.server.v1.DeploymentTagR\x04tags"J\n\x14SetTagWeightsRequest\x12\x32\n\x04tags\x18\x01 \x03(\x0b\x32\x1e.chalk.server.v1.DeploymentTagR\x04tags"K\n\x15SetTagWeightsResponse\x12\x32\n\x04tags\x18\x01 \x03(\x0b\x32\x1e.chalk.server.v1.DeploymentTagR\x04tags*\xe3\x03\n\x15\x44\x65ploymentBuildStatus\x12\'\n#DEPLOYMENT_BUILD_STATUS_UNSPECIFIED\x10\x00\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_UNKNOWN\x10\x01\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_PENDING\x10\x02\x12"\n\x1e\x44\x45PLOYMENT_BUILD_STATUS_QUEUED\x10\x03\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_WORKING\x10\x04\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_SUCCESS\x10\x05\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_FAILURE\x10\x06\x12*\n&DEPLOYMENT_BUILD_STATUS_INTERNAL_ERROR\x10\x07\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_TIMEOUT\x10\x08\x12%\n!DEPLOYMENT_BUILD_STATUS_CANCELLED\x10\t\x12#\n\x1f\x44\x45PLOYMENT_BUILD_STATUS_EXPIRED\x10\n\x12\'\n#DEPLOYMENT_BUILD_STATUS_BOOT_ERRORS\x10\x0b*\x82\x01\n\x12\x42ranchScalingState\x12$\n BRANCH_SCALING_STATE_UNSPECIFIED\x10\x00\x12 \n\x1c\x42RANCH_SCALING_STATE_SUCCESS\x10\x01\x12$\n BRANCH_SCALING_STATE_IN_PROGRESS\x10\x02\x32\x87\x1c\n\x0e\x42uilderService\x12l\n\x0fGetSearchConfig\x12\'.chalk.server.v1.GetSearchConfigRequest\x1a(.chalk.server.v1.GetSearchConfigResponse"\x06\x90\x02\x01\x80}\x02\x12r\n\x12\x41\x63tivateDeployment\x12*.chalk.server.v1.ActivateDeploymentRequest\x1a+.chalk.server.v1.ActivateDeploymentResponse"\x03\x80}\x0e\x12i\n\x0fIndexDeployment\x12\'.chalk.server.v1.IndexDeploymentRequest\x1a(.chalk.server.v1.IndexDeploymentResponse"\x03\x80}\x0e\x12x\n\x14\x44\x65ployKubeComponents\x12,.chalk.server.v1.DeployKubeComponentsRequest\x1a-.chalk.server.v1.DeployKubeComponentsResponse"\x03\x80}\x0c\x12o\n\x11RebuildDeployment\x12).chalk.server.v1.RebuildDeploymentRequest\x1a*.chalk.server.v1.RebuildDeploymentResponse"\x03\x80}\x0e\x12r\n\x12RedeployDeployment\x12*.chalk.server.v1.RedeployDeploymentRequest\x1a+.chalk.server.v1.RedeployDeploymentResponse"\x03\x80}\x0e\x12`\n\x0cUploadSource\x12$.chalk.server.v1.UploadSourceRequest\x1a%.chalk.server.v1.UploadSourceResponse"\x03\x80}\x0b\x12Z\n\nLintSource\x12".chalk.server.v1.LintSourceRequest\x1a#.chalk.server.v1.LintSourceResponse"\x03\x80}\x0b\x12r\n\x12GetDeploymentSteps\x12*.chalk.server.v1.GetDeploymentStepsRequest\x1a+.chalk.server.v1.GetDeploymentStepsResponse"\x03\x80}\x0b\x12o\n\x11GetDeploymentLogs\x12).chalk.server.v1.GetDeploymentLogsRequest\x1a*.chalk.server.v1.GetDeploymentLogsResponse"\x03\x80}\x0b\x12{\n\x15GetClusterTimescaleDB\x12-.chalk.server.v1.GetClusterTimescaleDBRequest\x1a..chalk.server.v1.GetClusterTimescaleDBResponse"\x03\x80}\x0b\x12o\n\x11GetClusterGateway\x12).chalk.server.v1.GetClusterGatewayRequest\x1a*.chalk.server.v1.GetClusterGatewayResponse"\x03\x80}\x0b\x12\x99\x01\n\x1fGetClusterBackgroundPersistence\x12\x37.chalk.server.v1.GetClusterBackgroundPersistenceRequest\x1a\x38.chalk.server.v1.GetClusterBackgroundPersistenceResponse"\x03\x80}\x0b\x12\x84\x01\n\x18\x43reateClusterTimescaleDB\x12\x30.chalk.server.v1.CreateClusterTimescaleDBRequest\x1a\x31.chalk.server.v1.CreateClusterTimescaleDBResponse"\x03\x80}\n\x12\x87\x01\n\x19MigrateClusterTimescaleDB\x12\x31.chalk.server.v1.MigrateClusterTimescaleDBRequest\x1a\x32.chalk.server.v1.MigrateClusterTimescaleDBResponse"\x03\x80}\n\x12x\n\x14\x43reateClusterGateway\x12,.chalk.server.v1.CreateClusterGatewayRequest\x1a-.chalk.server.v1.CreateClusterGatewayResponse"\x03\x80}\n\x12\xa2\x01\n"CreateClusterBackgroundPersistence\x12:.chalk.server.v1.CreateClusterBackgroundPersistenceRequest\x1a;.chalk.server.v1.CreateClusterBackgroundPersistenceResponse"\x03\x80}\n\x12\x8a\x01\n\x1aUpdateEnvironmentVariables\x12\x32.chalk.server.v1.UpdateEnvironmentVariablesRequest\x1a\x33.chalk.server.v1.UpdateEnvironmentVariablesResponse"\x03\x80}\x0c\x12]\n\x0bStartBranch\x12#.chalk.server.v1.StartBranchRequest\x1a$.chalk.server.v1.StartBranchResponse"\x03\x80}\x02\x12\x63\n\x0cGetNodepools\x12$.chalk.server.v1.GetNodepoolsRequest\x1a%.chalk.server.v1.GetNodepoolsResponse"\x06\x90\x02\x01\x80}\x0b\x12]\n\x0b\x41\x64\x64Nodepool\x12#.chalk.server.v1.AddNodepoolRequest\x1a$.chalk.server.v1.AddNodepoolResponse"\x03\x80}\x0c\x12\x66\n\x0eUpdateNodepool\x12&.chalk.server.v1.UpdateNodepoolRequest\x1a\'.chalk.server.v1.UpdateNodepoolResponse"\x03\x80}\x0c\x12\x66\n\x0e\x44\x65leteNodepool\x12&.chalk.server.v1.DeleteNodepoolRequest\x1a\'.chalk.server.v1.DeleteNodepoolResponse"\x03\x80}\x0c\x12~\n\x15GetKarpenterNodepools\x12-.chalk.server.v1.GetKarpenterNodepoolsRequest\x1a..chalk.server.v1.GetKarpenterNodepoolsResponse"\x06\x90\x02\x01\x80}\x0b\x12x\n\x14\x41\x64\x64KarpenterNodepool\x12,.chalk.server.v1.AddKarpenterNodepoolRequest\x1a-.chalk.server.v1.AddKarpenterNodepoolResponse"\x03\x80}\x0c\x12\x81\x01\n\x17UpdateKarpenterNodepool\x12/.chalk.server.v1.UpdateKarpenterNodepoolRequest\x1a\x30.chalk.server.v1.UpdateKarpenterNodepoolResponse"\x03\x80}\x0c\x12\x81\x01\n\x17\x44\x65leteKarpenterNodepool\x12/.chalk.server.v1.DeleteKarpenterNodepoolRequest\x1a\x30.chalk.server.v1.DeleteKarpenterNodepoolResponse"\x03\x80}\x0c\x12\x9f\x01\n GetKarpenterInstallationMetadata\x12\x38.chalk.server.v1.GetKarpenterInstallationMetadataRequest\x1a\x39.chalk.server.v1.GetKarpenterInstallationMetadataResponse"\x06\x90\x02\x01\x80}\x0b\x12\x63\n\rGetTagWeights\x12%.chalk.server.v1.GetTagWeightsRequest\x1a&.chalk.server.v1.GetTagWeightsResponse"\x03\x80}\x0b\x12\x63\n\rSetTagWeights\x12%.chalk.server.v1.SetTagWeightsRequest\x1a&.chalk.server.v1.SetTagWeightsResponse"\x03\x80}\x0c\x32\xf0\x01\n\x15\x43lusterBuilderService\x12o\n\x11\x43reateKafkaTopics\x12).chalk.server.v1.CreateKafkaTopicsRequest\x1a*.chalk.server.v1.CreateKafkaTopicsResponse"\x03\x80}\n\x12\x66\n\x0eGetKafkaTopics\x12&.chalk.server.v1.GetKafkaTopicsRequest\x1a\'.chalk.server.v1.GetKafkaTopicsResponse"\x03\x80}\nB\x95\x01\n\x13\x63om.chalk.server.v1B\x0c\x42uilderProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.builder_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\014BuilderProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_REDEPLOYDEPLOYMENTRESPONSE"].fields_by_name["build_id"]._options = None
    _globals["_REDEPLOYDEPLOYMENTRESPONSE"].fields_by_name["build_id"]._serialized_options = b"\030\001"
    _globals["_GETCLUSTERTIMESCALEDBRESPONSE"].fields_by_name["specs_string"]._options = None
    _globals["_GETCLUSTERTIMESCALEDBRESPONSE"].fields_by_name["specs_string"]._serialized_options = b"\030\001"
    _globals["_GETCLUSTERGATEWAYRESPONSE"].fields_by_name["specs_string"]._options = None
    _globals["_GETCLUSTERGATEWAYRESPONSE"].fields_by_name["specs_string"]._serialized_options = b"\030\001"
    _globals["_BACKGROUNDPERSISTENCE"].fields_by_name["specs_string"]._options = None
    _globals["_BACKGROUNDPERSISTENCE"].fields_by_name["specs_string"]._serialized_options = b"\030\001"
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"].fields_by_name["environment_id"]._options = None
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"].fields_by_name["environment_id"]._serialized_options = b"\030\001"
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"].fields_by_name["specs_string"]._options = None
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"].fields_by_name["specs_string"]._serialized_options = b"\030\001"
    _globals["_CLUSTERTIMESCALESPECS_POSTGRESPARAMETERSENTRY"]._options = None
    _globals["_CLUSTERTIMESCALESPECS_POSTGRESPARAMETERSENTRY"]._serialized_options = b"8\001"
    _globals["_CREATECLUSTERGATEWAYREQUEST"].fields_by_name["environment_id"]._options = None
    _globals["_CREATECLUSTERGATEWAYREQUEST"].fields_by_name["environment_id"]._serialized_options = b"\030\001"
    _globals["_CREATECLUSTERGATEWAYREQUEST"].fields_by_name["specs_string"]._options = None
    _globals["_CREATECLUSTERGATEWAYREQUEST"].fields_by_name["specs_string"]._serialized_options = b"\030\001"
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCEREQUEST"].fields_by_name["specs_string"]._options = None
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCEREQUEST"].fields_by_name[
        "specs_string"
    ]._serialized_options = b"\030\001"
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"].fields_by_name["bigquery_upload_bucket"]._options = None
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"].fields_by_name[
        "bigquery_upload_bucket"
    ]._serialized_options = b"\030\001"
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"].fields_by_name["bigquery_upload_topic"]._options = None
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"].fields_by_name[
        "bigquery_upload_topic"
    ]._serialized_options = b"\030\001"
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST_ENVIRONMENTVARIABLESENTRY"]._options = None
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST_ENVIRONMENTVARIABLESENTRY"]._serialized_options = b"8\001"
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE_DEPLOYMENTLABELSENTRY"]._options = None
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE_DEPLOYMENTLABELSENTRY"]._serialized_options = b"8\001"
    _globals["_BUILDERSERVICE"].methods_by_name["GetSearchConfig"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetSearchConfig"]._serialized_options = b"\220\002\001\200}\002"
    _globals["_BUILDERSERVICE"].methods_by_name["ActivateDeployment"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["ActivateDeployment"]._serialized_options = b"\200}\016"
    _globals["_BUILDERSERVICE"].methods_by_name["IndexDeployment"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["IndexDeployment"]._serialized_options = b"\200}\016"
    _globals["_BUILDERSERVICE"].methods_by_name["DeployKubeComponents"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["DeployKubeComponents"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["RebuildDeployment"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["RebuildDeployment"]._serialized_options = b"\200}\016"
    _globals["_BUILDERSERVICE"].methods_by_name["RedeployDeployment"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["RedeployDeployment"]._serialized_options = b"\200}\016"
    _globals["_BUILDERSERVICE"].methods_by_name["UploadSource"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["UploadSource"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["LintSource"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["LintSource"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetDeploymentSteps"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetDeploymentSteps"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetDeploymentLogs"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetDeploymentLogs"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterTimescaleDB"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterTimescaleDB"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterGateway"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterGateway"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterBackgroundPersistence"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetClusterBackgroundPersistence"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterTimescaleDB"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterTimescaleDB"]._serialized_options = b"\200}\n"
    _globals["_BUILDERSERVICE"].methods_by_name["MigrateClusterTimescaleDB"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["MigrateClusterTimescaleDB"]._serialized_options = b"\200}\n"
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterGateway"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterGateway"]._serialized_options = b"\200}\n"
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterBackgroundPersistence"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["CreateClusterBackgroundPersistence"]._serialized_options = b"\200}\n"
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateEnvironmentVariables"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateEnvironmentVariables"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["StartBranch"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["StartBranch"]._serialized_options = b"\200}\002"
    _globals["_BUILDERSERVICE"].methods_by_name["GetNodepools"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetNodepools"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["AddNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["AddNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["DeleteNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["DeleteNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["GetKarpenterNodepools"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetKarpenterNodepools"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["AddKarpenterNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["AddKarpenterNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateKarpenterNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["UpdateKarpenterNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["DeleteKarpenterNodepool"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["DeleteKarpenterNodepool"]._serialized_options = b"\200}\014"
    _globals["_BUILDERSERVICE"].methods_by_name["GetKarpenterInstallationMetadata"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name[
        "GetKarpenterInstallationMetadata"
    ]._serialized_options = b"\220\002\001\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["GetTagWeights"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["GetTagWeights"]._serialized_options = b"\200}\013"
    _globals["_BUILDERSERVICE"].methods_by_name["SetTagWeights"]._options = None
    _globals["_BUILDERSERVICE"].methods_by_name["SetTagWeights"]._serialized_options = b"\200}\014"
    _globals["_CLUSTERBUILDERSERVICE"].methods_by_name["CreateKafkaTopics"]._options = None
    _globals["_CLUSTERBUILDERSERVICE"].methods_by_name["CreateKafkaTopics"]._serialized_options = b"\200}\n"
    _globals["_CLUSTERBUILDERSERVICE"].methods_by_name["GetKafkaTopics"]._options = None
    _globals["_CLUSTERBUILDERSERVICE"].methods_by_name["GetKafkaTopics"]._serialized_options = b"\200}\n"
    _globals["_DEPLOYMENTBUILDSTATUS"]._serialized_start = 14950
    _globals["_DEPLOYMENTBUILDSTATUS"]._serialized_end = 15433
    _globals["_BRANCHSCALINGSTATE"]._serialized_start = 15436
    _globals["_BRANCHSCALINGSTATE"]._serialized_end = 15566
    _globals["_ACTIVATEDEPLOYMENTTARGET"]._serialized_start = 295
    _globals["_ACTIVATEDEPLOYMENTTARGET"]._serialized_end = 404
    _globals["_ACTIVATEDEPLOYMENTREQUEST"]._serialized_start = 407
    _globals["_ACTIVATEDEPLOYMENTREQUEST"]._serialized_end = 557
    _globals["_ACTIVATEDEPLOYMENTRESPONSE"]._serialized_start = 559
    _globals["_ACTIVATEDEPLOYMENTRESPONSE"]._serialized_end = 587
    _globals["_INDEXDEPLOYMENTREQUEST"]._serialized_start = 589
    _globals["_INDEXDEPLOYMENTREQUEST"]._serialized_end = 667
    _globals["_INDEXDEPLOYMENTRESPONSE"]._serialized_start = 669
    _globals["_INDEXDEPLOYMENTRESPONSE"]._serialized_end = 694
    _globals["_DEPLOYKUBECOMPONENTSREQUEST"]._serialized_start = 697
    _globals["_DEPLOYKUBECOMPONENTSREQUEST"]._serialized_end = 849
    _globals["_DEPLOYKUBECOMPONENTSRESPONSE"]._serialized_start = 851
    _globals["_DEPLOYKUBECOMPONENTSRESPONSE"]._serialized_end = 881
    _globals["_REBUILDDEPLOYMENTREQUEST"]._serialized_start = 884
    _globals["_REBUILDDEPLOYMENTREQUEST"]._serialized_end = 1120
    _globals["_REBUILDDEPLOYMENTRESPONSE"]._serialized_start = 1122
    _globals["_REBUILDDEPLOYMENTRESPONSE"]._serialized_end = 1176
    _globals["_REDEPLOYDEPLOYMENTREQUEST"]._serialized_start = 1179
    _globals["_REDEPLOYDEPLOYMENTREQUEST"]._serialized_end = 1421
    _globals["_REDEPLOYDEPLOYMENTRESPONSE"]._serialized_start = 1423
    _globals["_REDEPLOYDEPLOYMENTRESPONSE"]._serialized_end = 1519
    _globals["_UPLOADSOURCEREQUEST"]._serialized_start = 1522
    _globals["_UPLOADSOURCEREQUEST"]._serialized_end = 1850
    _globals["_UPLOADSOURCERESPONSE"]._serialized_start = 1852
    _globals["_UPLOADSOURCERESPONSE"]._serialized_end = 1933
    _globals["_LINTSOURCEREQUEST"]._serialized_start = 1935
    _globals["_LINTSOURCEREQUEST"]._serialized_end = 1980
    _globals["_LINTSOURCERESPONSE"]._serialized_start = 1982
    _globals["_LINTSOURCERESPONSE"]._serialized_end = 2084
    _globals["_GETDEPLOYMENTSTEPSREQUEST"]._serialized_start = 2086
    _globals["_GETDEPLOYMENTSTEPSREQUEST"]._serialized_end = 2150
    _globals["_DEPLOYMENTBUILDSTEP"]._serialized_start = 2153
    _globals["_DEPLOYMENTBUILDSTEP"]._serialized_end = 2403
    _globals["_GETDEPLOYMENTSTEPSRESPONSE"]._serialized_start = 2406
    _globals["_GETDEPLOYMENTSTEPSRESPONSE"]._serialized_end = 2555
    _globals["_GETDEPLOYMENTLOGSREQUEST"]._serialized_start = 2557
    _globals["_GETDEPLOYMENTLOGSREQUEST"]._serialized_end = 2620
    _globals["_GETDEPLOYMENTLOGSRESPONSE"]._serialized_start = 2622
    _globals["_GETDEPLOYMENTLOGSRESPONSE"]._serialized_end = 2696
    _globals["_GETCLUSTERTIMESCALEDBREQUEST"]._serialized_start = 2698
    _globals["_GETCLUSTERTIMESCALEDBREQUEST"]._serialized_end = 2767
    _globals["_GETCLUSTERTIMESCALEDBRESPONSE"]._serialized_start = 2770
    _globals["_GETCLUSTERTIMESCALEDBRESPONSE"]._serialized_end = 3036
    _globals["_GETCLUSTERGATEWAYREQUEST"]._serialized_start = 3038
    _globals["_GETCLUSTERGATEWAYREQUEST"]._serialized_end = 3103
    _globals["_GETCLUSTERGATEWAYRESPONSE"]._serialized_start = 3106
    _globals["_GETCLUSTERGATEWAYRESPONSE"]._serialized_end = 3364
    _globals["_BACKGROUNDPERSISTENCE"]._serialized_start = 3367
    _globals["_BACKGROUNDPERSISTENCE"]._serialized_end = 3660
    _globals["_GETCLUSTERBACKGROUNDPERSISTENCEREQUEST"]._serialized_start = 3662
    _globals["_GETCLUSTERBACKGROUNDPERSISTENCEREQUEST"]._serialized_end = 3741
    _globals["_GETCLUSTERBACKGROUNDPERSISTENCERESPONSE"]._serialized_start = 3744
    _globals["_GETCLUSTERBACKGROUNDPERSISTENCERESPONSE"]._serialized_end = 3880
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"]._serialized_start = 3883
    _globals["_CREATECLUSTERTIMESCALEDBREQUEST"]._serialized_end = 4101
    _globals["_KUBERESOURCECONFIG"]._serialized_start = 4104
    _globals["_KUBERESOURCECONFIG"]._serialized_end = 4237
    _globals["_CLUSTERTIMESCALESPECS"]._serialized_start = 4240
    _globals["_CLUSTERTIMESCALESPECS"]._serialized_end = 5450
    _globals["_CLUSTERTIMESCALESPECS_POSTGRESPARAMETERSENTRY"]._serialized_start = 5311
    _globals["_CLUSTERTIMESCALESPECS_POSTGRESPARAMETERSENTRY"]._serialized_end = 5380
    _globals["_CREATECLUSTERTIMESCALEDBRESPONSE"]._serialized_start = 5452
    _globals["_CREATECLUSTERTIMESCALEDBRESPONSE"]._serialized_end = 5486
    _globals["_MIGRATECLUSTERTIMESCALEDBREQUEST"]._serialized_start = 5489
    _globals["_MIGRATECLUSTERTIMESCALEDBREQUEST"]._serialized_end = 5680
    _globals["_MIGRATECLUSTERTIMESCALEDBRESPONSE"]._serialized_start = 5682
    _globals["_MIGRATECLUSTERTIMESCALEDBRESPONSE"]._serialized_end = 5717
    _globals["_CREATECLUSTERGATEWAYREQUEST"]._serialized_start = 5720
    _globals["_CREATECLUSTERGATEWAYREQUEST"]._serialized_end = 5930
    _globals["_ENVOYGATEWAYSPECS"]._serialized_start = 5933
    _globals["_ENVOYGATEWAYSPECS"]._serialized_end = 6275
    _globals["_ENVOYGATEWAYLISTENER"]._serialized_start = 6278
    _globals["_ENVOYGATEWAYLISTENER"]._serialized_end = 6451
    _globals["_ENVOYGATEWAYALLOWEDROUTES"]._serialized_start = 6453
    _globals["_ENVOYGATEWAYALLOWEDROUTES"]._serialized_end = 6560
    _globals["_ENVOYGATEWAYALLOWEDNAMESPACES"]._serialized_start = 6562
    _globals["_ENVOYGATEWAYALLOWEDNAMESPACES"]._serialized_end = 6613
    _globals["_GATEWAYPROVIDERCONFIG"]._serialized_start = 6616
    _globals["_GATEWAYPROVIDERCONFIG"]._serialized_end = 6781
    _globals["_ENVOYGATEWAYPROVIDERCONFIG"]._serialized_start = 6784
    _globals["_ENVOYGATEWAYPROVIDERCONFIG"]._serialized_end = 7044
    _globals["_GCPGATEWAYPROVIDERCONFIG"]._serialized_start = 7046
    _globals["_GCPGATEWAYPROVIDERCONFIG"]._serialized_end = 7107
    _globals["_CREATECLUSTERGATEWAYRESPONSE"]._serialized_start = 7109
    _globals["_CREATECLUSTERGATEWAYRESPONSE"]._serialized_end = 7139
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCEREQUEST"]._serialized_start = 7142
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCEREQUEST"]._serialized_end = 7342
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"]._serialized_start = 7345
    _globals["_BACKGROUNDPERSISTENCECOMMONSPECS"]._serialized_end = 9014
    _globals["_BACKGROUNDPERSISTENCEWRITERHPASPECS"]._serialized_start = 9017
    _globals["_BACKGROUNDPERSISTENCEWRITERHPASPECS"]._serialized_end = 9342
    _globals["_BACKGROUNDPERSISTENCEWRITERSPECS"]._serialized_start = 9345
    _globals["_BACKGROUNDPERSISTENCEWRITERSPECS"]._serialized_end = 10837
    _globals["_BACKGROUNDPERSISTENCEDEPLOYMENTSPECS"]._serialized_start = 10840
    _globals["_BACKGROUNDPERSISTENCEDEPLOYMENTSPECS"]._serialized_end = 11580
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCERESPONSE"]._serialized_start = 11582
    _globals["_CREATECLUSTERBACKGROUNDPERSISTENCERESPONSE"]._serialized_end = 11626
    _globals["_GETSEARCHCONFIGREQUEST"]._serialized_start = 11628
    _globals["_GETSEARCHCONFIGREQUEST"]._serialized_end = 11652
    _globals["_GETSEARCHCONFIGRESPONSE"]._serialized_start = 11654
    _globals["_GETSEARCHCONFIGRESPONSE"]._serialized_end = 11738
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST"]._serialized_start = 11741
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST"]._serialized_end = 11981
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST_ENVIRONMENTVARIABLESENTRY"]._serialized_start = 11910
    _globals["_UPDATEENVIRONMENTVARIABLESREQUEST_ENVIRONMENTVARIABLESENTRY"]._serialized_end = 11981
    _globals["_UPDATEENVIRONMENTVARIABLESRESPONSE"]._serialized_start = 11983
    _globals["_UPDATEENVIRONMENTVARIABLESRESPONSE"]._serialized_end = 12019
    _globals["_STARTBRANCHREQUEST"]._serialized_start = 12021
    _globals["_STARTBRANCHREQUEST"]._serialized_end = 12041
    _globals["_STARTBRANCHRESPONSE"]._serialized_start = 12043
    _globals["_STARTBRANCHRESPONSE"]._serialized_end = 12123
    _globals["_KAFKATOPIC"]._serialized_start = 12126
    _globals["_KAFKATOPIC"]._serialized_end = 12280
    _globals["_CREATEKAFKATOPICSREQUEST"]._serialized_start = 12282
    _globals["_CREATEKAFKATOPICSREQUEST"]._serialized_end = 12361
    _globals["_CREATEKAFKATOPICSRESPONSE"]._serialized_start = 12363
    _globals["_CREATEKAFKATOPICSRESPONSE"]._serialized_end = 12390
    _globals["_GETKAFKATOPICSREQUEST"]._serialized_start = 12392
    _globals["_GETKAFKATOPICSREQUEST"]._serialized_end = 12415
    _globals["_GETKAFKATOPICSRESPONSE"]._serialized_start = 12417
    _globals["_GETKAFKATOPICSRESPONSE"]._serialized_end = 12494
    _globals["_GETNODEPOOLSREQUEST"]._serialized_start = 12496
    _globals["_GETNODEPOOLSREQUEST"]._serialized_end = 12517
    _globals["_GETNODEPOOLSRESPONSE"]._serialized_start = 12520
    _globals["_GETNODEPOOLSRESPONSE"]._serialized_end = 12700
    _globals["_ADDNODEPOOLREQUEST"]._serialized_start = 12703
    _globals["_ADDNODEPOOLREQUEST"]._serialized_end = 12893
    _globals["_ADDNODEPOOLRESPONSE"]._serialized_start = 12896
    _globals["_ADDNODEPOOLRESPONSE"]._serialized_end = 13087
    _globals["_UPDATENODEPOOLREQUEST"]._serialized_start = 13090
    _globals["_UPDATENODEPOOLREQUEST"]._serialized_end = 13303
    _globals["_UPDATENODEPOOLRESPONSE"]._serialized_start = 13306
    _globals["_UPDATENODEPOOLRESPONSE"]._serialized_end = 13500
    _globals["_DELETENODEPOOLREQUEST"]._serialized_start = 13502
    _globals["_DELETENODEPOOLREQUEST"]._serialized_end = 13545
    _globals["_DELETENODEPOOLRESPONSE"]._serialized_start = 13547
    _globals["_DELETENODEPOOLRESPONSE"]._serialized_end = 13571
    _globals["_GETKARPENTERNODEPOOLSREQUEST"]._serialized_start = 13573
    _globals["_GETKARPENTERNODEPOOLSREQUEST"]._serialized_end = 13603
    _globals["_GETKARPENTERNODEPOOLSRESPONSE"]._serialized_start = 13605
    _globals["_GETKARPENTERNODEPOOLSRESPONSE"]._serialized_end = 13705
    _globals["_ADDKARPENTERNODEPOOLREQUEST"]._serialized_start = 13707
    _globals["_ADDKARPENTERNODEPOOLREQUEST"]._serialized_end = 13803
    _globals["_ADDKARPENTERNODEPOOLRESPONSE"]._serialized_start = 13805
    _globals["_ADDKARPENTERNODEPOOLRESPONSE"]._serialized_end = 13902
    _globals["_UPDATEKARPENTERNODEPOOLREQUEST"]._serialized_start = 13904
    _globals["_UPDATEKARPENTERNODEPOOLREQUEST"]._serialized_end = 14023
    _globals["_UPDATEKARPENTERNODEPOOLRESPONSE"]._serialized_start = 14025
    _globals["_UPDATEKARPENTERNODEPOOLRESPONSE"]._serialized_end = 14125
    _globals["_DELETEKARPENTERNODEPOOLREQUEST"]._serialized_start = 14127
    _globals["_DELETEKARPENTERNODEPOOLREQUEST"]._serialized_end = 14179
    _globals["_DELETEKARPENTERNODEPOOLRESPONSE"]._serialized_start = 14181
    _globals["_DELETEKARPENTERNODEPOOLRESPONSE"]._serialized_end = 14214
    _globals["_GETKARPENTERINSTALLATIONMETADATAREQUEST"]._serialized_start = 14216
    _globals["_GETKARPENTERINSTALLATIONMETADATAREQUEST"]._serialized_end = 14257
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE"]._serialized_start = 14260
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE"]._serialized_end = 14497
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE_DEPLOYMENTLABELSENTRY"]._serialized_start = 14430
    _globals["_GETKARPENTERINSTALLATIONMETADATARESPONSE_DEPLOYMENTLABELSENTRY"]._serialized_end = 14497
    _globals["_DEPLOYMENTTAG"]._serialized_start = 14500
    _globals["_DEPLOYMENTTAG"]._serialized_end = 14693
    _globals["_GETTAGWEIGHTSREQUEST"]._serialized_start = 14695
    _globals["_GETTAGWEIGHTSREQUEST"]._serialized_end = 14717
    _globals["_GETTAGWEIGHTSRESPONSE"]._serialized_start = 14719
    _globals["_GETTAGWEIGHTSRESPONSE"]._serialized_end = 14794
    _globals["_SETTAGWEIGHTSREQUEST"]._serialized_start = 14796
    _globals["_SETTAGWEIGHTSREQUEST"]._serialized_end = 14870
    _globals["_SETTAGWEIGHTSRESPONSE"]._serialized_start = 14872
    _globals["_SETTAGWEIGHTSRESPONSE"]._serialized_end = 14947
    _globals["_BUILDERSERVICE"]._serialized_start = 15569
    _globals["_BUILDERSERVICE"]._serialized_end = 19160
    _globals["_CLUSTERBUILDERSERVICE"]._serialized_start = 19163
    _globals["_CLUSTERBUILDERSERVICE"]._serialized_end = 19403
# @@protoc_insertion_point(module_scope)
