# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/python/v1/types.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.arrow.v1 import arrow_pb2 as chalk_dot_arrow_dot_v1_dot_arrow__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/python/v1/types.proto\x12\x0f\x63halk.python.v1\x1a\x1a\x63halk/arrow/v1/arrow.proto"2\n\x05TySet\x12)\n\x05items\x18\x01 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x05items"3\n\x06TyList\x12)\n\x05items\x18\x01 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x05items"7\n\nTyIterable\x12)\n\x05items\x18\x01 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x05items"(\n\nTyDatetime\x12\x13\n\x02tz\x18\x01 \x01(\tH\x00R\x02tz\x88\x01\x01\x42\x05\n\x03_tz"Z\n\x06TyDict\x12%\n\x03key\x18\x01 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x03key\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x05value"U\n\x07TyTuple\x12)\n\x05\x66ixed\x18\x01 \x03(\x0b\x32\x13.chalk.python.v1.TyR\x05\x66ixed\x12\x1f\n\x0bis_variable\x18\x02 \x01(\x08R\nisVariable"\xbd\x06\n\x02Ty\x12\x1a\n\x08nullable\x18\x01 \x01(\x08R\x08nullable\x12\x31\n\x03int\x18\x02 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x03int\x12\x31\n\x03str\x18\x03 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x03str\x12\x33\n\x04\x62ool\x18\x04 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x04\x62ool\x12\x35\n\x05\x66loat\x18\x05 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x05\x66loat\x12-\n\x04list\x18\x06 \x01(\x0b\x32\x17.chalk.python.v1.TyListH\x00R\x04list\x12*\n\x03set\x18\x07 \x01(\x0b\x32\x16.chalk.python.v1.TySetH\x00R\x03set\x12\x33\n\x04none\x18\x08 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x04none\x12\x31\n\x03\x61ny\x18\t \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x03\x61ny\x12\x39\n\x08iterable\x18\n \x01(\x0b\x32\x1b.chalk.python.v1.TyIterableH\x00R\x08iterable\x12\x39\n\x08\x64\x61tetime\x18\x0b \x01(\x0b\x32\x1b.chalk.python.v1.TyDatetimeH\x00R\x08\x64\x61tetime\x12=\n\ttimedelta\x18\x0c \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\ttimedelta\x12\x33\n\x04\x64\x61te\x18\r \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x04\x64\x61te\x12\x30\n\x05tuple\x18\x0e \x01(\x0b\x32\x18.chalk.python.v1.TyTupleH\x00R\x05tuple\x12-\n\x04\x64ict\x18\x0f \x01(\x0b\x32\x17.chalk.python.v1.TyDictH\x00R\x04\x64ict\x12\x35\n\x05\x62ytes\x18\x10 \x01(\x0b\x32\x1d.chalk.python.v1.EmptyMessageH\x00R\x05\x62ytesB\x04\n\x02ty"\x0e\n\x0c\x45mptyMessage"g\n\rSymbolicConst\x12#\n\x02ty\x18\x01 \x01(\x0b\x32\x13.chalk.python.v1.TyR\x02ty\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value":\n\x0c\x43odeVariable\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n\x06module\x18\x02 \x01(\tR\x06module"\x84\x01\n\x11\x43odeVariableValue\x12\x39\n\x08variable\x18\x01 \x01(\x0b\x32\x1d.chalk.python.v1.CodeVariableR\x08variable\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32\x1e.chalk.python.v1.SymbolicConstR\x05value"\x9e\x02\n\x13GlobalVariablesInfo\x12I\n\x0e\x63ode_variables\x18\x01 \x03(\x0b\x32".chalk.python.v1.CodeVariableValueR\rcodeVariables\x12s\n\x15\x65nvironment_variables\x18\x02 \x03(\x0b\x32>.chalk.python.v1.GlobalVariablesInfo.EnvironmentVariablesEntryR\x14\x65nvironmentVariables\x1aG\n\x19\x45nvironmentVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x7f\n\x13\x63om.chalk.python.v1B\nTypesProtoP\x01\xa2\x02\x03\x43PX\xaa\x02\x0f\x43halk.Python.V1\xca\x02\x0f\x43halk\\Python\\V1\xe2\x02\x1b\x43halk\\Python\\V1\\GPBMetadata\xea\x02\x11\x43halk::Python::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.python.v1.types_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.python.v1B\nTypesProtoP\001\242\002\003CPX\252\002\017Chalk.Python.V1\312\002\017Chalk\\Python\\V1\342\002\033Chalk\\Python\\V1\\GPBMetadata\352\002\021Chalk::Python::V1"
    _globals["_GLOBALVARIABLESINFO_ENVIRONMENTVARIABLESENTRY"]._options = None
    _globals["_GLOBALVARIABLESINFO_ENVIRONMENTVARIABLESENTRY"]._serialized_options = b"8\001"
    _globals["_TYSET"]._serialized_start = 76
    _globals["_TYSET"]._serialized_end = 126
    _globals["_TYLIST"]._serialized_start = 128
    _globals["_TYLIST"]._serialized_end = 179
    _globals["_TYITERABLE"]._serialized_start = 181
    _globals["_TYITERABLE"]._serialized_end = 236
    _globals["_TYDATETIME"]._serialized_start = 238
    _globals["_TYDATETIME"]._serialized_end = 278
    _globals["_TYDICT"]._serialized_start = 280
    _globals["_TYDICT"]._serialized_end = 370
    _globals["_TYTUPLE"]._serialized_start = 372
    _globals["_TYTUPLE"]._serialized_end = 457
    _globals["_TY"]._serialized_start = 460
    _globals["_TY"]._serialized_end = 1289
    _globals["_EMPTYMESSAGE"]._serialized_start = 1291
    _globals["_EMPTYMESSAGE"]._serialized_end = 1305
    _globals["_SYMBOLICCONST"]._serialized_start = 1307
    _globals["_SYMBOLICCONST"]._serialized_end = 1410
    _globals["_CODEVARIABLE"]._serialized_start = 1412
    _globals["_CODEVARIABLE"]._serialized_end = 1470
    _globals["_CODEVARIABLEVALUE"]._serialized_start = 1473
    _globals["_CODEVARIABLEVALUE"]._serialized_end = 1605
    _globals["_GLOBALVARIABLESINFO"]._serialized_start = 1608
    _globals["_GLOBALVARIABLESINFO"]._serialized_end = 1894
    _globals["_GLOBALVARIABLESINFO_ENVIRONMENTVARIABLESENTRY"]._serialized_start = 1823
    _globals["_GLOBALVARIABLESINFO_ENVIRONMENTVARIABLESENTRY"]._serialized_end = 1894
# @@protoc_insertion_point(module_scope)
