# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/graph/v2/sources.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/graph/v2/sources.proto\x12\x0e\x63halk.graph.v2\x1a\x1cgoogle/protobuf/struct.proto"N\n\x17\x44\x61tabaseSourceReference\x12\x1f\n\x0bsource_type\x18\x01 \x01(\tR\nsourceType\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name"\xe0\x01\n\x0e\x44\x61tabaseSource\x12\x1f\n\x0bsource_type\x18\x01 \x01(\tR\nsourceType\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x45\n\x07options\x18\x03 \x03(\x0b\x32+.chalk.graph.v2.DatabaseSource.OptionsEntryR\x07options\x1aR\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01"\xc3\x02\n\x13\x44\x61tabaseSourceGroup\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12N\n\x0e\x64\x65\x66\x61ult_source\x18\x02 \x01(\x0b\x32\'.chalk.graph.v2.DatabaseSourceReferenceR\rdefaultSource\x12]\n\x0etagged_sources\x18\x03 \x03(\x0b\x32\x36.chalk.graph.v2.DatabaseSourceGroup.TaggedSourcesEntryR\rtaggedSources\x1ai\n\x12TaggedSourcesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12=\n\x05value\x18\x02 \x01(\x0b\x32\'.chalk.graph.v2.DatabaseSourceReferenceR\x05value:\x02\x38\x01"L\n\x15StreamSourceReference\x12\x1f\n\x0bsource_type\x18\x01 \x01(\tR\nsourceType\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name"\xdc\x01\n\x0cStreamSource\x12\x1f\n\x0bsource_type\x18\x01 \x01(\tR\nsourceType\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x43\n\x07options\x18\x03 \x03(\x0b\x32).chalk.graph.v2.StreamSource.OptionsEntryR\x07options\x1aR\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01"\x91\x01\n\rSourceSecrets\x12\x44\n\x07secrets\x18\x01 \x03(\x0b\x32*.chalk.graph.v2.SourceSecrets.SecretsEntryR\x07secrets\x1a:\n\x0cSecretsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42|\n\x12\x63om.chalk.graph.v2B\x0cSourcesProtoP\x01\xa2\x02\x03\x43GX\xaa\x02\x0e\x43halk.Graph.V2\xca\x02\x0e\x43halk\\Graph\\V2\xe2\x02\x1a\x43halk\\Graph\\V2\\GPBMetadata\xea\x02\x10\x43halk::Graph::V2b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.graph.v2.sources_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\022com.chalk.graph.v2B\014SourcesProtoP\001\242\002\003CGX\252\002\016Chalk.Graph.V2\312\002\016Chalk\\Graph\\V2\342\002\032Chalk\\Graph\\V2\\GPBMetadata\352\002\020Chalk::Graph::V2"
    _globals["_DATABASESOURCE_OPTIONSENTRY"]._options = None
    _globals["_DATABASESOURCE_OPTIONSENTRY"]._serialized_options = b"8\001"
    _globals["_DATABASESOURCEGROUP_TAGGEDSOURCESENTRY"]._options = None
    _globals["_DATABASESOURCEGROUP_TAGGEDSOURCESENTRY"]._serialized_options = b"8\001"
    _globals["_STREAMSOURCE_OPTIONSENTRY"]._options = None
    _globals["_STREAMSOURCE_OPTIONSENTRY"]._serialized_options = b"8\001"
    _globals["_SOURCESECRETS_SECRETSENTRY"]._options = None
    _globals["_SOURCESECRETS_SECRETSENTRY"]._serialized_options = b"8\001"
    _globals["_DATABASESOURCEREFERENCE"]._serialized_start = 78
    _globals["_DATABASESOURCEREFERENCE"]._serialized_end = 156
    _globals["_DATABASESOURCE"]._serialized_start = 159
    _globals["_DATABASESOURCE"]._serialized_end = 383
    _globals["_DATABASESOURCE_OPTIONSENTRY"]._serialized_start = 301
    _globals["_DATABASESOURCE_OPTIONSENTRY"]._serialized_end = 383
    _globals["_DATABASESOURCEGROUP"]._serialized_start = 386
    _globals["_DATABASESOURCEGROUP"]._serialized_end = 709
    _globals["_DATABASESOURCEGROUP_TAGGEDSOURCESENTRY"]._serialized_start = 604
    _globals["_DATABASESOURCEGROUP_TAGGEDSOURCESENTRY"]._serialized_end = 709
    _globals["_STREAMSOURCEREFERENCE"]._serialized_start = 711
    _globals["_STREAMSOURCEREFERENCE"]._serialized_end = 787
    _globals["_STREAMSOURCE"]._serialized_start = 790
    _globals["_STREAMSOURCE"]._serialized_end = 1010
    _globals["_STREAMSOURCE_OPTIONSENTRY"]._serialized_start = 301
    _globals["_STREAMSOURCE_OPTIONSENTRY"]._serialized_end = 383
    _globals["_SOURCESECRETS"]._serialized_start = 1013
    _globals["_SOURCESECRETS"]._serialized_end = 1158
    _globals["_SOURCESECRETS_SECRETSENTRY"]._serialized_start = 1100
    _globals["_SOURCESECRETS_SECRETSENTRY"]._serialized_end = 1158
# @@protoc_insertion_point(module_scope)
