# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/common/v1/chart.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/common/v1/chart.proto\x12\x0f\x63halk.common.v1"P\n\x06Series\x12\x16\n\x06points\x18\x01 \x03(\x01R\x06points\x12\x14\n\x05label\x18\x02 \x01(\tR\x05label\x12\x14\n\x05units\x18\x03 \x01(\tR\x05units:\x02\x18\x01"x\n\x05\x43hart\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12/\n\x06series\x18\x02 \x03(\x0b\x32\x17.chalk.common.v1.SeriesR\x06series\x12$\n\x0ex_timestamp_ms\x18\x04 \x03(\x03R\x0cxTimestampMs:\x02\x18\x01\x42\x7f\n\x13\x63om.chalk.common.v1B\nChartProtoP\x01\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43halk.Common.V1\xca\x02\x0f\x43halk\\Common\\V1\xe2\x02\x1b\x43halk\\Common\\V1\\GPBMetadata\xea\x02\x11\x43halk::Common::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.common.v1.chart_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.common.v1B\nChartProtoP\001\242\002\003CCX\252\002\017Chalk.Common.V1\312\002\017Chalk\\Common\\V1\342\002\033Chalk\\Common\\V1\\GPBMetadata\352\002\021Chalk::Common::V1"
    _globals["_SERIES"]._options = None
    _globals["_SERIES"]._serialized_options = b"\030\001"
    _globals["_CHART"]._options = None
    _globals["_CHART"]._serialized_options = b"\030\001"
    _globals["_SERIES"]._serialized_start = 48
    _globals["_SERIES"]._serialized_end = 128
    _globals["_CHART"]._serialized_start = 130
    _globals["_CHART"]._serialized_end = 250
# @@protoc_insertion_point(module_scope)
