# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/nodepools/v1/gke.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.rpc import code_pb2 as google_dot_rpc_dot_code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/nodepools/v1/gke.proto\x12\x12\x63halk.nodepools.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x15google/rpc/code.proto"\xa0\x03\n\x0fStatusCondition\x12@\n\x04\x63ode\x18\x01 \x01(\x0e\x32(.chalk.nodepools.v1.StatusCondition.CodeB\x02\x18\x01R\x04\x63ode\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x37\n\x0e\x63\x61nonical_code\x18\x03 \x01(\x0e\x32\x10.google.rpc.CodeR\rcanonicalCode"\xf7\x01\n\x04\x43ode\x12\x14\n\x10\x43ODE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x43ODE_GCE_STOCKOUT\x10\x01\x12$\n CODE_GKE_SERVICE_ACCOUNT_DELETED\x10\x02\x12\x1b\n\x17\x43ODE_GCE_QUOTA_EXCEEDED\x10\x03\x12\x18\n\x14\x43ODE_SET_BY_OPERATOR\x10\x04\x12\x1c\n\x18\x43ODE_CLOUD_KMS_KEY_ERROR\x10\x07\x12\x14\n\x10\x43ODE_CA_EXPIRING\x10\t\x12\x31\n-CODE_NODE_SERVICE_ACCOUNT_MISSING_PERMISSIONS\x10\n"\xd2\x03\n\x13NodePoolAutoscaling\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12$\n\x0emin_node_count\x18\x02 \x01(\x05R\x0cminNodeCount\x12$\n\x0emax_node_count\x18\x03 \x01(\x05R\x0cmaxNodeCount\x12(\n\x0f\x61utoprovisioned\x18\x04 \x01(\x08R\x0f\x61utoprovisioned\x12_\n\x0flocation_policy\x18\x05 \x01(\x0e\x32\x36.chalk.nodepools.v1.NodePoolAutoscaling.LocationPolicyR\x0elocationPolicy\x12/\n\x14total_min_node_count\x18\x06 \x01(\x05R\x11totalMinNodeCount\x12/\n\x14total_max_node_count\x18\x07 \x01(\x05R\x11totalMaxNodeCount"h\n\x0eLocationPolicy\x12\x1f\n\x1bLOCATION_POLICY_UNSPECIFIED\x10\x00\x12\x1c\n\x18LOCATION_POLICY_BALANCED\x10\x01\x12\x17\n\x13LOCATION_POLICY_ANY\x10\x02"\xe1\x01\n\tNodeTaint\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\x12<\n\x06\x65\x66\x66\x65\x63t\x18\x03 \x01(\x0e\x32$.chalk.nodepools.v1.NodeTaint.EffectR\x06\x65\x66\x66\x65\x63t"n\n\x06\x45\x66\x66\x65\x63t\x12\x16\n\x12\x45\x46\x46\x45\x43T_UNSPECIFIED\x10\x00\x12\x16\n\x12\x45\x46\x46\x45\x43T_NO_SCHEDULE\x10\x01\x12\x1d\n\x19\x45\x46\x46\x45\x43T_PREFER_NO_SCHEDULE\x10\x02\x12\x15\n\x11\x45\x46\x46\x45\x43T_NO_EXECUTE\x10\x03">\n\x11MaxPodsConstraint\x12)\n\x11max_pods_per_node\x18\x01 \x01(\x03R\x0emaxPodsPerNode"b\n\x16\x42\x65stEffortProvisioning\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12.\n\x13min_provision_nodes\x18\x02 \x01(\x05R\x11minProvisionNodes"\xa5\x01\n\x0eNodeManagement\x12!\n\x0c\x61uto_upgrade\x18\x01 \x01(\x08R\x0b\x61utoUpgrade\x12\x1f\n\x0b\x61uto_repair\x18\x02 \x01(\x08R\nautoRepair\x12O\n\x0fupgrade_options\x18\n \x01(\x0b\x32&.chalk.nodepools.v1.AutoUpgradeOptionsR\x0eupgradeOptions"m\n\x12\x41utoUpgradeOptions\x12\x35\n\x17\x61uto_upgrade_start_time\x18\x01 \x01(\tR\x14\x61utoUpgradeStartTime\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription"\xc4\x0b\n\nNodeConfig\x12!\n\x0cmachine_type\x18\x01 \x01(\tR\x0bmachineType\x12 \n\x0c\x64isk_size_gb\x18\x02 \x01(\x05R\ndiskSizeGb\x12!\n\x0coauth_scopes\x18\x03 \x03(\tR\x0boauthScopes\x12\'\n\x0fservice_account\x18\t \x01(\tR\x0eserviceAccount\x12H\n\x08metadata\x18\x04 \x03(\x0b\x32,.chalk.nodepools.v1.NodeConfig.MetadataEntryR\x08metadata\x12\x1d\n\nimage_type\x18\x05 \x01(\tR\timageType\x12\x42\n\x06labels\x18\x06 \x03(\x0b\x32*.chalk.nodepools.v1.NodeConfig.LabelsEntryR\x06labels\x12&\n\x0flocal_ssd_count\x18\x07 \x01(\x05R\rlocalSsdCount\x12\x12\n\x04tags\x18\x08 \x03(\tR\x04tags\x12 \n\x0bpreemptible\x18\n \x01(\x08R\x0bpreemptible\x12\x1b\n\tdisk_type\x18\x0c \x01(\tR\x08\x64iskType\x12(\n\x10min_cpu_platform\x18\r \x01(\tR\x0eminCpuPlatform\x12\x35\n\x06taints\x18\x0f \x03(\x0b\x32\x1d.chalk.nodepools.v1.NodeTaintR\x06taints\x12\x1d\n\nnode_group\x18\x12 \x01(\tR\tnodeGroup\x12)\n\x11\x62oot_disk_kms_key\x18\x17 \x01(\tR\x0e\x62ootDiskKmsKey\x12\x12\n\x04spot\x18  \x01(\x08R\x04spot\x12[\n\x0fresource_labels\x18% \x03(\x0b\x32\x32.chalk.nodepools.v1.NodeConfig.ResourceLabelsEntryR\x0eresourceLabels\x12u\n\x19local_ssd_encryption_mode\x18\x36 \x01(\x0e\x32\x35.chalk.nodepools.v1.NodeConfig.LocalSsdEncryptionModeH\x00R\x16localSsdEncryptionMode\x88\x01\x01\x12\x66\n\x15\x65\x66\x66\x65\x63tive_cgroup_mode\x18\x37 \x01(\x0e\x32\x32.chalk.nodepools.v1.NodeConfig.EffectiveCgroupModeR\x13\x65\x66\x66\x65\x63tiveCgroupMode\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x41\n\x13ResourceLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xae\x01\n\x16LocalSsdEncryptionMode\x12)\n%LOCAL_SSD_ENCRYPTION_MODE_UNSPECIFIED\x10\x00\x12\x31\n-LOCAL_SSD_ENCRYPTION_MODE_STANDARD_ENCRYPTION\x10\x01\x12\x36\n2LOCAL_SSD_ENCRYPTION_MODE_EPHEMERAL_KEY_ENCRYPTION\x10\x02"x\n\x13\x45\x66\x66\x65\x63tiveCgroupMode\x12%\n!EFFECTIVE_CGROUP_MODE_UNSPECIFIED\x10\x00\x12\x1c\n\x18\x45\x46\x46\x45\x43TIVE_CGROUP_MODE_V1\x10\x01\x12\x1c\n\x18\x45\x46\x46\x45\x43TIVE_CGROUP_MODE_V2\x10\x02\x42\x1c\n\x1a_local_ssd_encryption_mode"\xff\x03\n\x11\x42lueGreenSettings\x12u\n\x17standard_rollout_policy\x18\x01 \x01(\x0b\x32;.chalk.nodepools.v1.BlueGreenSettings.StandardRolloutPolicyH\x00R\x15standardRolloutPolicy\x12U\n\x17node_pool_soak_duration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x01R\x14nodePoolSoakDuration\x88\x01\x01\x1a\xed\x01\n\x15StandardRolloutPolicy\x12+\n\x10\x62\x61tch_percentage\x18\x01 \x01(\x02H\x00R\x0f\x62\x61tchPercentage\x12*\n\x10\x62\x61tch_node_count\x18\x02 \x01(\x05H\x00R\x0e\x62\x61tchNodeCount\x12N\n\x13\x62\x61tch_soak_duration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationH\x01R\x11\x62\x61tchSoakDuration\x88\x01\x01\x42\x13\n\x11update_batch_sizeB\x16\n\x14_batch_soak_durationB\x10\n\x0erollout_policyB\x1a\n\x18_node_pool_soak_duration"\xb6\x14\n\x0bGKENodePool\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x36\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x1e.chalk.nodepools.v1.NodeConfigR\x06\x63onfig\x12,\n\x12initial_node_count\x18\x03 \x01(\x05R\x10initialNodeCount\x12\x1c\n\tlocations\x18\r \x03(\tR\tlocations\x12\x1b\n\tself_link\x18\x64 \x01(\tR\x08selfLink\x12\x18\n\x07version\x18\x65 \x01(\tR\x07version\x12.\n\x13instance_group_urls\x18\x66 \x03(\tR\x11instanceGroupUrls\x12>\n\x06status\x18g \x01(\x0e\x32&.chalk.nodepools.v1.GKENodePool.StatusR\x06status\x12)\n\x0estatus_message\x18h \x01(\tB\x02\x18\x01R\rstatusMessage\x12I\n\x0b\x61utoscaling\x18\x04 \x01(\x0b\x32\'.chalk.nodepools.v1.NodePoolAutoscalingR\x0b\x61utoscaling\x12\x42\n\nmanagement\x18\x05 \x01(\x0b\x32".chalk.nodepools.v1.NodeManagementR\nmanagement\x12U\n\x13max_pods_constraint\x18\x06 \x01(\x0b\x32%.chalk.nodepools.v1.MaxPodsConstraintR\x11maxPodsConstraint\x12\x43\n\nconditions\x18i \x03(\x0b\x32#.chalk.nodepools.v1.StatusConditionR\nconditions\x12+\n\x12pod_ipv4_cidr_size\x18\x07 \x01(\x05R\x0fpodIpv4CidrSize\x12Z\n\x10upgrade_settings\x18k \x01(\x0b\x32/.chalk.nodepools.v1.GKENodePool.UpgradeSettingsR\x0fupgradeSettings\x12Z\n\x10placement_policy\x18l \x01(\x0b\x32/.chalk.nodepools.v1.GKENodePool.PlacementPolicyR\x0fplacementPolicy\x12K\n\x0bupdate_info\x18m \x01(\x0b\x32*.chalk.nodepools.v1.GKENodePool.UpdateInfoR\nupdateInfo\x12\x12\n\x04\x65tag\x18n \x01(\tR\x04\x65tag\x12\x63\n\x13queued_provisioning\x18p \x01(\x0b\x32\x32.chalk.nodepools.v1.GKENodePool.QueuedProvisioningR\x12queuedProvisioning\x12\x64\n\x18\x62\x65st_effort_provisioning\x18q \x01(\x0b\x32*.chalk.nodepools.v1.BestEffortProvisioningR\x16\x62\x65stEffortProvisioning\x1a\xa5\x02\n\x0fUpgradeSettings\x12\x1b\n\tmax_surge\x18\x01 \x01(\x05R\x08maxSurge\x12\'\n\x0fmax_unavailable\x18\x02 \x01(\x05R\x0emaxUnavailable\x12K\n\x08strategy\x18\x03 \x01(\x0e\x32*.chalk.nodepools.v1.NodePoolUpdateStrategyH\x00R\x08strategy\x88\x01\x01\x12Z\n\x13\x62lue_green_settings\x18\x04 \x01(\x0b\x32%.chalk.nodepools.v1.BlueGreenSettingsH\x01R\x11\x62lueGreenSettings\x88\x01\x01\x42\x0b\n\t_strategyB\x16\n\x14_blue_green_settings\x1a\xa8\x05\n\nUpdateInfo\x12`\n\x0f\x62lue_green_info\x18\x01 \x01(\x0b\x32\x38.chalk.nodepools.v1.GKENodePool.UpdateInfo.BlueGreenInfoR\rblueGreenInfo\x1a\xb7\x04\n\rBlueGreenInfo\x12T\n\x05phase\x18\x01 \x01(\x0e\x32>.chalk.nodepools.v1.GKENodePool.UpdateInfo.BlueGreenInfo.PhaseR\x05phase\x12\x37\n\x18\x62lue_instance_group_urls\x18\x02 \x03(\tR\x15\x62lueInstanceGroupUrls\x12\x39\n\x19green_instance_group_urls\x18\x03 \x03(\tR\x16greenInstanceGroupUrls\x12@\n\x1d\x62lue_pool_deletion_start_time\x18\x04 \x01(\tR\x19\x62luePoolDeletionStartTime\x12,\n\x12green_pool_version\x18\x05 \x01(\tR\x10greenPoolVersion"\xeb\x01\n\x05Phase\x12\x15\n\x11PHASE_UNSPECIFIED\x10\x00\x12\x18\n\x14PHASE_UPDATE_STARTED\x10\x01\x12\x1d\n\x19PHASE_CREATING_GREEN_POOL\x10\x02\x12\x1d\n\x19PHASE_CORDONING_BLUE_POOL\x10\x03\x12\x1c\n\x18PHASE_DRAINING_BLUE_POOL\x10\x04\x12\x1b\n\x17PHASE_NODE_POOL_SOAKING\x10\x05\x12\x1c\n\x18PHASE_DELETING_BLUE_POOL\x10\x06\x12\x1a\n\x16PHASE_ROLLBACK_STARTED\x10\x07\x1a\xcf\x01\n\x0fPlacementPolicy\x12H\n\x04type\x18\x01 \x01(\x0e\x32\x34.chalk.nodepools.v1.GKENodePool.PlacementPolicy.TypeR\x04type\x12!\n\x0ctpu_topology\x18\x02 \x01(\tR\x0btpuTopology\x12\x1f\n\x0bpolicy_name\x18\x03 \x01(\tR\npolicyName".\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x10\n\x0cTYPE_COMPACT\x10\x01\x1a.\n\x12QueuedProvisioning\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled"\xab\x01\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x17\n\x13STATUS_PROVISIONING\x10\x01\x12\x12\n\x0eSTATUS_RUNNING\x10\x02\x12\x1d\n\x19STATUS_RUNNING_WITH_ERROR\x10\x03\x12\x16\n\x12STATUS_RECONCILING\x10\x04\x12\x13\n\x0fSTATUS_STOPPING\x10\x05\x12\x10\n\x0cSTATUS_ERROR\x10\x06*\x92\x01\n\x16NodePoolUpdateStrategy\x12)\n%NODE_POOL_UPDATE_STRATEGY_UNSPECIFIED\x10\x00\x12(\n$NODE_POOL_UPDATE_STRATEGY_BLUE_GREEN\x10\x02\x12#\n\x1fNODE_POOL_UPDATE_STRATEGY_SURGE\x10\x03\x42\xa6\x01\n\x16\x63om.chalk.nodepools.v1B\x08GkeProtoP\x01Z\x18nodepools/v1;nodepoolsv1\xa2\x02\x03\x43NX\xaa\x02\x12\x43halk.Nodepools.V1\xca\x02\x12\x43halk\\Nodepools\\V1\xe2\x02\x1e\x43halk\\Nodepools\\V1\\GPBMetadata\xea\x02\x14\x43halk::Nodepools::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.nodepools.v1.gke_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\026com.chalk.nodepools.v1B\010GkeProtoP\001Z\030nodepools/v1;nodepoolsv1\242\002\003CNX\252\002\022Chalk.Nodepools.V1\312\002\022Chalk\\Nodepools\\V1\342\002\036Chalk\\Nodepools\\V1\\GPBMetadata\352\002\024Chalk::Nodepools::V1"
    _globals["_STATUSCONDITION"].fields_by_name["code"]._options = None
    _globals["_STATUSCONDITION"].fields_by_name["code"]._serialized_options = b"\030\001"
    _globals["_NODECONFIG_METADATAENTRY"]._options = None
    _globals["_NODECONFIG_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_NODECONFIG_LABELSENTRY"]._options = None
    _globals["_NODECONFIG_LABELSENTRY"]._serialized_options = b"8\001"
    _globals["_NODECONFIG_RESOURCELABELSENTRY"]._options = None
    _globals["_NODECONFIG_RESOURCELABELSENTRY"]._serialized_options = b"8\001"
    _globals["_GKENODEPOOL"].fields_by_name["status_message"]._options = None
    _globals["_GKENODEPOOL"].fields_by_name["status_message"]._serialized_options = b"\030\001"
    _globals["_NODEPOOLUPDATESTRATEGY"]._serialized_start = 6277
    _globals["_NODEPOOLUPDATESTRATEGY"]._serialized_end = 6423
    _globals["_STATUSCONDITION"]._serialized_start = 108
    _globals["_STATUSCONDITION"]._serialized_end = 524
    _globals["_STATUSCONDITION_CODE"]._serialized_start = 277
    _globals["_STATUSCONDITION_CODE"]._serialized_end = 524
    _globals["_NODEPOOLAUTOSCALING"]._serialized_start = 527
    _globals["_NODEPOOLAUTOSCALING"]._serialized_end = 993
    _globals["_NODEPOOLAUTOSCALING_LOCATIONPOLICY"]._serialized_start = 889
    _globals["_NODEPOOLAUTOSCALING_LOCATIONPOLICY"]._serialized_end = 993
    _globals["_NODETAINT"]._serialized_start = 996
    _globals["_NODETAINT"]._serialized_end = 1221
    _globals["_NODETAINT_EFFECT"]._serialized_start = 1111
    _globals["_NODETAINT_EFFECT"]._serialized_end = 1221
    _globals["_MAXPODSCONSTRAINT"]._serialized_start = 1223
    _globals["_MAXPODSCONSTRAINT"]._serialized_end = 1285
    _globals["_BESTEFFORTPROVISIONING"]._serialized_start = 1287
    _globals["_BESTEFFORTPROVISIONING"]._serialized_end = 1385
    _globals["_NODEMANAGEMENT"]._serialized_start = 1388
    _globals["_NODEMANAGEMENT"]._serialized_end = 1553
    _globals["_AUTOUPGRADEOPTIONS"]._serialized_start = 1555
    _globals["_AUTOUPGRADEOPTIONS"]._serialized_end = 1664
    _globals["_NODECONFIG"]._serialized_start = 1667
    _globals["_NODECONFIG"]._serialized_end = 3143
    _globals["_NODECONFIG_METADATAENTRY"]._serialized_start = 2629
    _globals["_NODECONFIG_METADATAENTRY"]._serialized_end = 2688
    _globals["_NODECONFIG_LABELSENTRY"]._serialized_start = 2690
    _globals["_NODECONFIG_LABELSENTRY"]._serialized_end = 2747
    _globals["_NODECONFIG_RESOURCELABELSENTRY"]._serialized_start = 2749
    _globals["_NODECONFIG_RESOURCELABELSENTRY"]._serialized_end = 2814
    _globals["_NODECONFIG_LOCALSSDENCRYPTIONMODE"]._serialized_start = 2817
    _globals["_NODECONFIG_LOCALSSDENCRYPTIONMODE"]._serialized_end = 2991
    _globals["_NODECONFIG_EFFECTIVECGROUPMODE"]._serialized_start = 2993
    _globals["_NODECONFIG_EFFECTIVECGROUPMODE"]._serialized_end = 3113
    _globals["_BLUEGREENSETTINGS"]._serialized_start = 3146
    _globals["_BLUEGREENSETTINGS"]._serialized_end = 3657
    _globals["_BLUEGREENSETTINGS_STANDARDROLLOUTPOLICY"]._serialized_start = 3374
    _globals["_BLUEGREENSETTINGS_STANDARDROLLOUTPOLICY"]._serialized_end = 3611
    _globals["_GKENODEPOOL"]._serialized_start = 3660
    _globals["_GKENODEPOOL"]._serialized_end = 6274
    _globals["_GKENODEPOOL_UPGRADESETTINGS"]._serialized_start = 4866
    _globals["_GKENODEPOOL_UPGRADESETTINGS"]._serialized_end = 5159
    _globals["_GKENODEPOOL_UPDATEINFO"]._serialized_start = 5162
    _globals["_GKENODEPOOL_UPDATEINFO"]._serialized_end = 5842
    _globals["_GKENODEPOOL_UPDATEINFO_BLUEGREENINFO"]._serialized_start = 5275
    _globals["_GKENODEPOOL_UPDATEINFO_BLUEGREENINFO"]._serialized_end = 5842
    _globals["_GKENODEPOOL_UPDATEINFO_BLUEGREENINFO_PHASE"]._serialized_start = 5607
    _globals["_GKENODEPOOL_UPDATEINFO_BLUEGREENINFO_PHASE"]._serialized_end = 5842
    _globals["_GKENODEPOOL_PLACEMENTPOLICY"]._serialized_start = 5845
    _globals["_GKENODEPOOL_PLACEMENTPOLICY"]._serialized_end = 6052
    _globals["_GKENODEPOOL_PLACEMENTPOLICY_TYPE"]._serialized_start = 6006
    _globals["_GKENODEPOOL_PLACEMENTPOLICY_TYPE"]._serialized_end = 6052
    _globals["_GKENODEPOOL_QUEUEDPROVISIONING"]._serialized_start = 6054
    _globals["_GKENODEPOOL_QUEUEDPROVISIONING"]._serialized_end = 6100
    _globals["_GKENODEPOOL_STATUS"]._serialized_start = 6103
    _globals["_GKENODEPOOL_STATUS"]._serialized_end = 6274
# @@protoc_insertion_point(module_scope)
