# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v2/feature_values_chart.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.chart.v1 import densetimeserieschart_pb2 as chalk_dot_chart_dot_v1_dot_densetimeserieschart__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n*chalk/engine/v2/feature_values_chart.proto\x12\x0f\x63halk.engine.v2\x1a)chalk/chart/v1/densetimeserieschart.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"F\n$FeatureValuePercentileWindowFunction\x12\x1e\n\npercentile\x18\x01 \x01(\x01R\npercentile"\xdd\x02\n\x12\x46\x65\x61tureValueSeries\x12\x1f\n\x0b\x66\x65\x61ture_fqn\x18\x01 \x01(\tR\nfeatureFqn\x12&\n\x0cseries_title\x18\x02 \x01(\tH\x01R\x0bseriesTitle\x88\x01\x01\x12\x63\n\x14\x62\x61se_window_function\x18\x03 \x01(\x0e\x32/.chalk.engine.v2.FeatureValueBaseWindowFunctionH\x00R\x12\x62\x61seWindowFunction\x12u\n\x1apercentile_window_function\x18\x04 \x01(\x0b\x32\x35.chalk.engine.v2.FeatureValuePercentileWindowFunctionH\x00R\x18percentileWindowFunctionB\x11\n\x0fwindow_functionB\x0f\n\r_series_title"\x89\x05\n&GetFeatureValuesTimeSeriesChartRequest\x12\x19\n\x05title\x18\x01 \x01(\tH\x00R\x05title\x88\x01\x01\x12;\n\x06series\x18\x02 \x03(\x0b\x32#.chalk.engine.v2.FeatureValueSeriesR\x06series\x12>\n\rwindow_period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cwindowPeriod\x12V\n\x19start_timestamp_inclusive\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x17startTimestampInclusive\x12W\n\x17\x65nd_timestamp_exclusive\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01R\x15\x65ndTimestampExclusive\x88\x01\x01\x12M\n\x0btime_series\x18\x06 \x01(\x0e\x32\'.chalk.engine.v2.FeatureValueTimeSeriesH\x02R\ntimeSeries\x88\x01\x01\x12?\n\x08group_by\x18\x07 \x03(\x0e\x32$.chalk.engine.v2.FeatureValueGroupByR\x07groupBy\x12\x36\n\x15group_by_series_limit\x18\x08 \x01(\x05H\x03R\x12groupBySeriesLimit\x88\x01\x01\x42\x08\n\x06_titleB\x1a\n\x18_end_timestamp_exclusiveB\x0e\n\x0c_time_seriesB\x18\n\x16_group_by_series_limit"e\n\'GetFeatureValuesTimeSeriesChartResponse\x12:\n\x05\x63hart\x18\x01 \x01(\x0b\x32$.chalk.chart.v1.DenseTimeSeriesChartR\x05\x63hart*\xc6\x03\n\x1e\x46\x65\x61tureValueBaseWindowFunction\x12\x32\n.FEATURE_VALUE_BASE_WINDOW_FUNCTION_UNSPECIFIED\x10\x00\x12\x34\n0FEATURE_VALUE_BASE_WINDOW_FUNCTION_UNIQUE_VALUES\x10\x01\x12\x39\n5FEATURE_VALUE_BASE_WINDOW_FUNCTION_TOTAL_OBSERVATIONS\x10\x02\x12\x36\n2FEATURE_VALUE_BASE_WINDOW_FUNCTION_NULL_PERCENTAGE\x10\x03\x12\x30\n,FEATURE_VALUE_BASE_WINDOW_FUNCTION_MAX_VALUE\x10\x04\x12\x30\n,FEATURE_VALUE_BASE_WINDOW_FUNCTION_MIN_VALUE\x10\x05\x12.\n*FEATURE_VALUE_BASE_WINDOW_FUNCTION_AVERAGE\x10\x06\x12\x33\n/FEATURE_VALUE_BASE_WINDOW_FUNCTION_UNIQUE_PKEYS\x10\x07*\xe2\x02\n\x13\x46\x65\x61tureValueGroupBy\x12&\n"FEATURE_VALUE_GROUP_BY_UNSPECIFIED\x10\x00\x12,\n(FEATURE_VALUE_GROUP_BY_CATEGORICAL_VALUE\x10\x01\x12&\n"FEATURE_VALUE_GROUP_BY_PRIMARY_KEY\x10\x02\x12(\n$FEATURE_VALUE_GROUP_BY_DEPLOYMENT_ID\x10\x03\x12#\n\x1f\x46\x45\x41TURE_VALUE_GROUP_BY_RESOLVER\x10\x04\x12)\n%FEATURE_VALUE_GROUP_BY_OPERATION_KIND\x10\x05\x12\'\n#FEATURE_VALUE_GROUP_BY_OPERATION_ID\x10\x06\x12*\n&FEATURE_VALUE_GROUP_BY_FEATURE_VERSION\x10\x07*\x99\x01\n\x16\x46\x65\x61tureValueTimeSeries\x12)\n%FEATURE_VALUE_TIME_SERIES_UNSPECIFIED\x10\x00\x12)\n%FEATURE_VALUE_TIME_SERIES_INSERTED_AT\x10\x01\x12)\n%FEATURE_VALUE_TIME_SERIES_OBSERVED_AT\x10\x02\x42\x8c\x01\n\x13\x63om.chalk.engine.v2B\x17\x46\x65\x61tureValuesChartProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V2\xca\x02\x0f\x43halk\\Engine\\V2\xe2\x02\x1b\x43halk\\Engine\\V2\\GPBMetadata\xea\x02\x11\x43halk::Engine::V2b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v2.feature_values_chart_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v2B\027FeatureValuesChartProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V2\312\002\017Chalk\\Engine\\V2\342\002\033Chalk\\Engine\\V2\\GPBMetadata\352\002\021Chalk::Engine::V2"
    _globals["_FEATUREVALUEBASEWINDOWFUNCTION"]._serialized_start = 1351
    _globals["_FEATUREVALUEBASEWINDOWFUNCTION"]._serialized_end = 1805
    _globals["_FEATUREVALUEGROUPBY"]._serialized_start = 1808
    _globals["_FEATUREVALUEGROUPBY"]._serialized_end = 2162
    _globals["_FEATUREVALUETIMESERIES"]._serialized_start = 2165
    _globals["_FEATUREVALUETIMESERIES"]._serialized_end = 2318
    _globals["_FEATUREVALUEPERCENTILEWINDOWFUNCTION"]._serialized_start = 171
    _globals["_FEATUREVALUEPERCENTILEWINDOWFUNCTION"]._serialized_end = 241
    _globals["_FEATUREVALUESERIES"]._serialized_start = 244
    _globals["_FEATUREVALUESERIES"]._serialized_end = 593
    _globals["_GETFEATUREVALUESTIMESERIESCHARTREQUEST"]._serialized_start = 596
    _globals["_GETFEATUREVALUESTIMESERIESCHARTREQUEST"]._serialized_end = 1245
    _globals["_GETFEATUREVALUESTIMESERIESCHARTRESPONSE"]._serialized_start = 1247
    _globals["_GETFEATUREVALUESTIMESERIESCHARTRESPONSE"]._serialized_end = 1348
# @@protoc_insertion_point(module_scope)
