"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.scheduler_pb2 import (
    ManualTriggerScheduledQueryRequest,
    ManualTriggerScheduledQueryResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class SchedulerServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    ManualTriggerScheduledQuery: UnaryUnaryMultiCallable[
        ManualTriggerScheduledQueryRequest,
        ManualTriggerScheduledQueryResponse,
    ]

class SchedulerServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def ManualTriggerScheduledQuery(
        self,
        request: ManualTriggerScheduledQueryRequest,
        context: ServicerContext,
    ) -> ManualTriggerScheduledQueryResponse: ...

def add_SchedulerServiceServicer_to_server(servicer: SchedulerServiceServicer, server: Server) -> None: ...
