# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v1/bloom_filter.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n"chalk/engine/v1/bloom_filter.proto\x12\x0f\x63halk.engine.v1"\xf2\x01\n\x0b\x42loomFilter\x12 \n\x0b\x65nvironment\x18\x01 \x01(\tR\x0b\x65nvironment\x12\x1c\n\tnamespace\x18\x02 \x01(\tR\tnamespace\x12\x1f\n\x0bnum_entries\x18\x03 \x01(\x04R\nnumEntries\x12\x30\n\x14num_expected_entries\x18\x04 \x01(\x04R\x12numExpectedEntries\x12\x1d\n\nnum_hashes\x18\x05 \x01(\x04R\tnumHashes\x12\x1d\n\nsize_bytes\x18\x06 \x01(\x04R\tsizeBytes\x12\x12\n\x04\x64\x61ta\x18\x07 \x01(\x0cR\x04\x64\x61taB\x85\x01\n\x13\x63om.chalk.engine.v1B\x10\x42loomFilterProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V1\xca\x02\x0f\x43halk\\Engine\\V1\xe2\x02\x1b\x43halk\\Engine\\V1\\GPBMetadata\xea\x02\x11\x43halk::Engine::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v1.bloom_filter_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v1B\020BloomFilterProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V1\312\002\017Chalk\\Engine\\V1\342\002\033Chalk\\Engine\\V1\\GPBMetadata\352\002\021Chalk::Engine::V1"
    _globals["_BLOOMFILTER"]._serialized_start = 56
    _globals["_BLOOMFILTER"]._serialized_end = 298
# @@protoc_insertion_point(module_scope)
