# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/common/v1/operation_kind.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b"\n$chalk/common/v1/operation_kind.proto\x12\x0f\x63halk.common.v1*\x94\x02\n\rOperationKind\x12\x1e\n\x1aOPERATION_KIND_UNSPECIFIED\x10\x00\x12\x1f\n\x1bOPERATION_KIND_ONLINE_QUERY\x10\x01\x12\x1c\n\x18OPERATION_KIND_MIGRATION\x10\x02\x12\x17\n\x13OPERATION_KIND_CRON\x10\x03\x12\x1c\n\x18OPERATION_KIND_STREAMING\x10\x04\x12$\n OPERATION_KIND_MIGRATION_SAMPLER\x10\x05\x12%\n!OPERATION_KIND_WINDOWED_STREAMING\x10\x06\x12 \n\x1cOPERATION_KIND_OFFLINE_QUERY\x10\x07\x42\x87\x01\n\x13\x63om.chalk.common.v1B\x12OperationKindProtoP\x01\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43halk.Common.V1\xca\x02\x0f\x43halk\\Common\\V1\xe2\x02\x1b\x43halk\\Common\\V1\\GPBMetadata\xea\x02\x11\x43halk::Common::V1b\x06proto3"
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.common.v1.operation_kind_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.common.v1B\022OperationKindProtoP\001\242\002\003CCX\252\002\017Chalk.Common.V1\312\002\017Chalk\\Common\\V1\342\002\033Chalk\\Common\\V1\\GPBMetadata\352\002\021Chalk::Common::V1"
    _globals["_OPERATIONKIND"]._serialized_start = 58
    _globals["_OPERATIONKIND"]._serialized_end = 334
# @@protoc_insertion_point(module_scope)
