"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.flag_pb2 import (
    GetFeatureFlagsRequest,
    GetFeatureFlagsResponse,
    SetFeatureFlagRequest,
    SetFeatureFlagResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class FeatureFlagServiceStub:
    def __init__(self, channel: Channel) -> None: ...
    GetFeatureFlags: UnaryUnaryMultiCallable[
        GetFeatureFlagsRequest,
        GetFeatureFlagsResponse,
    ]
    SetFeatureFlag: UnaryUnaryMultiCallable[
        SetFeatureFlagRequest,
        SetFeatureFlagResponse,
    ]

class FeatureFlagServiceServicer(metaclass=ABCMeta):
    @abstractmethod
    def GetFeatureFlags(
        self,
        request: GetFeatureFlagsRequest,
        context: ServicerContext,
    ) -> GetFeatureFlagsResponse: ...
    @abstractmethod
    def SetFeatureFlag(
        self,
        request: SetFeatureFlagRequest,
        context: ServicerContext,
    ) -> SetFeatureFlagResponse: ...

def add_FeatureFlagServiceServicer_to_server(servicer: FeatureFlagServiceServicer, server: Server) -> None: ...
