from chalk._gen.chalk.auth.v1 import permissions_pb2 as _permissions_pb2
from chalk._gen.chalk.common.v1 import feature_values_pb2 as _feature_values_pb2
from chalk._gen.chalk.common.v1 import query_log_pb2 as _query_log_pb2
from chalk._gen.chalk.common.v1 import query_values_pb2 as _query_values_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class OfflineStoreServiceDummy(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...
